close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

run startdsgeprioeuro;
!ps -A;
rand('state',sum(100*clock))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para, ...
        varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,JustMode, ...
        stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix] = varspecb(chdata);

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f',[mspec,dataset,mprior]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,nvar,nlags,chdata,mspec);

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);
        



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% doing rolling sample estimation?
ROLL = 1
SAVEALL = 0
DOFORECAST = 1
if ROLL 
	DOFORECAST = 1;
	SAVEALL = 0;
end
start_period = stime;                               % minimum # quarters to compute VAR estimates 


%mkdir(spath,'/2/');
spath = strcat(spath,'/1/');
%start_period = nobsall-qahead+1-3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% CHAnGE

trial_period = nobsall-start_period-qahead+1;      % # quarters to compute RMSE


%%  Configure the Metropolis Algorithm
cc0     = 0.002;
cc      = 0.15;

 
%% set output file
runn = strcat('outcompres',lmodel,lprior,dataset,st);
outfile = strcat(spath,runn,'.tex');		% output file name 
fid = fopen(outfile,'w');		% output file (set to write)
fprintf(fid,'\n \\documentclass[12pt]{article}');
fprintf(fid,'\n \\begin{document}');
fprintf(fid,'\n \\setlength{\\tabcolsep}{.7mm}');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      load results: Lambda sequence
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

date_q = (1:1:qahead)';

Y = zeros(qahead,nvar+1,length(lambdaseq));
Fhall = zeros(qahead,nvar,trial_period,length(lambdaseq));
Mhall = zeros(trial_period,length(lambdaseq));

fixparam = fixparamseq          
L = 0;
for lambdas = lambdaseq     
            
        L = L+1;        
        

	if ROLL        
		Iseq = start_period+1:start_period+trial_period;
	Iseq = start_period+trial_period-9:start_period+trial_period;

		fprintf(1,'\n beginning: %2.4f ; end:  %2.4f \n',[ti(Iseq(1)),ti(Iseq(end))]);
		fprintf(fid,'\n beginning: %2.4f ; end:  %2.4f \n',[ti(Iseq(1)),ti(Iseq(end))]);
	else

		disp('set ROLL = 1');
		return;

    	end
        for I = Iseq;

	       	load(strcat(spath,'Fh',lmodel,lprior,dataset,st,num2str(fixparam),num2str(JustMode),num2str(100*lambdas),num2str(100*ti(I))),'Fh');
		if ndims(Fh) == 3
			jfind1 = find(squeeze(Fh(1,1,:)) == 0);
			jfind2 = find(squeeze(Fh(qahead,nvar,:)) == 0);
			if isempty(jfind1) & isempty(jfind1) %% we are at the end
				Fh = squeeze(Fh(:,:,end))
			else
				if jfind1(1) == jfind2(1)
					jfind = jfind1(1)-1;
				else 
					disp('problem'); return;
				end
				Fh = squeeze(Fh(:,:,jfind));
			end
		end
		Fhall(:,:,I-start_period,L) = Fh;

		if lambdas ~= 0
	           	load(strcat(spath,'Mh',lmodel,lprior,dataset,st,num2str(fixparam),num2str(JustMode),num2str(100*lambdas),num2str(100*ti(I))),'pint0');
			Mhall(I-start_period,L) = pint0;
		end
	end %% end roll

        FhL = squeeze(Fhall(:,:,:,L));

	Y(:,1:nvar,L) = sqrt(mean(FhL(:,:,:).^2,3));
            
	for j = 1:qahead
        	fj = squeeze(FhL(j,:,:))';
	        matfj = fj'*fj/size(fj,1);
        	Y(j,nvar+1,L) = -log(det(matfj))/(2*nvar);
	end

 end %% end lambdaseq
 
%% compute forecasts for lambda hat

%% for zero, put -Inf
if any(lambdaseq == 0)
        Mhall(:,1) = -inf;
        L = 1;
        lambdaseqmh = lambdaseq;
        lambdaseqmh(find(lambdaseqmh == 0)) = [];
else
        L = 0;
        lambdaseqmh = lambdaseq;
end
Mhall = real(Mhall);


[(1:1:size(Mhall,1))',Mhall]
 pause
    
[mhval,mhI] = max(Mhall,[],2);
	
lambdahat = [];
Fhlambdahat = zeros(qahead,nvar,trial_period);
	
for i = 1:length(mhI)
        lambdahat = [lambdahat;lambdaseq(mhI(i))];
        Fhlambdahat(:,:,i) = Fhall(:,:,i,mhI(i));
end
save(strcat(spath,'lambdahat.txt'),'lambdahat','-ascii');
Ylambdahat = zeros(qahead,nvar+1);
Ylambdahat(:,1:nvar) = sqrt(mean(Fhlambdahat(:,:,:).^2,3));
            
for j = 1:qahead
        fj = squeeze(Fhlambdahat(j,:,:))';
        matfj = fj'*fj/size(fj,1);
        Ylambdahat(j,nvar+1) = -log(det(matfj))/(2*nvar);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      display results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      Lambda plot
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Lambdaseq =  [1:1:length(lambdaseq)]';
nselex = (3:2:(length(lambdaseq)-1));
lambdaticks = num2str(flipdim(lambdaseq(Lambdaseq)',1))
lambdaticks(nselex,:) = ' ';

sele = [1 2:2:16];
date_q = (1:1:qahead)';
datt = [1:1:length(date_q(sele,:))]';
datte = [date_q(sele,:)];


for i = 1:nvar
    
    figure(1);
    subplot(ceil((nvar+1)/2),2,i);
    Q = squeeze(Y(sele,i,:));
    Q1 = repmat(Q(:,1),[1 length(lambdaseq)]);     
    Q = flipdim(100*(1-Q./Q1),2);
    if i == 1
        %Q = flipdim(Q,1);        
    end
    
    mesh(Lambdaseq,datt,Q,'FaceColor','none','LineWidth',1,'LineStyle','-','EdgeColor',[0 0 0]);
    Q1 = zeros(2,2);      
    hold on;
    mesh([Lambdaseq(1);Lambdaseq(end)],[datt(1);datt(end)],Q1,'FaceColor','none','LineWidth',1,'LineStyle',':','EdgeColor',[0 0 0]);
    hold off;
    ylabel('quarters ahead');
    set(gca,'YTick',datt);
    set(gca,'YTickLabel',num2str(datte));
    if i == 1
       set(gca,'ZLim',[-10 30])
    elseif i == 2
       set(gca,'ZLim',[0 20])
    elseif i == 3
       set(gca,'ZLim',[0 10])
    end        
    xlabel('\lambda');
    set(gca,'XTick',Lambdaseq);
    set(gca,'XTickLabel',lambdaticks);  
    zlabel('%');
    
    title(varnames{i});
    
end
figure(1);
subplot(ceil((nvar+1)/2),2,nvar+1);
Q = squeeze(Y(sele,nvar+1,:));
Q1 = repmat(Q(:,1),[1 length(lambdaseq)]);     
Q = flipdim(100*(Q-Q1),2);
mesh(Lambdaseq,datt,Q,'FaceColor','none','LineWidth',1,'LineStyle','-','EdgeColor',[0 0 0]);
Q1 = zeros(2,2);      
hold on;
mesh([Lambdaseq(1);Lambdaseq(end)],[datt(1);datt(end)],Q1,'FaceColor','none','LineWidth',1,'LineStyle',':','EdgeColor',[0 0 0]);
hold off;
ylabel('quarters ahead');
set(gca,'YTick',datt);
set(gca,'YTickLabel',num2str(datte));
xlabel('\lambda');
set(gca,'XTick',Lambdaseq);
set(gca,'XTickLabel',lambdaticks);  
zlabel('%');
set(gca,'ZLim',[0  20])
title('multivariate statistic');
	
runn = strcat('comprmses',lmodel,lprior,dataset,st,num2str(fixparam),num2str(JustMode),num2str(min(lambdaseq)),num2str(max(lambdaseq)));

set(figure(1),'PaperPosition',[0 0 8 6])
ng=strcat(gpath,runn,'.eps')
print(char(ng),'-deps');
    

    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      Tables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% choose
chlambdaseq = []
chiotaseq = [];

for chla = chlambdaseq 

	chlambda = chla

	Il = find( floor(10*lambdaseq)==floor(10*chlambda) )

		
	tabletitle = strcat('Percentage gain in RMSEs: Unrestricted VAR ');
	fprintf(fid,'\n \\begin{table} \\begin{center}');
	fprintf(fid,'\n \\caption{'); fprintf(fid,tabletitle); fprintf(fid,'}');
		
	fprintf(fid,'\n \\vspace{.2cm} \n \\begin{tabular}{|l|');
	FORMAT = '\n {\\tiny  %2.0f}';
	datt = [datte];
	for i = 1:(nvar+1),
	        fprintf(fid,'c'); 
        	FORMAT = strcat(FORMAT,' & {\\small  %5.3f}'); 
            	if i == nvar+1
                	datt = [datt,100*(squeeze(Y(sele,i,Il))-squeeze(Y(sele,i,1)))];
            	else
                	datt = [datt,-100*(squeeze(Y(sele,i,Il))./squeeze(Y(sele,i,1))-1)];
            	end
            		
            	fprintf(fid,'|'); 
	end
	fprintf(fid,' }\\hline');
	FORMAT = strcat(FORMAT,' \\\\');  
		
	fprintf(fid,'\n  ');
	for i = 1:nvar,
        	fprintf(fid,' & \\multicolumn{%2.0f}{c}',1); 
            	fprintf(fid,'{%9.92s}	 ',varnames{i}); 
	end
	fprintf(fid,' & \\multicolumn{%2.0f}{c|}',1); 
	fprintf(fid,'{multivariate}	\\\\ \\hline '); 
		
	fprintf(fid,'\n  ');
	for i = 1:(nvar+1),
	        fprintf(fid,' & {\\tiny  $ \\lambda $= %2.1f}',chlambda); 
        end
	fprintf(fid,'\\\\ \\hline '); 
		
	fprintf(fid,FORMAT,datt');
	fprintf(fid,' \\hline');
	fprintf(fid,'\n \\end{tabular} \\end{center}');        
	fprintf(fid,'\n \\end{table} ');
		

	tabletitle = strcat('RMSEs ');
	fprintf(fid,'\n \\begin{table} \\begin{center}');
	fprintf(fid,'\n \\caption{'); fprintf(fid,tabletitle); fprintf(fid,'}');
		
	fprintf(fid,'\n \\vspace{.2cm} \n \\begin{tabular}{|l|');
	FORMAT = '\n {\\tiny  %2.0f}';
	datt = [datte];
	for i = 1:(nvar+1),
	        fprintf(fid,'c'); 
        	FORMAT = strcat(FORMAT,' & {\\small  %5.3f}'); 

               	datt = [datt,squeeze(Y(sele,i,Il))];
            	fprintf(fid,'|'); 
	end
	fprintf(fid,' }\\hline');
	FORMAT = strcat(FORMAT,' \\\\');  
		
	fprintf(fid,'\n  ');
	for i = 1:nvar,
        	fprintf(fid,' & \\multicolumn{%2.0f}{c}',1); 
            	fprintf(fid,'{%9.92s}	 ',varnames{i}); 
	end
	fprintf(fid,' & \\multicolumn{%2.0f}{c|}',1); 
	fprintf(fid,'{multivariate}	\\\\ \\hline '); 
		
	fprintf(fid,'\n  ');
	for i = 1:(nvar+1),
	        fprintf(fid,' & {\\tiny  $ \\lambda $= %2.1f}',chlambda); 
        end
	fprintf(fid,'\\\\ \\hline '); 
		
	fprintf(fid,FORMAT,datt');
	fprintf(fid,' \\hline');
	fprintf(fid,'\n \\end{tabular} \\end{center}');        
	fprintf(fid,'\n \\end{table} ');

end

%%% lambdahat & iotahat


	
tabletitle = strcat('Percentage gain in RMSEs');
fprintf(fid,'\n \\begin{table} \\begin{center}');
fprintf(fid,'\n \\caption{'); fprintf(fid,tabletitle); fprintf(fid,'}');
	
fprintf(fid,'\n \\vspace{.2cm} \n \\begin{tabular}{|l|');
FORMAT = '\n {\\tiny  %2.0f}';
datt = [datte];
for i = 1:(nvar+1),
	fprintf(fid,'c'); 
        FORMAT = strcat(FORMAT,' & {\\small  %5.3f}'); 
        if i == nvar+1
            datt = [datt,100*(squeeze(Ylambdahat(sele,i))-squeeze(Y(sele,i,1)))];
        else
            datt = [datt,-100*(squeeze(Ylambdahat(sele,i))./squeeze(Y(sele,i,1))-1)];
        end
        
	
        fprintf(fid,'|'); 
end
fprintf(fid,' }\\hline');
FORMAT = strcat(FORMAT,' \\\\');  
	
fprintf(fid,'\n  ');
for i = 1:nvar,
        fprintf(fid,' & \\multicolumn{%2.0f}{c}',1); 
        fprintf(fid,'{%9.92s}	 ',varnames{i}); 
end
fprintf(fid,' & \\multicolumn{%2.0f}{c|}',1); 
fprintf(fid,'{multivariate}	\\\\ \\hline '); 
	
fprintf(fid,'\n  ');
for i = 1:(nvar+1),
        fprintf(fid,' & {\\tiny  $ \\hat{\\lambda} $}'); 
end
fprintf(fid,'\\\\ \\hline '); 
	
fprintf(fid,FORMAT,datt');
fprintf(fid,' \\hline');
fprintf(fid,'\n \\end{tabular} \\end{center}');        
fprintf(fid,'\n \\end{table} ');
	
	
tabletitle = strcat('RMSEs: DSGE VAR level');
fprintf(fid,'\n \\begin{table} \\begin{center}');
fprintf(fid,'\n \\caption{'); fprintf(fid,tabletitle); fprintf(fid,'}');
	
fprintf(fid,'\n \\vspace{.2cm} \n \\begin{tabular}{|l|');
FORMAT = '\n {\\tiny  %2.0f}';
datt = [datte];
for i = 1:(nvar+1),
        fprintf(fid,'c'); 
        FORMAT = strcat(FORMAT,' & {\\small  %5.3f}'); 
        datt = [datt,squeeze(Ylambdahat(sele,i))];

        fprintf(fid,'|'); 
end
fprintf(fid,' }\\hline');
FORMAT = strcat(FORMAT,' \\\\');  
	
fprintf(fid,'\n  ');
for i = 1:nvar,
        fprintf(fid,' & \\multicolumn{%2.0f}{c}',1); 
        fprintf(fid,'{%9.92s}	 ',varnames{i}); 
end
fprintf(fid,' & \\multicolumn{%2.0f}{c}',1); 
fprintf(fid,'{multivariate}	\\\\ \\hline '); 
	
fprintf(fid,'\n  ');
for i = 1:(nvar+1),
        fprintf(fid,' & {\\tiny  $ \\hat{\\lambda} $}'); 
end
fprintf(fid,'\\\\ \\hline '); 
	
fprintf(fid,FORMAT,datt');
fprintf(fid,' \\hline');
fprintf(fid,'\n \\end{tabular} \\end{center}');        
fprintf(fid,'\n \\end{table} ');


fprintf(fid,'\n \\end{document}');
fclose('all');		% close output file

disp('press any key to clear all plots');
pause;
close all;

