function pyt = dsgelh2(para,YY,nlags,nvar,mspec,npara,coint);
%% This function evaluates the likelihood fcn associated with the DSGE model

%% Solve the DSGE model
[TTT,RRR,valid] = dsgesolv(para,mspec);

%% Create State Space representation
%% solution to DSGE model - delivers transition equation for the state variables  S_t
%% transition equation: S_t = TC+TTT S_{t-1} +RRR eps_t, where var(eps_t) = QQ
%% define the measurement equation: X_t = ZZ S_t +D+u_t
%% where var(u_t) = HH, cov(eps_t,u_t) = VV

[ZZ,DD,QQ,EE,MM,retcode] = measur(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint);

if retcode == 0
    % invalid parameterization
    pyt = -1E10;
    return;
    
end;

HH = EE+MM*QQ*MM';
VV = QQ*MM';

%% We can now define the initial mean and variance for the state vector
At = zeros(size(TTT,1),1);
Pt = dlyap(TTT,RRR*QQ*RRR');

%% compute likelihood with Kalman filter

loglh = 0;

for t = 1:size(YY,1)

      At1 = At;
      Pt1 = Pt;
      
      % Forecasting
      alphahat = TTT*At1;
      Phat     = TTT*Pt1*TTT' + RRR*QQ*RRR';
      yhat     = ZZ*alphahat + DD;
      nu       = YY(t,:) - yhat';

   
      Ft   = ZZ*Phat*ZZ' + HH + ZZ*RRR*VV + (ZZ*RRR*VV)';
      Ft   = 0.5*(Ft + Ft');

   
      % Likelihood contribution
      if t > 0
         loglh = loglh -0.5*size(YY,2)*log(2*pi) - 0.5*log(det(Ft)) - 0.5*nu*inv(Ft)*nu';
      end
 
      % Updating
      At = alphahat + (Phat*ZZ' + RRR*VV)*inv(Ft)*nu';
      Pt = Phat - (Phat*ZZ'+RRR*VV)*inv(Ft)*(Phat*ZZ'+RRR*VV)';


end

pyt = loglh;


