function [yrmse,ystdrmse,DD] = dsgermse(para,YY,nobs,nlags,nvar,mspec,npara,coint);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% step 1: solution to DSGE model - delivers transition equation for the state variables  S_t
%% transition equation: S_t = TC+TTT S_{t-1} +RRR eps_t, where var(eps_t) = QQ
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[TTT,RRR,valid] = dsgesolv(para,mspec);


if valid < 1;
    pyt = -1E10;
    return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% step 2: define the measurement equation: X_t = ZZ S_t +D+u_t
%% where u_t = eta_t+MM* eps_t with var(eta_t) = EE
%% where var(u_t) = HH = EE+MM QQ MM', cov(eps_t,u_t) = VV = QQ*MM'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[ZZ,DD,QQ,EE,MM,retcode] = measur(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint);

if retcode == 0
    % invalid parameterization
    pyt = -1E10;
    return;
    
end;

nshocks = size(RRR,2);
nstate = length(TTT);

HH = EE+MM*QQ*MM';
VV = QQ*MM';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% step 3: compute log-likelihood using Kalman filter - written by Iskander
%%         note that Iskander's program assumes a transition equation written as:
%%         S_t = TTT S_{t-1} +eps2_t, where eps2_t = RRReps_t 
%%         therefore redefine QQ2 = var(eps2_t) = RRR*QQ*RRR'
%%         and  VV2 = cov(eps2_t,u_u) = RRR*VV
%%         define VVall as the joint variance of the two shocks VVall = var([eps2_t;u_t])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

VVall = [[RRR*QQ*RRR',RRR*VV];[VV'*RRR',HH]];

%% Define the initial mean and variance for the state vector
At = zeros(nstate,1);
Pt = dlyap(TTT,RRR*QQ*RRR');

[pyt_,yprederror,ystdprederror,yrmse,ystdrmse] = kalcvf3(YY',1,zeros(nstate,1),TTT,DD,ZZ,VVall,At,Pt); 

if any(mspec == [1,2,4,5])
        varnames = {'Y';'C';'I';'H';'W';'inflation';'M2';'R'};
elseif any(mspec == [6,7])
        varnames = {'Y';'C';'I';'H';'inflation';'R'};
elseif mspec == 3
    varnames = {'Y';'inflation';'R'};
end


titlenames = varnames;

tprederror = (1:1:size(yprederror,2))';

for ivar = 1: nvar
    figure(900);
    subplot(3,3,ivar);                  
    P = plot(tprederror,yprederror(ivar,:)','-');
    set(P,'LineWidth',1);              
    hold off;
    title(titlenames(ivar));
    set(gca,'XLim',[min(tprederror),max(tprederror)]);
    set(gca,'XTick',tprederror([1 25:25:end]));
    set(gca,'XTickLabel',num2str(tprederror([1 25:25:end])));                                           
end   

for ivar = 1: nvar
    figure(901);
    subplot(3,3,ivar);                  
    P = plot(tprederror,ystdprederror(ivar,:)','-');
    set(P,'LineWidth',1);              
    hold off;
    title(titlenames(ivar));
    set(gca,'XLim',[min(tprederror),max(tprederror)]);
    set(gca,'XTick',tprederror([1 25:25:end]));
    set(gca,'XTickLabel',num2str(tprederror([1 25:25:end])));                                           
end   

for ivar = 1: nvar
    figure(902);
    subplot(3,3,ivar);                  
    P = plot(tprederror,cat(2,(YY(:,ivar)-yprederror(ivar,:)'),YY(:,ivar)),'-');
    set(P,'LineWidth',1);              
    hold off;
    title(titlenames(ivar));
    set(gca,'XLim',[min(tprederror),max(tprederror)]);
    set(gca,'XTick',tprederror([1 25:25:end]));
    set(gca,'XTickLabel',num2str(tprederror([1 25:25:end])));                                           
end   
