%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SOLVING THE DSGE MODELS                 
% Input:    para (DSGE parameters)
%       mspec (model specification)
% Output:   T1,T0,TC, matrices for transition equation
%       S_t = TC + T1 S_{t-1} + T0 eps_t
%       RC: flag for gensys
%
%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [TTT,RRR,valid] = dsgesolv(para,mspec);


valid = 1;
TTT = 0;
RRR = 0;


%% write the canonical system in terms of the states
%% GAM0 S(t) = C+ GAM1  S(t-1) + PSI eps(t)+ PPI eta(t)
%% S(t) = states
%% eps(t) = fundamental shocks
%% eta(t) = expectational shocks


if any(mspec == [101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16])

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% step 1: assign names to the parameters
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %% Parameters (explicit)
    
    [alp,zeta_p,iota_p,del,ups,Bigphi,s2,h,a2,nu_l,nu_m,zeta_w,iota_w,law,rstar,psi1,psi2,rho_r,pistar,...
    gam,Lstar,chi,laf,gstar,Ladj,rho_z,rho_phi,rho_chi,rho_laf,rho_mu,rho_b,rho_g,...
    sig_z,sig_phi,sig_chi,sig_laf,sig_mu,sig_b,sig_g,sig_r,...
    bet,zstar,phi,istokbarst,rkstar,cstar,ystar,istar,kstar,kbarstar,Rstarn,wstar,wadj,mstar] = getpara(para,mspec);

    if (rstar < 1) 
           nstate = 1;  
           disp('rstar < 1');
           %disp(gam)
           disp(bet)
keyboard;
           TTT = zeros(nstate,nstate);
           RRR = zeros(nstate,2);
           valid = - 3;
           return;
    elseif ( cstar <= 0 ) | ( ystar <= 0 ) | (istar <= 0 )
           nstate = 1;  
           disp('cstar or ystar or istar < 0 ');
           TTT = zeros(nstate,nstate);
           RRR = zeros(nstate,2);
           valid = - 3;
           return;
    elseif phi < 0
           nstate = 1;  
               disp('phi < 0');
           TTT = zeros(nstate,nstate);
           RRR = zeros(nstate,2);
           valid = - 3;
           return;
    end;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% step 2: assign names to the columns of GAM0, GAM1 -- state variables
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    states;
    

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% step 3: assign numbers to equations 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    eqs;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% step 3: map parameters into values for matrices GAM0, GAM1, C, PPSI, PPI
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %% total number of states   
    nstate = n_end+n_exo+n_exp;
    %% number of equations
    neq  = nstate;

    if neq ~= (n_eqc+n_exo+n_exp); disp('\n problem: number of equations and number of states do not match! \n'); return; end;
    
    %% define matrices of canonical system
    GAM0 = zeros(neq,neq);
    GAM1 = zeros(neq,neq);
    C = zeros(neq,1);
    PPSI = zeros(neq,nex);
    PPI =  zeros(neq,nend);


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% equilibrium conditions
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    eqcond;
    
end

[T1,TC,T0,M,TZ,TY,gev,RC]=gensys(GAM0,GAM1,C,PPSI,PPI,1);

if (RC(1) ~= 1) | (RC(2) ~= 1);
   TTT = zeros(nstate,nstate);
   RRR = zeros(nstate,2);
   valid = - 3;

end;

TTT =    real( T1 );
RRR =    real( T0 );


if  any(mspec == [101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16])

    %% Augment the state vector by y(t-1)
    numAdd = 1;%5;
    AddTTT = zeros(numAdd,size(TTT,2));
    AddTTT(1,y_t) = 1;
    %AddTTT(2,c_t) = 1;
    %AddTTT(3,i_t) = 1;
    %AddTTT(4,w_t) = 1;
    %AddTTT(5,m_t) = 1;
    TTT = [[TTT,zeros(size(TTT,1),numAdd)];[AddTTT,zeros(numAdd)]];
    RRR = [RRR ; zeros(numAdd,size(RRR,2))];

end   
