% this function performs the inverse of the kxk SYMMETRIC square matrix S, 
% after partitioning S is into 16 nxn blocks, (ie, S11,..S44), the partitioned 
% matrix inverse formula is used. 
% of course k must be a multiple of 4
function F=V4(S,n);

%inverse of the 2x2 SE block
D44=inv(Sp(4,4,S,n));
SE11=inv(Sp(3,3,S,n)-Sp(3,4,S,n)*D44*Sp(4,3,S,n));
SE12=-SE11*Sp(3,4,S,n)*D44;
SE22=D44*(eye(n)-Sp(4,3,S,n)*SE12);


%inverse of the 2x2 NW block
D11=Sp(1,3,S,n)*SE11+Sp(1,4,S,n)*SE12.';
D12=Sp(1,3,S,n)*SE12+Sp(1,4,S,n)*SE22;
D21=Sp(2,3,S,n)*SE11+Sp(2,4,S,n)*SE12.';
D22= Sp(2,3,S,n)*SE12+Sp(2,4,S,n)*SE22;

E11=Sp(1,1,S,n)-(D11*Sp(3,1,S,n)+D12*Sp(4,1,S,n));
E12=Sp(1,2,S,n)-(D11*Sp(3,2,S,n)+D12*Sp(4,2,S,n));
E22=Sp(2,2,S,n)-(D21*Sp(3,2,S,n)+D22*Sp(4,2,S,n));
%alternative
%E11=Sp(1,1,S,n)-(D11*Sp(3,1,S,n)+Sp(1,4,S,n)*SE12.'*Sp(3,1,S,n)+...
%                 Sp(1,3,S,n)*SE12*Sp(4,1,S,n)+Sp(1,4,S,n)*SE22*Sp(4,1,S,n));
%E12=Sp(1,2,S,n)-(Sp(1,3,S,n)*SE11*Sp(3,2,S,n)+Sp(1,4,S,n)*SE12.'*Sp(3,2,S,n)+...
%                 Sp(1,3,S,n)*SE12*Sp(4,2,S,n)+Sp(1,4,S,n)*SE22*Sp(4,2,S,n));
%E22=Sp(2,2,S,n)-(Sp(2,3,S,n)*SE11*Sp(3,2,S,n)+Sp(2,4,S,n)*SE12.'*Sp(3,2,S,n)+...
%                 Sp(2,3,S,n)*SE12*Sp(4,2,S,n)+Sp(2,4,S,n)*SE22*Sp(4,2,S,n));

F11=inv(E11-E12*inv(E22)*E12.');
F22=inv(E22-E12.'*inv(E11)*E12);
F12=-inv(E11)*E12*F22;

%inverse of the 2x2 NE block
F13=-(F11*D11+F12*D21);
F14=-(F11*D12+F12*D22);
F23=-(F12.'*D11+F22*D21);
F24=-(F12.'*D12+F22*D22);

%inverse of the 2x2 SE block
D11=Sp(1,3,S,n).'*F13+Sp(2,3,S,n).'*F23;
D12=Sp(1,3,S,n).'*F14+Sp(2,3,S,n).'*F24;
D21=Sp(1,4,S,n).'*F13+Sp(2,4,S,n).'*F23;
D22=Sp(1,4,S,n).'*F14+Sp(2,4,S,n).'*F24;

F33=SE11-(SE11*D11+SE12*D21);
F34=SE12-(SE11*D12+SE12*D22);
F44=SE22-(SE12.'*D12+SE22*D22);

F=[ [F11,F12,F13,F14];[F12.',F22,F23,F24];...
      [F13.',F23.',F33,F34]; [F14.',F24.',F34.',F44 ] ];
