close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

run startdsswfed;

!hostname;
!ps -C matlab;
rand('state',sum(100*clock))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para, ...
 varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode, ...
 stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix,para_aux,mspec_ref,nobsdrat,lambdac] = varspec(chdata);

%[fpath,spath,gpath] = pathspecmh(num2str(floor(dataset)),chdata,coint);
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);

fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);

if isempty(mspec_ref)
  lmodel_ref = [];
  nobsdrat_ = [];
  lambdac_ = [];
else
  lmodel_ref = strcat('m',num2str(mspec_ref));
  nobsdrat_ = num2str(nobsdrat);
  if isinf(lambdac)
      lambdac_ = [];
  else
      lambdac_ = num2str(100*lambdac);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% if JustMode: do not do simulations and compute forecasts at mode
DOPLOTSONLY = 0%input('if = 1, just Do plots - if 0 compute irfs'); 

nirf = 41;
%% cumulative irf
ccum_irf = [];
for II=1:length(cum_irf)
  ccum_irf = [ccum_irf,cum_irf(II):nvar:nvar^2];    
end

%% set date
I = find(ti == dates);

%%  Configure the Metropolis Algorithm
if JustMode
  nblocks = 1;
  nsim = 1;
end


Verbose = 0;

if ~DOPLOTSONLY
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % do Lambda loop
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  for fixparam = fixparamseq             
    for lambdas = lambdaseq     
      
      fprintf(1,'\n Lambda = %2.3f',[lambdas]);
      fprintf(1,'\n Time %3.2f ',[ti(I)]);

      infofile = strcat(spath,'/info',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
      load(infofile,'nstate','nshocks');
      %                	            [TTT,RRR,valid] = dsgesolv(paranew,mspec);
      %	            nstate = length(TTT);
      %	            nshocks = size(RRR,2);


      
      %% open write files
      %% params file
      infile1 = strcat(spath,'mhpara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
      fid1 = fopen(infile1,'r');		% output file (set to read)   
      num1 = nblocks*nsim*npara*4;            % number of bites per file
      numb1 = nburn*npara*4;                  % number of bites to dicard
      status = fseek(fid1,numb1,'bof');
      

      if ~isnan(lambdas)                
        %% VAR file
        infile2 = strcat(spath,'varpara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
        fid2 = fopen(infile2,'r');		% output file (set to read)   
        num2 = nblocks*nsim*(nvar*(coint+1+nlags*nvar)+nvar^2)*4;            % number of bites per file
        numb2 = nburn*(nvar*(coint+1+nlags*nvar)+nvar^2)*4;                  % number of bites to dicard
        status = fseek(fid2,numb2,'bof');
      end
      
      %% MA representation: TTT (nstate x nstate)
      infile11 = strcat(spath,'/TTT',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));	
      fid11 = fopen(infile11,'r');		% output file (set to read)   
      numb11 = nburn*nstate^2*4;                  % number of bites to dicard
      status = fseek(fid11,numb11,'bof');

      %% MA representation: RRR (nstate x nshocks)
      infile12 = strcat(spath,'/RRR',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));	
      fid12 = fopen(infile12,'r');		% output file (set to read)   
      numb12 = nburn*nstate*nshocks*4;                  % number of bites to dicard
      status = fseek(fid12,numb12,'bof');
      
      %% Rotation file
      infile13 = strcat(spath,'/rot',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
      fid13 = fopen(infile13,'r');		% output file (set to write)   
      numb13 = nburn*nvar^2*4;                  % number of bites to dicard
      status = fseek(fid13,numb13,'bof');

      
      %% Irf DSGE file
      outfile4 = strcat(spath,'/irf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
      fid4 = fopen(outfile4,'w');		% output file (set to write)   
      if ~isnan(lambdas)
        %% Irf VAR file
        outfile5 = strcat(spath,'/virf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
        fid5 = fopen(outfile5,'w');		% output file (set to write)   
      end	                
      %% set initial values for simulations
      
      
      iblock= 0;
      tic
      %for iblock = 1:nblocks  
      
      while ( ftell(fid1) < num1 )
        iblock = iblock+1;
        fprintf(1,' block: %2.0f %3.2f ;',[iblock,toc/(iblock-1)]);
        
        parasimm = fread(fid1,[npara,nsim],'single')';
        TTTsim = fread(fid11,[nstate^2,nsim],'single')';
        RRRsim = fread(fid12,[nstate*nshocks,nsim],'single')';
        rotsim = fread(fid13,[nvar^2,nsim],'single')';

        irfs = zeros(nsim,nvar^2*nirf);
        if ~isnan(lambdas)
          varpara = fread(fid2,[nvar*(coint+1+nlags*nvar)+nvar^2,nsim],'single')';        
          virfs = zeros(size(varpara,1),nvar^2*nirf)';
        end
        

        for j = 1:nsim
          
          params = parasimm(j,:)';
          
          %% model's irf
          TTT = reshape(TTTsim(j,:),nstate,nstate);
          RRR = reshape(RRRsim(j,:),nstate,nshocks);


          aairf = irfdsge(TTT,RRR,params,nvar,mspec,npara,nirf,cum_irf,1,nlags,coint,cointadd,cointall);

          irfs(j,:) = aairf(:)';                      
          
          %% this phimat rotates the VAR approximation to the MA so that the IRF on impact are the same
          %% need lower triangular cholesky
          phimat = reshape(rotsim(j,:),nvar,nvar);  


          if ~isnan(lambdas)                        
            %% var's irf
            
            cct_sim = reshape(varpara(j,1:nvar*(coint+1+nlags*nvar)),coint+1+nlags*nvar,nvar)'; 
            sig_sim = reshape(varpara(j,nvar*(coint+1+nlags*nvar)+1:end),nvar,nvar);  
            
            % Compute MA representation from companion form VAR
            sig_chol = chol(sig_sim)';

            cc_tilded =  [cct_sim(:,coint+1+1:coint+1+nlags*nvar); ...
                          [eye(nvar*(nlags-1)),zeros(nvar*(nlags-1),nvar)]];
            
            if coint > 0
              cc_coint  =  cct_sim(:,1:coint);
            else
              cc_coint  =  zeros(nvar,coint);
            end
            
            cc_cointtilde = [cc_coint ; zeros(nvar*(nlags-1),coint)];

            cc_dy   = eye(nvar*nlags);
            cc_y    = eye(nvar*nlags);
            msel    = [eye(nvar) ; zeros((nlags-1)*nvar,nvar)];
            
            bbirf    = zeros(nirf,nvar^2);
            
            out         = msel'*cc_dy*msel*sig_chol;
            bbirf(1,:)  = out(:)';
            
            for t = 2:nirf;                 
              cc_dy = cc_tilded*cc_dy;
              if coint
                cc_dy = cc_dy+(cc_cointtilde*cointvec*msel')*cc_y;
                cc_y  = cc_y + cc_dy;
              end
              out = msel'*cc_dy*msel*sig_chol;
              bbirf(t,:)    = out(:)';  
            end;
            
            bbirf(:,ccum_irf) = cumsum(bbirf(:,ccum_irf),1);
            
            bbirf = bbirf*kron(phimat,eye(nvar));       
            
            virfs(j,:) = bbirf(:)'; 
          end      
          
          
        end;
        
        fwrite(fid4,irfs','single');
        if ~isnan(lambdas)
          fwrite(fid5,virfs','single');
        end
      end;		% end simulation - given lambda and time period
      fclose(fid1);
      fclose(fid11);
      fclose(fid12);
      fclose(fid13);

      num4 = ftell(fid4);
      fclose(fid4);
    end         % end lambdaseq
  end % end fixparam
  
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% COMPUTE MEANS AND BANDS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DOPLOTSONLY = input('if = 1, just Do plots - if 0 compute means and bands'); 
if ~DOPLOTSONLY

  %% settings
  csize = nirf*nvar^2;
  nburn = 0;
  percent = .90;
  docov = 0;
  fixparam = 0
  L=0;
  for lambdas = lambdaseq     
    L = L+1;
    
    
    fprintf(1,'\n Lambda = %2.3f',[lambdas]);
    fprintf(1,'\n Time %3.2f ',[ti(I)]);

    step_ = 10;
    
    %% Irf model file
    infile4 = strcat(spath,'/irf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
    fid4 = fopen(infile4,'r');		% output file (set to write)   

    data = [];
    for iblock = 1:(nblocks-1)
      tic
      % Read blocks of size nsim
      datadd = fread(fid4,[csize,nsim],'single')';
      dadd = datadd(1:step_:end,:);
      if all(size(datadd) == [nsim,csize])
        data = [data;dadd];
      end
      clear datadd dadd;
      toc
    end;
    fclose(fid4);
    tic
    msel = mean(data,1)';
    bands = zeros(2*length(percent),size(data,2));
    for j=1:length(percent)
      bands(2*(j-1)+1:2*j,:) = hpdint(data,percent(j),1);
    end;
    bandssel =  bands';
    clear data
    toc
    disp('mean')
    
    if ~isnan(lambdas)
      %% Irf VAR file
      infile5 = strcat(spath,'/virf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
      fid5 = fopen(infile5,'r');		% output file (set to write)   

      %%%% compute means and bands - VAR
      data = [];
      for iblock = 1:(nblocks-1)
        tic
        % Read blocks of size nsim
        datadd = fread(fid5,[csize,nsim],'single')';
        dadd = datadd(1:step_:end,:);
        if all(size(datadd) == [nsim,csize])
          data = [data;dadd];
        end
        clear datadd dadd;
        toc
      end;
      fclose(fid5);

      mselv = mean(data,1)';
      bands = zeros(2*length(percent),size(data,2));
      for j=1:length(percent)
        bands(2*(j-1)+1:2*j,:) = hpdint(data,percent(j),1);
      end;
      bandsselv =  bands';
      clear data

    end	                
    
    for Iirf = 1:nvar

      disp('Iirf: ');
      disp(Iirf);

      selirf = nvar*nirf*(Iirf-1)+1:nvar*nirf*Iirf;
      
      outfile41 = strcat(spath,'/Mirf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		
      fid41 = fopen(outfile41,'w');		% output file (set to write)   
      
      out = [msel(selirf),bandssel(selirf,:)];
      fwrite(fid41,out(:),'single');
      fclose(fid41);
      
      if ~isnan(lambdas)

        outfile51 = strcat(spath,'/Mvirf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		
        fid51 = fopen(outfile51,'w');		% output file (set to write)   
        
        out = [mselv(selirf),bandsselv(selirf,:)];
        fwrite(fid51,out(:),'single');
        fclose(fid51);
      end	                
      
    end; %% end shock
    
  end         % end lambdaseq

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% PLOTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
doplots = 1
if doplots
I_Inf = input('compare with inf? put inf if yes')
  
fixparam = 0

tirf = (1:1:nirf)';
sirf = (1:17);

titlenames = varnames;
%titlenames(cum_irf) = strcat(titlenames(cum_irf),' - cumulative');

intx = tirf(sirf);

Iirf = menu('Choose shock',names_shocks);
selirf = nvar*nirf*(Iirf-1)+1:nvar*nirf*Iirf;
L = 0;

%% set YLim
ylims = zeros(nvar,2);
ylims(1,:) = [-40 60];        %% output
ylims(2,:) = [-40 10];        %% infl
ylims(3,:) = [-20 30];        %% fed fund

ylims = [];


Legend102 = cell(length(lambdaseq),1);
Color102 = eye(max(length(lambdaseq),3)); Color102 = Color102(:,1:3);

for lambdas = lambdaseq     
  L = L+1;
  
  fprintf(1,'\n Lambda = %2.3f',[lambdas]);
  fprintf(1,'\n Time %3.2f ',[ti(I)]);

  %% open write files
  %% Irf model file

  if isinf(I_Inf)
    lambdas_ = Inf;
    infile41 = strcat(spath,'/Mvirf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas_),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		
  else
    infile41 = strcat(spath,'/Mirf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		
  end
  
  fid41 = fopen(infile41,'r');		% output file (set to read)   

  out = fread(fid41,[nirf*nvar,3],'single');
  msel = out(:,1);
  bandssel = out(:,2:end);
  fclose(fid41);

  bubu = []
  if ~isempty(bubu)
    lmodel_2 = strcat('m',num2str(106));
    lambdas_2 = 1.25;
    infile41 = strcat(spath,'/Mvirf',lmodel_2,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas_2),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		
    %    infile41 = strcat(spath,'/Mirf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		

    
    fid41 = fopen(infile41,'r');		% output file (set to read)   

    out = fread(fid41,[nirf*nvar,3],'single');
    msel_2 = out(:,1);
    bandssel_2 = out(:,2:end);
    fclose(fid41);
  end

  if ~isnan(lambdas)
    %% Irf VAR file
    infile51 = strcat(spath,'/Mvirf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		
    fid51 = fopen(infile51,'r');		% output file (set to read)   

    out = fread(fid51,[nirf*nvar,3],'single');
    mselv = out(:,1);
    bandsselv = out(:,2:end);
    fclose(fid51);
    
  end
  
  for V = 1:nvar
    figure(101);
    %subplot(nvar,length(lambdaseq),V+nvar*(L-1));   
    subplot(2,ceil(nvar/2),V+nvar*(L-1));   
    mlp = msel(nirf*(V-1)+1:nirf*V);
    bandsp = bandssel(nirf*(V-1)+1:nirf*V,:);
    P = plot(intx,mlp(sirf),'k-');
    set(P,'Color',[.7 .7 .7]);
    set(P,'LineWidth',1);
    hold on;
    P = plot(intx,bandsp(sirf,1),'k:',intx,bandsp(sirf,2),'k:');    set(P,'LineWidth',.3);    set(P,'Color',[.7 .7 .7]);     hold on;
    if ~isnan(lambdas)        
      mlpv = mselv(nirf*(V-1)+1:nirf*V);
      bandspv = bandsselv(nirf*(V-1)+1:nirf*V,:);
      P = plot(intx,mlpv(sirf),'k-.');
      set(P,'LineWidth',1);
      hold on;
      P = plot(intx,bandspv(sirf,1),'k:',intx,bandspv(sirf,2),'k:');
      set(P,'LineWidth',.3);

    end	    
    if ~isempty(bubu);     
      hold on;       mlp = msel_2(nirf*(V-1)+1:nirf*V);      bandsp = bandssel_2(nirf*(V-1)+1:nirf*V,:);
      P = plot(intx,mlp(sirf),'r-'); 
      hold on;P = plot(intx,bandsp(sirf,1),'r:',intx,bandsp(sirf,2),'r:');
    end
    hold off;
    
    if L == 1
      title(titlenames(V));
    end
    if ~isempty(ylims)
      set(gca,'YLim',ylims(V,:));
    end
    
    set(gca,'XLim',[min(tirf(sirf)),max(tirf(sirf))]);
    set(gca,'XTick',intx([1 5:4:end]));
    set(gca,'XTickLabel',num2str(intx([1 5:4:end])-1));


    figure(102);
    subplot(nvar,1,V);   
    mlp = msel(nirf*(V-1)+1:nirf*V);
    bandsp = bandssel(nirf*(V-1)+1:nirf*V,:);
    P = plot(intx,mlp(sirf),'-');        set(P,'LineWidth',1);
    hold on;
P = plot(intx,bandsp(sirf,1),'k:',intx,bandsp(sirf,2),'k:');    set(P,'LineWidth',.3);    set(P,'Color',[.7 .7 .7]);     hold on;
    if ~isnan(lambdas)
      mlpv = mselv(nirf*(V-1)+1:nirf*V);
      bandspv = bandsselv(nirf*(V-1)+1:nirf*V,:);
      P = plot(intx,mlpv(sirf),'k-');
      set(P,'LineWidth',1);
      set(P,'Color',Color102(L,:));
      hold on;
      P = plot(intx,bandspv(sirf,1),'k:',intx,bandspv(sirf,2),'k:'); set(P,'LineWidth',1);
    end	        
    hold off;
    
    if L == 1
      title(titlenames(V));
    end
    if ~isempty(ylims)
      set(gca,'YLim',ylims(V,:));
    end
    Legend102{L} = strcat('\lambda = ',num2str(lambdas));
    
    set(gca,'XLim',[min(tirf(sirf)),max(tirf(sirf))]);
    set(gca,'XTick',intx([1 5:4:end]));
    set(gca,'XTickLabel',num2str(intx([1 5:4:end])-1));
    
  end   



end         % end lambdaseq

figure(102);
legend(Legend102);
hold off;

figure(101);
%set(figure(101),'PaperPosition',[0 0 7 9])
ng=strcat(gpath,lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*min(lambdaseq)),num2str(100*max(lambdaseq)),num2str(100*ti(I)),num2str(nlags),num2str(JustMode),names_shocks(Iirf),'irf',num2str(bubu));

saveas(figure(101),char(ng),'fig');
figure(101);
print(char(strcat(ng,'.eps')),'-depsc2');
print(char(strcat(ng,'.pdf')),'-dpdf');

disp('close')


close all;
pause
for lambdas = lambdaseq     
  L = L+1;
  
  fprintf(1,'\n Lambda = %2.3f',[lambdas]);
  fprintf(1,'\n Time %3.2f ',[ti(I)]);
  
  for Iirf = 1:nvar
    
    selirf = nvar*nirf*(Iirf-1)+1:nvar*nirf*Iirf;
    L = 0;
    
    %% set YLim
    ylims_ = cell(nvar,nvar);
    ylims_(1,1) = {[0 2]};          %% Y, tech
    ylims_(1,2) = {[-2 0]};          %% Y, phi
    ylims_(1,3) = {[0 2]};          %% Y, mu
    ylims_(1,4) = {[-1 1]};          %% Y, b
    ylims_(1,5) = {[-0.5  1]};          %% Y, g
    ylims_(1,6) = {[-1 1]};          %% Y, laf
    ylims_(1,7) = {[-1 0]};          %% Y, M


    ylims_(2,1) = {[-.5 2]};          %% H, tech
    ylims_(2,2) = {[-2 0]};          %% H, phi
    ylims_(2,3) = {[0 2]};          %% H, mu
    ylims_(2,4) = {[-.5 1]};          %% H, b
    ylims_(2,5) = {[-.5  1]};          %% H, g
    ylims_(2,6) = {[-1 .5]};          %% H, laf
    ylims_(2,7) = {[-1 0]};          %% H, M


    ylims_(3,1) = {[-1 .5]};          %% Pi, tech
    ylims_(3,2) = {[0 1.2]};          %% Pi, phi
    ylims_(3,3) = {[-.5 .8]};          %% Pi, mu
    ylims_(3,4) = {[0 1.2]};          %% Pi, b
    ylims_(3,5) = {[-.2  .5]};          %% Pi, g
    ylims_(3,6) = {[-.2 1.5]};          %% Pi, laf
    ylims_(3,7) = {[-.5 .2]};          %% Pi, M

    ylims_(4,1) = {[-1.5 0]};          %% R, tech
    ylims_(4,2) = {[0 1.5]};          %% R, phi
    ylims_(4,3) = {[-.5 1]};          %% R, mu
    ylims_(4,4) = {[0 1.5]};          %% R, b
    ylims_(4,5) = {[-.25  .5]};          %% R, g
    ylims_(4,6) = {[-.5 1]};          %% R, laf
    ylims_(4,7) = {[-.5 1.5]};          %% R, M

    
    ylims = []; %if not [], if yes 1
    
    Legend102 = cell(length(lambdaseq),1);
    Color102 = eye(max(length(lambdaseq),3)); Color102 = Color102(:,1:3);
    

    %% open write files
    %% Irf model file
    if isinf(I_Inf)
      lambdas_ = Inf;
      infile41 = strcat(spath,'/Mvirf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas_),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		
    else
      infile41 = strcat(spath,'/Mirf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		
    end
    fid41 = fopen(infile41,'r');		% output file (set to read)   

    out = fread(fid41,[nirf*nvar,3],'single');
    msel = out(:,1);
    bandssel = out(:,2:end);
    fclose(fid41);
    

    if ~isnan(lambdas)
      %% Irf VAR file
      infile51 = strcat(spath,'/Mvirf',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		
      fid51 = fopen(infile51,'r');		% output file (set to read)   

      out = fread(fid51,[nirf*nvar,3],'single');
      mselv = out(:,1);
      bandsselv = out(:,2:end);
      fclose(fid51);
      
    end

    
    for V = 1:nvar
      figure(1001);
      subplot(nvar,nvar,V+nvar*(Iirf-1));   
      mlp = msel(nirf*(V-1)+1:nirf*V);
      bandsp = bandssel(nirf*(V-1)+1:nirf*V,:);
      P = plot(intx,mlp(sirf),'k-');
      set(P,'Color',[.7 .7 .7]);
      set(P,'LineWidth',1);
      hold on;
      %P = plot(intx,bandsp(sirf,1),':',intx,bandsp(sirf,2),':');  set(P,'Color',[.7 .7 .7]); set(P,'LineWidth',.8);
      hold on;
      
    if ~isnan(lambdas)

      mlpv = mselv(nirf*(V-1)+1:nirf*V);
      bandspv = bandsselv(nirf*(V-1)+1:nirf*V,:);
      P = plot(intx,mlpv(sirf),'k-.');
      set(P,'LineWidth',1);
      hold on;
      P = plot(intx,bandspv(sirf,1),'k:',intx,bandspv(sirf,2),'k:');
      set(P,'LineWidth',.8);
end
      
      hold off;
      
      if Iirf == 1
        title(titlenames(V));
      end
      
      if V == 1 
        ylabel(names_shocks(Iirf));
      end
      if ~isempty(ylims)
        set(gca,'YLim',ylims_{V,Iirf});
      end
      
      set(gca,'XLim',[min(tirf(sirf)),max(tirf(sirf))]);
      set(gca,'XTick',intx([1 5:4:end]));
      
      if Iirf == nvar
        set(gca,'XTickLabel',num2str(intx([1 5:4:end])-1));
      else 
        set(gca,'XTickLabel',[]);
      end	   	   
      set(gca,'FontSize',8.5);
      
      
    end   
    
    
    
  end; %% end shock
  
  %   set(figure(1001),'PaperPosition',[0 0 8 6])
  ng=strcat(gpath,lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*min(lambdaseq)),num2str(100*max(lambdaseq)),num2str(100*ti(I)),num2str(nlags),num2str(JustMode),num2str(100*lambdas),'irfallshocks');
  %saveas(figure(1001),ng,'fig');
  figure(1001);
  print(char(strcat(ng,'.eps')),'-depsc2');
  print(char(strcat(ng,'.pdf')),'-dpdf');
  
  pause
  
end         % end lambdaseq
end % end do plots

clear all;
close all;
