close all;
clear all;
!hostname;
!ps -C matlab;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

run startdsgeprioeuro;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para, ...
 varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode, ...
 stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix] = varspec(chdata);

%[fpath,spath,gpath] = pathspecmh(num2str(floor(dataset)),chdata,coint);
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% if JustMode: do not do simulations and compute forecasts at mode
DOPLOTSONLY = 0%input('if = 1, just Do plots - if 0 compute irfs'); 

nirf = 41;
%% cumulative irf
ccum_irf = [];
for II=1:length(cum_irf)
  ccum_irf = [ccum_irf,cum_irf(II):nvar:nvar^2];    
end

%% set date
I = find(ti == dates);

%%  Configure the Metropolis Algorithm
if JustMode
  nblocks = 1;
  nsim = 1;
end


Verbose = 0;

if ~DOPLOTSONLY
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % do Lambda loop
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  for fixparam = fixparamseq             
    for lambdas = lambdaseq     
      
      fprintf(1,'\n Lambda = %2.3f',[lambdas]);
      fprintf(1,'\n Time %3.2f ',[ti(I)]);

      infofile = strcat(spath,'/info',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
      load(infofile,'nstate','nshocks');
      %                	            [TTT,RRR,valid] = dsgesolv(paranew,mspec);
      %	            nstate = length(TTT);
      %	            nshocks = size(RRR,2);


      
      %% open write files
      %% params file
      infile1 = strcat(spath,'mhpara',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
      fid1 = fopen(infile1,'r');		% output file (set to read)   
      num1 = nblocks*nsim*npara*4;            % number of bites per file
      numb1 = nburn*npara*4;                  % number of bites to dicard
      status = fseek(fid1,numb1,'bof');
      

      if ~isnan(lambdas)                
        %% VAR file
        infile2 = strcat(spath,'varpara',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
        fid2 = fopen(infile2,'r');		% output file (set to read)   
        num2 = nblocks*nsim*(nvar*(coint+1+nlags*nvar)+nvar^2)*4;            % number of bites per file
        numb2 = nburn*(nvar*(coint+1+nlags*nvar)+nvar^2)*4;                  % number of bites to dicard
        status = fseek(fid2,numb2,'bof');
      end
      
      %% MA representation: TTT (nstate x nstate)
      infile11 = strcat(spath,'/TTT',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));	
      fid11 = fopen(infile11,'r');		% output file (set to read)   
      numb11 = nburn*nstate^2*4;                  % number of bites to dicard
      status = fseek(fid11,numb11,'bof');

      %% MA representation: RRR (nstate x nshocks)
      infile12 = strcat(spath,'/RRR',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));	
      fid12 = fopen(infile12,'r');		% output file (set to read)   
      numb12 = nburn*nstate*nshocks*4;                  % number of bites to dicard
      status = fseek(fid12,numb12,'bof');
      
      %% Rotation file
      infile13 = strcat(spath,'/rot',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
      fid13 = fopen(infile13,'r');		% output file (set to write)   
      numb13 = nburn*nvar^2*4;                  % number of bites to dicard
      status = fseek(fid13,numb13,'bof');

      
      %% Irf DSGE file
      outfile4 = strcat(spath,'/irf',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
      fid4 = fopen(outfile4,'w');		% output file (set to write)   
      if ~isnan(lambdas)
        %% Irf VAR file
        outfile5 = strcat(spath,'/virf',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
        fid5 = fopen(outfile5,'w');		% output file (set to write)   
      end	                
      %% set initial values for simulations
      
      
      iblock= 0;
      tic
      %for iblock = 1:nblocks  
      
      while ( ftell(fid1) < num1 )
        iblock = iblock+1;
        fprintf(1,' block: %2.0f %3.2f ;',[iblock,toc/(iblock-1)]);
        
        parasimm = fread(fid1,[nsim,npara],'single');
        TTTsim = fread(fid11,[nsim,nstate^2],'single');
        RRRsim = fread(fid12,[nsim,nstate*nshocks],'single');
        rotsim = fread(fid13,[nsim,nvar^2],'single');

        irfs = zeros(nsim,nvar^2*nirf);
        if ~isnan(lambdas)
          varpara = fread(fid2,[nsim,nvar*(coint+1+nlags*nvar)+nvar^2],'single');        
          virfs = zeros(size(varpara,1),nvar^2*nirf);
        end
        

        for j = 1:nsim
          
          params = parasimm(j,:)';
          
          %% model's irf
          TTT = reshape(TTTsim(j,:),nstate,nstate);
          RRR = reshape(RRRsim(j,:),nstate,nshocks);


          aairf = irfdsge(TTT,RRR,params,nvar,mspec,npara,nirf,cum_irf,1,nlags,coint,cointadd,cointall);

          irfs(j,:) = aairf(:)';                      
          
          %% this phimat rotates the VAR approximation to the MA so that the IRF on impact are the same
          %% need lower triangular cholesky
          phimat = reshape(rotsim(j,:),nvar,nvar);  


          if ~isnan(lambdas)                        
            %% var's irf
            
            cct_sim = reshape(varpara(j,1:nvar*(coint+1+nlags*nvar)),coint+1+nlags*nvar,nvar)'; 
            sig_sim = reshape(varpara(j,nvar*(coint+1+nlags*nvar)+1:end),nvar,nvar);  
            
            % Compute MA representation from companion form VAR
            [sig_chol,p_chol] = chol(sig_sim);
            if p_chol > 0 
              [V_chol,D_chol] = eig(sig_sim);
              sig_sim = V_chol*abs(D_chol)*inv(V_chol);
              [sig_chol,p_chol] = chol(sig_sim);
            end
            sig_chol = sig_chol';

            cc_tilded =  [cct_sim(:,coint+1+1:coint+1+nlags*nvar); ...
                          [eye(nvar*(nlags-1)),zeros(nvar*(nlags-1),nvar)]];
            
            if coint > 0
              cc_coint  =  cct_sim(:,1:coint);
            else
              cc_coint  =  zeros(nvar,coint);
            end
            
            cc_cointtilde = [cc_coint ; zeros(nvar*(nlags-1),coint)];

            cc_dy   = eye(nvar*nlags);
            cc_y    = eye(nvar*nlags);
            msel    = [eye(nvar) ; zeros((nlags-1)*nvar,nvar)];
            
            bbirf    = zeros(nirf,nvar^2);
            
            out         = msel'*cc_dy*msel*sig_chol;
            bbirf(1,:)  = out(:)';
            
            for t = 2:nirf;                 
              cc_dy = cc_tilded*cc_dy;
              if coint
                cc_dy = cc_dy+(cc_cointtilde*cointvec*msel')*cc_y;
                cc_y  = cc_y + cc_dy;
              end
              out = msel'*cc_dy*msel*sig_chol;
              bbirf(t,:)    = out(:)';  
            end;
            
            bbirf(:,ccum_irf) = cumsum(bbirf(:,ccum_irf),1);
            
            bbirf = bbirf*kron(phimat,eye(nvar));       
            
            virfs(j,:) = bbirf(:)'; 
          end      
          
          
        end;
        
        fwrite(fid4,irfs,'single');
        if ~isnan(lambdas)
          fwrite(fid5,virfs,'single');
        end
      end;		% end simulation - given lambda and time period
      fclose(fid1);
      fclose(fid11);
      fclose(fid12);
      fclose(fid13);

      num4 = ftell(fid4);
      fclose(fid4);
    end         % end lambdaseq
  end % end fixparam
  
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% COMPUTE MEANS AND BANDS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DOPLOTSONLY = input('if = 1, just Do plots - if 0 compute means and bands'); 
if ~DOPLOTSONLY

  %% settings
  csize = nirf*nvar^2;
  nburn = 0;
  percent = .90;
  docov = 0;
  fixparam = 0
  L=0;
  for lambdas = lambdaseq     
    L = L+1;
    
    
    fprintf(1,'\n Lambda = %2.3f',[lambdas]);
    fprintf(1,'\n Time %3.2f ',[ti(I)]);

    step_ = 10;
    
    %% Irf model file
    infile4 = strcat(spath,'/irf',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
    fid4 = fopen(infile4,'r');		% output file (set to write)   

    data = [];
    for iblock = 1:(nblocks-1)
      tic
      % Read blocks of size nsim
      datadd = fread(fid4,[nsim,csize],'single');
      dadd = datadd(1:step_:end,:);
      if all(size(datadd) == [nsim,csize])
        data = [data;dadd];
      end
      clear datadd dadd;
      toc
    end;
    fclose(fid4);
    tic
    msel = mean(data,1)';
    bands = zeros(2*length(percent),size(data,2));
    for j=1:length(percent)
      bands(2*(j-1)+1:2*j,:) = hpdint(data,percent(j),1);
    end;
    bandssel =  bands';
    clear data
    toc
    disp('mean')
    
    if ~isnan(lambdas)
      %% Irf VAR file
      infile5 = strcat(spath,'/virf',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
      fid5 = fopen(infile5,'r');		% output file (set to write)   

      %%%% compute means and bands - VAR
      data = [];
      for iblock = 1:(nblocks-1)
        tic
        % Read blocks of size nsim
        datadd = fread(fid5,[nsim,csize],'single');
        dadd = datadd(1:step_:end,:);
        if all(size(datadd) == [nsim,csize])
          data = [data;dadd];
        end
        clear datadd dadd;
        toc
      end;
      fclose(fid5);

      mselv = mean(data,1)';
      bands = zeros(2*length(percent),size(data,2));
      for j=1:length(percent)
        bands(2*(j-1)+1:2*j,:) = hpdint(data,percent(j),1);
      end;
      bandsselv =  bands';
      clear data

    end	                
    
    for Iirf = 1:nvar

      disp('Iirf: ');
      disp(Iirf);

      selirf = nvar*nirf*(Iirf-1)+1:nvar*nirf*Iirf;
      
      outfile41 = strcat(spath,'/Mirf',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		
      fid41 = fopen(outfile41,'w');		% output file (set to write)   
      
      out = [msel(selirf),bandssel(selirf,:)];
      fwrite(fid41,out(:),'single');
      fclose(fid41);
      
      if ~isnan(lambdas)

        outfile51 = strcat(spath,'/Mvirf',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode),'-I',num2str(Iirf));		
        fid51 = fopen(outfile51,'w');		% output file (set to write)   
        
        out = [mselv(selirf),bandsselv(selirf,:)];
        fwrite(fid51,out(:),'single');
        fclose(fid51);
      end	                
      
    end; %% end shock
    
  end         % end lambdaseq

end
clear all;
close all;
