% filename:    prio.m
% description: generates draws from the prior and computes prior moments for the parameters of interest
%              marginal log likelihood
%              with respect to the DSGE model parameters
close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
run startdsswfed;
!hostname;
!ps -C matlab;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varspec;

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);

if isempty(mspec_ref)
  lmodel_ref = [];
  nobsdrat_ = [];
  lambdac_ = [];
  nlagsc_ = [];
  n_s_ = [];

else
  lmodel_ref = strcat('m',num2str(mspec_ref));
  nobsdrat_ = num2str(100*nobsdrat);
  lambdac_ = [];
  n_s_ = num2str(n_s);
  if nlagsc == 4
    nlagsc_ = [];
  else
    nlagsc_ = num2str(nlagsc);
  end  
  
end
stup_ = num2str(stup);

%% set of parameters 
list_t0 = find(para_type == 0);
npara0 = length(list_t0)
list_t1 = find(para_type == 1);
npara1 = length(list_t1)
list_t2 = find(para_type == 2);
npara2 = length(list_t2)

dataset = num2str(10*dataset);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
if isempty(mspec_ref)
  priorfile = strcat('pri',lprior)
elseif ~isempty(mspec_ref)
  priorfile = strcat('pri',lprior)
end
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);

pmean = pmean.*(1-para_mask)+para_fix.*para_mask;

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);

fpath2 = strcat(fpath,'/',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,'/');
mkdir(fpath2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% doing rolling sample estimation?
ROLL = 0
SAVEALL = 0
DOFORECAST = 1
if ROLL 
  DOFORECAST = 1;
  SAVEALL = 0;
end
start_period = stime;                               % minimum # quarters to compute VAR estimates 


outfile00 = strcat(fpath2,lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,'MhPrio.tex');	% output file name 

fid00 = fopen(outfile00,'w');		% output file (set to write)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do loop over different types of prior
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mh = zeros(1,3);
percent = .90;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 1 take care of indeterminacy region
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

infile001 =  strcat(spath,'/priodet',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);
fid001 = fopen(infile001,'r');		% input file (set to read)   
detsim = fread(fid001,[1,inf],'int8')';
fclose(fid001);
Indrat = length(find(detsim))/length(detsim);
fprintf(1,'\n\n  ratio of indeterminacy draws %2.2f\n',Indrat);
fprintf(fid00,'\n\n  ratio of  indeterminacy draws %2.2f\n',Indrat);

Mh(1) = log(1-Indrat);
fprintf(fid00,'\n marg lik -- Indeterminacy part: %9.9f',[Mh(1)]);

for L = 2:3

  if L == 2
    %% type 0 params
    if isempty(list_t0)
      COMPMARG = 0
    else
      COMPMARG = 1

      infile1 = strcat(spath,'/mhprioparat0',lmodel,lprior(1),dataset);
      fid1 = fopen(infile1,'r');		% input file (set to read)   
      num1 = nblocks*nsim*npara0*4;            % number of bites per file
      numb1 = nburn*npara0*4;                  % number of bites to discard
      status = fseek(fid1,numb1,'bof');       % discard nburn 

      infile2 =  strcat(spath,'/priopostt0',lmodel,lprior(1),dataset);
      fid2 = fopen(infile2,'r');		% input file (set to read)   
      num2 = nblocks*nsim*4;            % number of bites per file
      numb2 = nburn*4;                  % number of bites to discard
      status = fseek(fid2,numb2,'bof');       % discard nburn 

      nfnpara = npara0
      densfac = -150;%-1100%
      fprintf(fid00,'\n marg lik -- type 0 params:');

    end
    
  elseif L == 3
    
    %% type 2 params
    if isempty(list_t2)
      COMPMARG = 0
    else
      COMPMARG = 1

      infile1 = strcat(spath,'/mhprioparat2',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);
      fid1 = fopen(infile1,'r');		% input file (set to read)   
      num1 = nblocks*nsim*npara2*4;            % number of bites per file
      numb1 = nburn*npara2*4;                  % number of bites to discard
      status = fseek(fid1,numb1,'bof');       % discard nburn 

      infile2 = strcat(spath,'/priopostt2',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);
      fid2 = fopen(infile2,'r');		% input file (set to read)   
      num2 = nblocks*nsim*4;            % number of bites per file
      numb2 = nburn*4;                  % number of bites to discard
      status = fseek(fid2,numb2,'bof');       % discard nburn 

      nfnpara = npara2
      densfac = -150;%-1100%
      
      infile212 =  strcat(spath,'/detrat2',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);
      fid212 = fopen(infile212,'r');		% input file (set to read)   
      Indrat2 = fread(fid212,[1],'single')
      fclose(fid212);

      Mh(L) = log(1-Indrat2);
      fprintf(fid00,'\n marg lik -- type 2 params:');

    end
  end
  

  if COMPMARG

    
    hmax = 5;
    p = (.1:.1:(9-1)*.1)';
    pcrit = chi2inv(p,nfnpara);
    

    %% params file
    theta = fread(fid1,[nfnpara,inf],'single')';
    fclose(fid1);

    ndraws = size(theta,1)
    
    
    %%%%%%%%%%% Compute posterior moments of the parameters
    thetahat = mean(theta,1)';
    thetasig = cov(theta,1);
    
    [u,s,v] = svd(thetasig);
    md = min(size(s)); 
    bigev = find(diag(s(1:md,1:md))>1e-6);
    thetasigdim = length(bigev);
    %[rank(thetasig),nfnpara,thetasigdim]		
    thetasiglndet = 0;
    for i = 1:nfnpara
      
      if i > thetasigdim
        s(i,i) = 0;
      else
        
        thetasiglndet = thetasiglndet+log(s(i,i));             
        s(i,i)    = 1/s(i,i);
      end
    end
    thetasiginv  = u*s*u';

    
    %thetasiginv = inv(thetasig);
    %thetasiglndet = log(det(thetasig))];
    
    %%%%%%%%%%% Compute density estimates
    suminvlike = zeros(length(p),1);
    laginvlike = zeros(hmax,length(p));
    gaminvlike = zeros(hmax,length(p));
    

    %% Posterior file
    postsim = fread(fid2,[1,inf],'single')';
    fclose(fid2);
    
    ndraws = 0;            
    
    
    for j = 1:size(theta,1)
      thetasim = theta(j,:)';
      
      post = postsim(j);
      
      lnftheta = -log(p)-.5*nfnpara*log(2*pi)-.5*thetasiglndet-.5*(thetasim-thetahat)'*thetasiginv*(thetasim-thetahat);
      %					lnftheta = -.5*nfnpara*log(2*pi)-.5*thetasiglndet-.5*(thetasim-thetahat)'*thetasiginv*(thetasim-thetahat);
      
      indtheta = ( (thetasim-thetahat)'*thetasiginv*(thetasim-thetahat) < pcrit );
      invlike = exp(lnftheta-post+densfac).*indtheta;
      %                   invlike = repmat(exp(-post+densfac),size(indtheta));
      % invlike = repmat(exp(lnftheta-post+densfac),size(indtheta));
      if ~isinf(invlike) & ~isnan(invlike)
        laginvlike = [invlike';laginvlike(1:hmax-1,:)];
        gaminvlike = gaminvlike + laginvlike.*(ones(hmax,1)*invlike');
        suminvlike = suminvlike+invlike;
        
        ndraws = ndraws+1;
      end
    end

    clear theta postsim;
    
    
    ndraws
    meaninvlike = suminvlike/ndraws;
    gaminvlike = gaminvlike/ndraws -ones(hmax,1)*(meaninvlike.^2)';
    
    %%% compute standard errors
    suminvlikeerror = gaminvlike(1,:);
    for j = 2:hmax;
      suminvlikeerror = suminvlikeerror+2*gaminvlike(j,:)*(1-((j-1)/hmax));
    end
    suminvlikeerror = 100*sqrt(suminvlikeerror/ndraws)./meaninvlike';
    
    pint = densfac-log(meaninvlike);
    
    pint0 = mean(pint(~isnan(pint) & ~isinf(pint)));
    
    fprintf(1,'\n marginal data density : %4.7f',pint);
    fprintf(1,'\n average across p : %4.7f',pint0)
    fprintf(1,'\n simulation error (percent) : %4.7f',suminvlikeerror);
    
    Mh(L) = Mh(L)+pint0
     fprintf(fid00,' %9.9f',[Mh(L)]);
    
  end
end


Mh = sum(Mh)

outfile0 = strcat(spath,'MhPrio',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_)


fid0 = fopen(outfile0,'w');     % output file (set to write)   
fwrite(fid0,Mh,'single');
fclose(fid0);


fprintf(1,'\n\n\n ');
fprintf(1,'\n marg lik: %9.9f',[Mh]);
fprintf(fid00,'\n marg lik: %9.9f',[Mh]);


fclose(fid00);		% close output file
