close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

run startdsswfed;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para, ...
 varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode, ...
 stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix,para_aux,mspec_ref] = varspec(chdata);


%[fpath,spath,gpath] = pathspecmh(num2str(floor(dataset)),chdata,coint);
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f',[mspec,dataset,mprior]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution
lmodel    = strcat('m',num2str(mspec));
lprior    = num2str(mprior);
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% doing rolling sample estimation?
ROLL = 0
SAVEALL = 0
DOFORECAST = 1
if ROLL 
  DOFORECAST = 1;
  SAVEALL = 0;
end
start_period = stime;                               % minimum # quarters to compute VAR estimates 


%mkdir(spath,'/2/');
%spath = strcat(spath,'/1/');
%start_period = nobsall-qahead+1-3

trial_period = nobsall-start_period-qahead+1;      % # quarters to compute RMSE

Verbose = 0;

fixparam = fixparamseq(1);
if fixparam ~= 0 
  fixparam
  break
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do Lambda loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lambdaseq = [ceil(100*((4*4+1)+4)/100)/100 .5 .75 1 1.25 1.5 2 5 Inf NaN];

mspecseq = [106,101,105,104];


allmodelnames = cell(20,1);
allmodelnames(1) = {'No Indexation'};
allmodelnames(6) = {'Baseline'};
allmodelnames(4) = {'No Price and Wage Stickiness'};
allmodelnames(5) = {'No Wage Stickiness'};
allmodelnames(8) = {'No Habit'};
allmodelnames(9) = {'No Investment Adj. Costs'};
allmodelnames(10) = {'No Capital Adj. Costs'};


%% 101 - baseline - no indexation
%% 102 - partial indexation w/ tech update
%% 106 - partial indexation w/o tech update
%% 107 - full indexation w/o tech update
%% 103 - output gap targeting (partial indexation w/o tech update)
%% 104 - No price and wage rigidity
%% 105 - No wage rigidity (OLD: \Bigphi (full indexation w/o tech update))
%% 108 - No habit persistence
%% 109 - No inv adj cost
%% 110 - No cap utilization
%% 111 - No persistence mu and b
%% 112 - No persistence phi
modelnames = cell(length(mspecseq),1);
for ii = 1:length(mspecseq)
  modelnames{ii} = {allmodelnames{mspecseq(ii)-100}};
end

Mh = zeros(length(mspecseq),length(lambdaseq));
Iseq = find(ti == dates);

M = 0;
for mspec = mspecseq;
  M = M+1;
  L = 0;
  for lambdas = lambdaseq     
    
    L = L+1;
    
    lmodel    = strcat('m',num2str(mspec));

    infile0 = strcat(spath,'Mh',lmodel,lprior,dataset,num2str(cointall),st,num2str(fixparam),num2str(JustMode),num2str(100*lambdas),num2str(100*ti(Iseq)));
    

    fid0 = fopen(infile0,'r');     % output file (set to write)   
    pint0 = fread(fid0,[1,1],'single');
    fclose(fid0);
    Mh(M,L) = pint0;

    
  end         % end lambdaseq
  
end %% end model

tirf = lambdaseq./(1+lambdaseq);
tirf(end-1) = 1;
sirf = tirf;
sirf(end) = 1.1;
tirf = num2str(tirf',2);
tirf(end,:) = 'DSGE';

Acolor = zeros(size(Mh,1),3);
Lstyle = cell(size(Mh,1),1);
Mstyle = cell(size(Mh,1),1);
Lstyle(1) = {'-'};
Mstyle(1) = {'x'};
Acolor(2,:) = [1 0 0];
Lstyle(2) = {'--'};
Mstyle(2) = {'*'};
Acolor(3,:) = [0 1 0];
Lstyle(3) = {'-.'};
Mstyle(3) = {'o'};
Acolor(4,:) = [.2 .2 .2];
Lstyle(4) = {'-.'};
Mstyle(4) = {'.'};

figure(101);
M = 0;
mspecname = [];
for mspec = mspecseq;
  mspecname = strcat(mspecname,'-',num2str(mspec));
  M = M+1;

  P = plot(sirf,Mh(M,:));
  set(P,'Color',Acolor(M,:));
  set(P,'LineStyle',Lstyle{M});
  set(P,'Marker',Mstyle{M});
  set(P,'MarkerSize',8);
  set(P,'LineWidth',1.5);

  T =text(.99*sirf(end),Mh(M,end),modelnames{M});
  set(T,'VerticalAlignment','top');
  set(T,'HorizontalAlignment','right');
  set(T,'FontSize',12);
  set(T,'FontWeight','bold');
  
  T =text(sirf(end),Mh(M,end),num2str(Mh(M,end),'%5.0f'));
  set(T,'VerticalAlignment','bottom');
  set(T,'HorizontalAlignment','left');
  set(T,'FontSize',11);
  set(T,'FontWeight','bold');
  set(T,'Color',Acolor(M,:));
  
  T =text(sirf(1),Mh(M,1),num2str(Mh(M,1),'%5.0f'));
  set(T,'VerticalAlignment','bottom');
  set(T,'HorizontalAlignment','left');
  set(T,'FontSize',11);
  set(T,'FontWeight','bold');
  set(T,'Color',Acolor(M,:));

  [ym,im] = max(Mh(M,:));
  T =text(sirf(im),Mh(M,im),num2str(Mh(M,im),'%5.0f'));
  set(T,'VerticalAlignment','bottom');
  set(T,'HorizontalAlignment','left');
  set(T,'FontSize',11);
  set(T,'FontWeight','bold');
  set(T,'Color',Acolor(M,:));

  hold on;

  set(gca,'YLim',[min(min(Mh(1:M,:)))-20,max(max(Mh(1:M,:)))+20]);
  set(gca,'XLim',[min(sirf),max(sirf)]);
  set(gca,'XTick',sirf');
  set(gca,'XTickLabel',tirf);

  xlabel('\lambda / (1+ \lambda)')
  figure(101);
  %set(figure(101),'PaperPosition',[0 0 7 9])
  ng=strcat(gpath,'lambdaplot',mspecname);

  %saveas(figure(101),char(ng),'fig');
  figure(101);
  print(char(strcat(ng,'.eps')),'-depsc2');
  print(char(strcat(ng,'.pdf')),'-dpdf');

end
hold off;


