function [ZZ,DD,DDcointadd,QQ,EE,MM,retcode] = measur(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint,cointadd,cointall);
%% description:
%% solution to DSGE model - delivers transition equation for the state variables  S_t
%% transition equation: S_t = TC+TTT S_{t-1} +RRR eps_t, where var(eps_t) = QQ
%% define the measurement equation: X_t = ZZ S_t +D+u_t
%% where u_t = eta_t+MM* eps_t with var(eta_t) = EE
%% where var(u_t) = HH = EE+MM QQ MM', cov(eps_t,u_t) = VV = QQ*MM'



retcode = 1;


if valid < 1;
    retcode = 0;

    ZZ = [];
    DD = [];
    QQ = [];
    EE = [];
    MM = [];    
    DDcointadd = [];
    
    return
end

nstate = size(TTT,1);
DDcointadd = 0;

if any(mspec == [101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16])

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% step 1: assign names to the parameters
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %% Parameters 
    [alp,zeta_p,iota_p,del,ups,Bigphi,s2,h,a2,nu_l,nu_m,zeta_w,iota_w,law,rstar,psi1,psi2,rho_r,pistar,...
    gam,Lstar,chi,laf,gstar,Ladj,rho_z,rho_phi,rho_chi,rho_laf,rho_mu,rho_b,rho_g,...
    sig_z,sig_phi,sig_chi,sig_laf,sig_mu,sig_b,sig_g,sig_r,...
    bet,zstar,phi,istokbarst,rkstar,cstar,ystar,istar,kstar,kbarstar,Rstarn,wstar,wadj,mstar] = getpara(para,mspec);


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% step 2: assign names to the columns of GAM0, GAM1 -- state variables
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    states;

    %% additional states
    
    y_t1 = n_end+n_exo+n_exp+1;
    %c_t1 = n_end+n_exo+n_exp+2;
    %i_t1 = n_end+n_exo+n_exp+3;
    %w_t1 = n_end+n_exo+n_exp+4;
    %m_t1 = n_end+n_exo+n_exp+5;
    
        if nstate ~= (n_end+n_exo+n_exp+1)%(n_end+n_exo+n_exp+5)

            retcode = 0;
        
            yyyyd = zeros(nvar,nvar);
            xxyyd = zeros(1+nlags*nvar,nvar);
            xxxxd = zeros(1+nlags*nvar,1+nlags*nvar);
        
            disp('\n\n number of states does not match in vaprio\n');
            return
        end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% step 3: assign measurement equation : X_t = ZZ S_t +D+u_t
    %% where u_t = eta_t+MM* eps_t with var(eta_t) = EE
    %% where var(u_t) = HH = EE+MM QQ MM', cov(eps_t,u_t) = VV = QQ*MM'
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% create system matrices for state space model
    ZZ = zeros(nvar+coint,nstate);
    %% constant
    DD = zeros(nvar+coint,1);
    %% cov(eps_t,u_t) = VV
    MM = zeros(nvar+coint,nex);   
    %% var(eta_t) = EE
    EE = zeros(nvar+coint,nvar+coint);
    %% var(eps_t) = QQ
    QQ =  zeros(nex,nex);
 
    
    
    if any(mspec == [101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116])  

        %% Output growth
        ZZ(1,y_t) = 1;
        ZZ(1,y_t1) = -1;
        ZZ(1,z_t) = 1;
        
        DD(1) = 100*(gam+(alp*log(ups)/(1-alp)));
        
        
        %% Hours
        ZZ(2,L_t) = 1;
        
        DD(2) = 100*log(Ladj);
        %+log(Lstar);
        
        %% Labor Share
        ZZ(3,L_t) = 1;
        ZZ(3,w_t) = 1;
        ZZ(3,y_t) = -1;
        
        DD(3) = 100*log((1-alp)/(1+laf));
        
        %% Inflation
        ZZ(4,pi_t) = 4;
        
        DD(4) = 400*log(pistar);      
            
        %% Nominal interest rate
        ZZ(5,R_t) = 4;
        
        DD(5) = 400*log(Rstarn);  
               
        QQ(z_sh,z_sh) = sig_z^2;
        QQ(phi_sh,phi_sh) = sig_phi^2;
        QQ(g_sh,g_sh) = sig_g^2;
        QQ(laf_sh,laf_sh) = sig_laf^2;        
        QQ(r_sh,r_sh) = sig_r^2;        
    
    elseif any(mspec == [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16])  

        %% Output growth
        ZZ(1,y_t) = 1;
        ZZ(1,y_t1) = -1;
        ZZ(1,z_t) = 1;
        
        DD(1) = 100*(gam+(alp*log(ups)/(1-alp)));
        
        
        %% Hours
        ZZ(2,L_t) = 1;
        
        DD(2) = 100*log(Ladj);
        %+log(Lstar);
        
        %% Inflation
        ZZ(3,pi_t) = 4;
        
        DD(3) = 400*log(pistar);      
            
        %% Nominal interest rate
        ZZ(4,R_t) = 4;
        
        DD(4) = 400*log(Rstarn);  
               
        QQ(z_sh,z_sh) = sig_z^2;
        QQ(phi_sh,phi_sh) = sig_phi^2;
        QQ(g_sh,g_sh) = sig_g^2;
        QQ(laf_sh,laf_sh) = sig_laf^2;        
        QQ(r_sh,r_sh) = sig_r^2;        
    
        
    end     
    
    
end

