close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
run startdsswfed;
!hostname;
!ps -C matlab;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para, ...
 varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode, ...
 stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix,para_aux,mspec_ref,nobsdrat,lambdac] = varspec(chdata);

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);

if isempty(mspec_ref)
  lmodel_ref = [];
  nobsdrat_ = [];
  lambdac_ = [];
  mm
else
  lmodel_ref = strcat('m',num2str(mspec_ref));
  nobsdrat_ = num2str(nobsdrat);
  if isinf(lambdac)
      lambdac_ = [];
  else
      lambdac_ = num2str(100*lambdac);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);
Iseq = find(ti == dates);
I = Iseq;
dataset = num2str(10*dataset);

if stime                           
  YY = YYall(I-stime+1:I,:);
  XX = XXall(I-stime+1:I,:);             
  tiI = ti(I-stime+1:I);
  
  YY_p = YYall(1:I-stime,:);
  XX_p = XXall(1:I-stime,:);             
  nobs_p = size(YY_p,1)

else
  YY     = YYall(1:I,:);
  XX     = XXall(1:I,:);                   
  tiI = ti(1:I);
end
nobs = size(YY,1)


YYYY = YY'*YY;
XXYY = XX'*YY;
XXXX = XX'*XX;

MeY_0 = mean(YY,1)';
VaY_0 = cov(YY,1);
stdY_0 = sqrt(diag(VaY_0));
CorY_0 = VaY_0./(stdY_0*stdY_0');
ACorY_0 = zeros(nvar,1);
for ii = 1:nvar;
  ACorY_0(ii) = corr(YY(:,ii),XX(:,1+cointall+ii));
end

stats_0 = [MeY_0' stdY_0' ACorY_0' CorY_0(:)'];


MeY_p = mean(YY_p,1)';
VaY_p = cov(YY_p,1);
stdY_p = sqrt(diag(VaY_p));
CorY_p = VaY_p./(stdY_p*stdY_p');
ACorY_p = zeros(nvar,1);
for ii = 1:nvar;
  ACorY_p(ii) = corr(YY_p(:,ii),XX_p(:,1+cointall+ii));
end

stats_p = [MeY_p' stdY_p' ACorY_p' CorY_p(:)'];
outfile200 = strcat(spath,'/statsPre');		
fid200 = fopen(outfile200,'w');		% output file (set to write)   
fwrite(fid200,stats_p,'single');
fclose(fid200);

%% take constant out and demean
XX_p = XX_p(:,2:end); 
YYYY_p = (YY_p-repmat(mean(YY_p,1),[size(YY_p,1) 1]))'*(YY_p-repmat(mean(YY_p,1),[size(YY_p,1) 1]))/nobs_p;
XXYY_p = (XX_p-repmat(mean(XX_p,1),[size(XX_p,1) 1]))'*(YY_p-repmat(mean(YY_p,1),[size(YY_p,1) 1]))/nobs_p;
XXXX_p = (XX_p-repmat(mean(XX_p,1),[size(XX_p,1) 1]))'*(XX_p-repmat(mean(XX_p,1),[size(XX_p,1) 1]))/nobs_p;

outfile2000 = strcat(spath,'/yyyy',dataset,num2str(nobs_p),'PreSample');		
fid2000 = fopen(outfile2000,'w');		% output file (set to write)   
outfile3000 = strcat(spath,'/xxyy',dataset,num2str(nobs_p),'PreSample');		
fid3000 = fopen(outfile3000,'w');		% output file (set to write)   
outfile4000 = strcat(spath,'/xxxx',dataset,num2str(nobs_p),'PreSample');		
fid4000 = fopen(outfile4000,'w');		% output file (set to write)   

fwrite(fid2000,YYYY_p,'single');
fwrite(fid3000,XXYY_p,'single');
fwrite(fid4000,XXXX_p,'single');

fclose(fid2000);
fclose(fid3000);
fclose(fid4000);
disp(nobs_p)



ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  Configure the Metropolis Algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cc0     = 0.01;
cc      = 0.12;

Verbose = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reference Model for Prior Second Moments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nobsd = (1/nobsdrat)*nobs;

if ~isnan(mspec_ref)

  lmodel_ref    = strcat('m',num2str(mspec_ref));

  %% WANT POPULATION OR SAMPLE MOMENTS?
  POPUL = 1
  if POPUL 
    POP_ = '';
  else
    POP_ = 'P';
  end

  infile02 = strcat(spath,'/yyyy',lmodel_ref,lprior,dataset,POP_,'ThMean');
  fid02 = fopen(infile02,'r');		% input file (set to write)   
  infile03 = strcat(spath,'/xxyy',lmodel_ref,lprior,dataset,POP_,'ThMean');		
  fid03 = fopen(infile03,'r');		% input file (set to write)   
  infile04 = strcat(spath,'/xxxx',lmodel_ref,lprior,dataset,POP_,'ThMean');		
  fid04 = fopen(infile04,'r');		% input file (set to write)   

  k = nlags*nvar;

  YYYYd = fread(fid02,[nvar nvar],'single');
  XXYYd = fread(fid03,[k  nvar],'single');
  XXXXd = fread(fid04,[k k],'single');
  
  fclose(fid02);
  fclose(fid03);
  fclose(fid04);

elseif isnan(mspec_ref)

  nobs_p = 94;
  k = nlags*nvar;

  infile02 = strcat(spath,'/yyyy',dataset,num2str(nobs_p),'PreSample');
  fid02 = fopen(infile02,'r');		% output file (set to write)   
  infile03 = strcat(spath,'/xxyy',dataset,num2str(nobs_p),'PreSample');		
  fid03 = fopen(infile03,'r');		% output file (set to write)   
  infile04 = strcat(spath,'/xxxx',dataset,num2str(nobs_p),'PreSample');		
  fid04 = fopen(infile04,'r');		% output file (set to write)   

  YYYYd = fread(fid02,[nvar nvar],'single')
  XXYYd = fread(fid03,[k  nvar],'single')
  XXXXd = fread(fid04,[k k],'single');
  
  fclose(fid02);
  fclose(fid03);
  fclose(fid04);

end

infile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,[],[],'Mean');
fid0 = fopen(infile0,'r');     % input file (set to read)   
params_1bar = fread(fid0,[npara,1],'single');
fclose(fid0);  
params_1bar = params_1bar.*(1-para_mask)+para_fix.*para_mask;


YYYYd = YYYYd*nobsd;
XXYYd = XXYYd*nobsd;
XXXXd = XXXXd*nobsd;


lambdacs = lambdac;
lambdac = lambdac*nobsd;
if isinf(lambdac)
  addc = 'inf';
else
  addc = '';
end

marglh = 'marglh';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set date
I = find(ti == dates);

%% prior params file 

infile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,'Mode');

if ~exist(infile0,'file');
  %  lprior0    = num2str(1);
  %infile0 = strcat(spath,lmodel,lprior0,'para',num2str(cointall),st,num2str(NaN),num2str(100*ti(I)),num2str(nlags),'Mean');
  infile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,[],[],'Mean')
end

fid0 = fopen(infile0,'r');     % input file (set to read)   
params = fread(fid0,[npara,1],'single');
fclose(fid0);
params = params.*(1-para_mask)+para_fix.*para_mask;

CH = 2%menu('Re -  Maximize the posterior density?','Yes','No');

  
if CH == 1
  %%     Maximize the posterior density
  x0= invtrans(params,trspec)+.0005*randn(npara,1);
  H0= eye(npara)*1E-4;
  nit = input('number of iterations?'); 

  crit= 1E-16;
  MIN = 1;
  
  [fh,xh,g,H,itct,fcount,retcode] = csminwel('objfcnprio',x0,H0,[],crit,nit,...
                                             params_1bar,YYYYd,XXYYd,XXXXd,nobsd,lambdac,nlags,nvar,mspec,npara,trspec,...
                                             pmean,pstdd,pshape,para_mask,para_fix,para_aux,marglh,coint,cointadd,cointall,MIN);

  xh = real(xh);
  params = trans(xh,trspec);
  params = params.*(1-para_mask)+para_fix.*para_mask;
  disp('move on?');	pause;

  outfile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,'Mode')
  fid0 = fopen(outfile0,'w');     % output file (set to write)   
  fwrite(fid0,params','single');
  fclose(fid0);
  
end


%% prior params file (first block has already been thrown out)
infile01 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,'Hessian');   
if ~exist(infile01,'file');
  CH = 1;
else
  fid01 = fopen(infile01,'r');     % input file (set to read)   
  hessian = fread(fid01,[npara,npara],'single');
  fclose(fid01);
  CH = 2%menu('Re-Compute hessian?','Yes','No','just finish');
end
if CH == 1
  MIN = 0;
  hessian = hessizero('objfcnprio',[params,para_mask],1,...
                      params_1bar,YYYYd,XXYYd,XXXXd,nobsd,lambdac,nlags,nvar,mspec,npara,trspec,...
                      pmean,pstdd,pshape,para_mask,para_fix,para_aux,marglh,coint,cointadd,cointall,MIN);
  disp('proceed?');     
  %pause
elseif CH == 3
  return;
end;
outfile01 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,'Hessian');   

fid01 = fopen(outfile01,'w');     % output file (set to write)   
fwrite(fid01,hessian(:),'single');
fclose(fid01);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set up Metropolis Hastings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rankhhm = npara;
% Create Inverse by Singular Value Decomposition
[u,s,v] = svd(hessian);
md = min(size(s)); 
bigev = find(diag(s(1:md,1:md))>1e-6);
sigpropdim = length(bigev);

sigproplndet = 0;
for i = 1:npara
  
  if i > sigpropdim
    s(i,i) = 0;
  else
    
    s(i,i)    = 1/s(i,i);
    sigproplndet = sigproplndet+log(s(i,i));  
    
  end
end
invhhm  = u*s*u';
sigscale = u*sqrt(s);
sigpropinv  =  hessian; 
clear hessian*;


statname = {'POST';'LIKE';'PROP';'PROPPOST';'REJECT'};

pred = [];
vpred = [];

%% Fix non-auxiliary parameters
params_ = params.*para_aux+params_1bar.*(1-para_aux);
[TTT,RRR,valid] = dsgesolv(params_,mspec);

[yyyyd,xxyyd,xxxxd,retcode] = varprio_dmean(TTT,RRR,valid,params_,nvar,nlags,mspec,npara,coint,cointadd,cointall);
[lnpost0,lnpy0] = feval(strcat('objfcnmh',addc),params,bounds,YYYYd,XXYYd,XXXXd,nobsd,lambdac,...
                        nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd,xxyyd,xxxxd,retcode,para_mask,coint,cointadd,cointall);


% Initialize PARANEW            
valid0 = 0;
while ~valid0
  
  paranew   = params + cc0*(sigscale*randn(npara,1));
  paranew = paranew.*(1-para_mask)+para_fix.*para_mask;
  %% Fix non-auxiliary parameters
  paranew_ = paranew.*para_aux+params_1bar.*(1-para_aux);
  [TTT,RRR,valid] = dsgesolv(paranew_,mspec);
  nstate = length(TTT);
  nshocks = size(RRR,2);
  
  valid
  
  [yyyyd,xxyyd,xxxxd,retcode] = varprio_dmean(TTT,RRR,valid,paranew_,nvar,nlags,mspec,npara,coint,cointadd,cointall);
  [postnew,likenew] = feval(strcat('objfcnmh',addc),paranew,bounds,YYYYd,XXYYd,XXXXd,nobsd,lambdac,...
                            nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd,xxyyd,xxxxd,retcode,para_mask,coint,cointadd,cointall);
  
  
  propdens  = -0.5*sigpropdim*log(2*pi) - 0.5*sigproplndet - 0.5*sigpropdim*log(cc0^2) ...
      -0.5*(paranew - params)'*sigpropinv*(paranew - params)/cc0^2;
  if postnew > -10000;
    valid0 = 1
  end
  disp('initializing')
  
end  

tic
fprintf(1,'\n Lambda = %2.3f, model = %2.3f ',[lambdacs,mspec]);
fprintf(1,'\n Time %3.2f ',[ti(I)]);
fprintf(1,'\n Peak = %2.3f',[lnpost0]);



%% prior params file 
outfile1 = strcat(spath,'/mhNIpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_);   
fid1 = fopen(outfile1,'w');		% input file (set to write)               

%% Posterior file
outfile4 = strcat(spath,'/priopost',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_);
fid4 = fopen(outfile4,'w');     % output file (set to write)   

%% MA representation: TTT (nstate x nstate) and RRR (nstate x nshocks)
outfile11 = strcat(spath,'/TTTprio',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_);    
fid11 = fopen(outfile11,'w');		% input file (set to write) 

outfile12 = strcat(spath,'/RRRprio',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_);  
fid12 = fopen(outfile12,'w');		% input file (set to write) 



%%  set initial values for simulations
Tim = 0;
eT = 0;	        
reje = 0;

for iblock = 1:nblocks  

  if iblock >1; fprintf(1,' block: %2.0f %3.2f ;',[iblock,toc/(iblock-1)]); end;

  if  (lambdacs > 0 | isnan(lambdac))
    parasimm = zeros(nsim,npara);
    postsim = zeros(nsim,1);
    TTTsim = zeros(nsim,nstate^2);
    RRRsim = zeros(nsim,nstate*nshocks);
    
  end
  
  for j = 1:nsim
    Tim = Tim+1;  	        
    
    if ( j == 1 ) & (iblock == 1)
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;     
      rej(1) = 0;

      yyyydold = yyyyd; 
      xxyydold = xxyyd; 
      xxxxdold = xxxxd; 
      

      TTTold = TTT;
      RRRold = RRR;
      validold = valid;
      
    end
    
    paranew          = paraold + cc*(sigscale*randn(npara,1));
    paranew = paranew.*(1-para_mask)+para_fix.*para_mask;
    %% Fix non-auxiliary parameters
    paranew_ = paranew.*para_aux+params_1bar.*(1-para_aux);
    [TTT,RRR,valid] = dsgesolv(paranew_,mspec);

    [yyyyd,xxyyd,xxxxd,retcode] = varprio_dmean(TTT,RRR,valid,paranew_,nvar,nlags,mspec,npara,coint,cointadd,cointall);
    [postnew,likenew] = feval(strcat('objfcnmh',addc),paranew,bounds,YYYYd,XXYYd,XXXXd,nobsd,lambdac,...
                              nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd,xxyyd,xxxxd,retcode,para_mask,coint,cointadd,cointall);
    propdens         = -0.5*sigpropdim*log(2*pi) - 0.5*sigproplndet - 0.5*sigpropdim*log(cc^2) ...
        -0.5*(paranew - paraold)'*sigpropinv*(paranew - paraold)/cc^2;
    
    r = min([1 ; exp( postnew - postold)]);      
    
    if rand(1,1) < r;
      % Accept proposed jump
      
      parasim = paranew;
      
      paraold = paranew;
      postold = postnew;
      likeold = likenew;
      
      yyyydold = yyyyd; 
      xxyydold = xxyyd; 
      xxxxdold = xxxxd;                                 
      
      TTTold = TTT;
      RRRold = RRR;
      validold = valid;
      
      parasimm(j,:) = paranew';
      postsim(j,1) = postnew;
      TTTsim(j,:) = TTT(:)';
      RRRsim(j,:) = RRR(:)';
      
    else;
      % Reject proposed jump
      parasim = paranew;
      
      reje = reje+1;
      

      yyyyd = yyyydold; 
      xxyyd = xxyydold; 
      xxxxd = xxxxdold; 
      
      TTT = TTTold;
      RRR = RRRold;
      valid = validold;
      
      parasimm(j,:) = paraold';
      postsim(j,1) = postold;
      TTTsim(j,:) = TTTold(:)';
      RRRsim(j,:) = RRRold(:)';
      
    end
    if Verbose
      fprintf(1,'\nBlock %2.2g of %2.2g',[iblock,nblocks]);
      fprintf(1,'\nSimulation step %2.0f of %2.0f',[j,nsim]);
      fprintf(1,'\nRejection r: %2.6f rej %2.2g perct %2.4f ',[r,rej(j),sum(rej)/j]);
      fprintf(1,'\nLikelihood: %4.6f Posterior: %4.6f Proposal Dens: %4.6f Estimated Score %4.6f ',[likesim(j,1),postsim(j,1),propsim(j,1),lnpy0+log(1/(sum(1/exp(likesim(1:j,1)-lnpy0))/j))]);
      fprintf(1,'\n Parameters: %2.6f',parasim);
      
    end                            
    
  end % end j=1:nsim  

  fwrite(fid1,parasimm','single');
  fwrite(fid4,postsim','single');
  fwrite(fid11,TTTsim','single');
  fwrite(fid12,RRRsim','single');

  
end;    % end simulation - given lambda and time period
fclose(fid1);
fclose(fid4);
fclose(fid11);
fclose(fid12);
toc
fprintf(1,'rejection rate = %2.4f',reje/Tim);

