close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
run startdsswfed;
!hostname;
!ps -C matlab;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varspec;

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);

%% set of parameters that are subject to this procedure
list_t0 = find(para_type == 0)
npara0 = length(list_t0)

para_names{list_t0}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,mspec);
Iseq = find(ti == dates);
I = Iseq;
size(YYall)
%%%% presample
nlagsc2 = max(nlagsc,1);
T0_p = nlagsc2;
[YYall_p,XXall_p,ti_p,nobsall_p] = loaddata(dataset,fpath,T0_p,Tend,coint,cointadd,cointall,nvar,nlagsc2,mspec);
I_p = find(ti_p == dates);
dataset = num2str(10*dataset);

if stime                           
  YY = YYall(I-stime+1:I,:);
  XX = XXall(I-stime+1:I,:);             
  tiI = ti(I-stime+1:I);  
else
  YY     = YYall(1:I,:);
  XX     = XXall(1:I,:);                   
  tiI = ti(1:I);
end
nobs = size(YY,1);

YYYY = YY'*YY;
XXYY = XX'*YY;
XXXX = XX'*XX;

MeY_0 = mean(YY,1)';
VaY_0 = cov(YY,1);
stdY_0 = sqrt(diag(VaY_0));
CorY_0 = VaY_0./(stdY_0*stdY_0');
ACorY_0 = zeros(nvar,1);
for ii = 1:nvar;
  ACorY_0(ii) = corr(YY(:,ii),XX(:,1+cointall+ii));
end
stats_0 = [MeY_0' stdY_0' ACorY_0' CorY_0(:)'];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
if isempty(mspec_ref)
  priorfile = strcat('pri',lprior)
elseif ~isempty(mspec_ref)
  priorfile = strcat('pri',lprior)
end
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);

pmean = pmean.*(1-para_mask)+para_fix.*para_mask;

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);

%%% use first part of lprior only
lprior = lprior(1)

marglh = 'marglh';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set date
I = find(ti == dates);

%% prior params file 
infile0 = strcat(spath,'/mhprioparat0',lmodel,lprior,dataset,'Mode')

lmodel00    = lmodel;%
lmodel00 = strcat('m',num2str(106)); 
%infile0 = strcat(spath,'/mhpriopara',lmodel00,lprior,dataset,'Mode')

if exist(infile0,'file');
  fid0 = fopen(infile0,'r');     % input file (set to read)   
  params = fread(fid0,[npara,1],'single');
  fclose(fid0);
else
  params = pmean;
end


%% redefine para_mask (and para_fix) to include type ~= 0
para_mask0 = para_mask + (para_type ~= 0).*(1-para_mask);
pshape0 = pshape.*(1-para_mask0);
para_fix0 = para_fix;

params = params.*(1-para_mask0)+para_fix0.*para_mask0;

CH = menu('Re -  Maximize the posterior density?','Yes','No');

if CH == 1
  %%     Maximize the posterior density
  x0= invtrans(params,trspec);%+.0005*randn(npara,1);
  H0= eye(npara)*1E-4;
  nit = input('number of iterations?'); 

  crit= 1E-16;
  MIN = 1;
  
  [fh,xh,g,H,itct,fcount,retcode] = csminwel('objfcnpriostst',x0,H0,[],crit,nit,...
                                             mspec,npara,trspec,pmean,pstdd,pshape0,para_mask0,para_fix0,MIN);

  xh = real(xh);
  params = trans(xh,trspec);
  params = params.*(1-para_mask0)+para_fix0.*para_mask0;
  disp('move on?');	pause;

  outfile0 = strcat(spath,'/mhprioparat0',lmodel,lprior,dataset,'Mode');
  fid0 = fopen(outfile0,'w');     % output file (set to write)   
  fwrite(fid0,params','single');
  fclose(fid0);
  
end

%% prior params file (first block has already been thrown out)
infile01 = strcat(spath,'/mhprioparat0',lmodel,lprior,dataset,'Hessian');   
if ~exist(infile01,'file');
  CH = 1;
else
  fid01 = fopen(infile01,'r');     % input file (set to read)   
  hessian = fread(fid01,[npara,npara],'single');
  fclose(fid01);
  CH = menu('Re-Compute hessian?','Yes','No','just finish');
end
if CH == 1
  MIN = 0;
  hessian = hessizero('objfcnpriostst',[params,para_mask0],1,...
                      mspec,npara,trspec,pmean,pstdd,pshape0,para_mask0,para_fix0,MIN);
  disp('proceed?');     
  %pause
elseif CH == 3
  return;
end;
outfile01 = strcat(spath,'/mhprioparat0',lmodel,lprior,dataset,'Hessian');   

fid01 = fopen(outfile01,'w');     % output file (set to write)   
fwrite(fid01,hessian(:),'single');
fclose(fid01);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set up Metropolis Hastings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DOD = 1%input('\n\n Skip Metropolis Hastings? (0 = just skip, else produce draws)');
if DOD

  rankhhm = npara;
  % Create Inverse by Singular Value Decomposition
  [u,s,v] = svd(hessian);
  md = min(size(s)); 
  bigev = find(diag(s(1:md,1:md))>1e-6);
  sigpropdim = length(bigev);

  sigproplndet = 0;
  for i = 1:npara
    
    if i > sigpropdim
      s(i,i) = 0;
    else
      
      s(i,i)    = 1/s(i,i);
      
      sigproplndet = sigproplndet+log(s(i,i));  
      
    end
  end


  invhhm  = u*s*u';
  sigscale = u*sqrt(s);
  sigpropinv  =  hessian; 
  clear hessian*;


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%  Configure the Metropolis Algorithm
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  cc0     = 0.01;
  cc      = 0.1;

  ntimes = 1

  Verbose = 0;

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  stmoms;
  
  
  statname = {'POST';'LIKE';'PROP';'PROPPOST';'REJECT'};

  pred = [];
  vpred = [];

  %% check bounds
  ind1 = (params > bounds(:,2));
  ind1(find(para_mask0)) = [];
  ind2 = (params < bounds(:,1));
  ind2(find(para_mask0)) = [];
  if any(ind1) | any(ind2)
    lnpost0 = -1E6;
    disp('mode out of bounds'); 
    return;

  else

    lnprio  = priodens(params,pmean,pstdd,pshape0);

    [M_vec] = getparaM(params,mspec)
    lnprioM = - sum( ( (M_vec-M_mean).^2 )./M_var );

    lnpost0  = real(lnprio) + real(lnprioM);
  end

  % Initialize PARANEW            
  valid0 = 0;
  while ~valid0
    
    paranew   = params + cc0*(sigscale*randn(npara,1));
    paranew = paranew.*(1-para_mask0)+para_fix0.*para_mask0;

    %% check bounds
    ind1 = (paranew > bounds(:,2));
    ind1(find(para_mask0)) = [];
    ind2 = (paranew < bounds(:,1));
    ind2(find(para_mask0)) = [];
    if any(ind1) | any(ind2)
      postnew = -1E6;
    else
      lnprionew  = priodens(paranew,pmean,pstdd,pshape0);

      [M_vec] = getparaM(paranew,mspec);
      lnprioMnew = - sum( ( (M_vec-M_mean).^2 )./M_var );

      postnew =  real(lnprionew) + real(lnprioMnew);
    end

    propdens  = -0.5*sigpropdim*log(2*pi) - 0.5*sigproplndet - 0.5*sigpropdim*log(cc0^2) ...
        -0.5*(paranew - params)'*sigpropinv*(paranew - params)/cc0^2;
    if postnew > -10000;
      valid0 = 1
    end
    disp('initializing')
    
  end  

  tic
  fprintf(1,'\n model = %2.3f ',[mspec]);
  fprintf(1,'\n Time %3.2f ',[ti(I)]);
  fprintf(1,'\n Peak = %2.3f',[lnpost0]);



  %% prior params file 
  outfile1 = strcat(spath,'/mhprioparat0',lmodel,lprior,dataset);
  fid1 = fopen(outfile1,'w');		% input file (set to write)               

  %% Posterior file
  outfile4 = strcat(spath,'/priopostt0',lmodel,lprior,dataset);
  fid4 = fopen(outfile4,'w');     % output file (set to write)   


  %%  set initial values for simulations
  Tim = 0;
  eT = 0;	        
  reje = 0;

  for iblock = 1:nblocks  

    if iblock >1; fprintf(1,' block: %2.0f %3.2f ;',[iblock,toc/(iblock-1)]); end;


    parasimm = zeros(nsim,npara0);
    postsim = zeros(nsim,1);
    
    rej = 0;
    bds = [0 0];
    for j = 1:nsim*ntimes
      % for j = 1:nsim
      Tim = Tim+1;  	        
      
      if ( j == 1 ) & (iblock == 1)
        postold      = postnew;
        paraold      = paranew;     
        rej(1) = 0;
        
      end
      
      paranew          = paraold + cc*(sigscale*randn(npara,1));
      paranew = paranew.*(1-para_mask0)+para_fix0.*para_mask0;

      
      %% check bounds
      ind1 = (paranew > bounds(:,2));
      %find1 = find(ind1);
      ind1(find(para_mask0)) = [];
      ind2 = (paranew < bounds(:,1));
      %find2 = find(ind2);
      ind2(find(para_mask0)) = [];
      if any(ind1) | any(ind2)
        postnew = -1E6;
      else
        lnprionew  = priodens(paranew,pmean,pstdd,pshape0);

        [M_vec] = getparaM(paranew,mspec);
        lnprioMnew = - sum( ( (M_vec-M_mean).^2 )./M_var );

        postnew =  real(lnprionew) + real(lnprioMnew);
      end

      
      propdens         = -0.5*sigpropdim*log(2*pi) - 0.5*sigproplndet - 0.5*sigpropdim*log(cc^2) ...
          -0.5*(paranew - paraold)'*sigpropinv*(paranew - paraold)/cc^2;
      
      r = min([1 ; exp( postnew - postold)]);      
      
      ra = rand(1,1);
      
      if ra  < r;
        % Accept proposed jump
        
        parasim = paranew;      
        paraold = paranew;
        postold = postnew;
        
        if (j/ntimes) == round(j/ntimes)
          parasimm(j/ntimes,:) = paranew(list_t0)';
          postsim(j/ntimes,1) = postnew;
        end      
      else;
        
        % Reject proposed jump
        parasim = paranew;
        
        reje = reje+1;
        rej = rej+1;
        
        if       postnew <= -1E6
          bds(1) = bds(1)+1;
        else
          bds(2) = bds(2)+1;
        end
        
        
        
        if (j/ntimes) == round(j/ntimes)
          parasimm(j/ntimes,:) = paraold(list_t0)';
          postsim(j/ntimes,1) = postold;
        end
        
      end
      if Verbose
        fprintf(1,'\nBlock %2.2g of %2.2g',[iblock,nblocks]);
        fprintf(1,'\nSimulation step %2.0f of %2.0f',[j,nsim]);
        fprintf(1,'\nRejection r: %2.6f rej %2.2g perct %2.4f ',[r,reje/Tim]);
        %    fprintf(1,'\n Parameters: %2.6f',parasim(26:end));
        
      end                            
      
    end % end j=1:nsim  
    fprintf(1,'\n block rejection rate = %2.4f, overall rejection rate = %2.4f',[rej/j reje/Tim]);

    fwrite(fid1,parasimm','single');
    fwrite(fid4,postsim','single');
    
  end;    % end simulation - given lambda and time period
  fclose(fid1);
  fclose(fid4);
  toc
  fprintf(1,'rejection rate = %2.4f',reje/Tim);

end


infile10 = strcat(spath,'/mhprioparat0',lmodel,lprior,dataset);
fid10 = fopen(infile10,'r');		% input file (set to read)   
num10 = nblocks*nsim*npara0*4;            % number of bites per file
numb10 = nburn*npara0*4;                  % number of bites to discard
status = fseek(fid10,numb10,'bof');       % discard nburn 

para0 = fread(fid10,[npara0,inf],'single')';
fclose(fid10);



ndraws = size(para0,1)

%% discard indeterminacy
%para(find(boundsim),:) = [];


para0hat = mean(para0,1)';
para0sig = cov(para0,1);
para0bands = hpdint(para0,percent,1)';


%% save mean
outfile0 = strcat(spath,'/mhprioparat0',lmodel,lprior,dataset,'Mean')
fid0 = fopen(outfile0,'w');     % output file (set to write)   
fwrite(fid0,para0hat','single');
fclose(fid0);
 

outfile00 = strcat(fpath,lmodel,lprior,dataset,'PrioMomt0.tex');		% output file name 
fid00 = fopen(outfile00,'w');		% output file (set to write)

fprintf(fid00,'\n \\documentclass[12pt]{article} \n \\usepackage[dvips]{color}\n ');
fprintf(fid00,'\n \\begin{document}');

fprintf(fid00,'\n \\documentclass{article}');
fprintf(fid00,'\n \\begin{document}');
fprintf(fid00,'\n \\setlength{\\tabcolsep}{.7mm}');


fprintf(1,'%4.99s  ','Mean');
for i0 = 1:npara0;
  i = list_t0(i0);
  fprintf(1,'\n %4.10s  ',para_names{i});
  fprintf(1,' %8.4f  ',para0hat(i0,:));
end    

outmat = [para0hat,sqrt(diag(para0sig)),para0bands];

fprintf(fid00,'\n');


colnames = {'Parameter' 'Support '  'Density '  'Prior Mean ' 'Prior Stdd ' strcat(num2str(100*percent),'\% LB') ...
            strcat(num2str(100*percent),'\% UB') };

[nrows,ncols] = size(outmat);

fprintf(fid00,'PARAMETERS \n $$');
fprintf(fid00,'\n \\vspace{.2cm} \n \\begin{array}{|@{\\hspace*{.5cm}} l @{\\hspace*{.5cm}}|@{\\hspace*{-.2cm}}c@{\\hspace*{-.2cm}}|@{\\hspace*{-.2cm}}c@{\\hspace*{-.2cm}}|');
FORMAT = '  ';
for j = 1:ncols
  fprintf(fid00,' @{\\hspace*{.5cm}} c'); 
  if j == 1
    FORMAT = strcat(FORMAT,' &  %8.3f  '); 
    % elseif (j == ncols) | (j == (ncols-1))
    %	FORMAT = strcat(FORMAT,' & \\mbox{{\\scriptsize $%8.3f$ }} '); 
  else
    FORMAT = strcat(FORMAT,' & %8.3f '); 
  end

end

fprintf(fid00,'|}\\hline');
FORMAT = strcat(FORMAT,' \\\\');  

fprintf(fid00,'\n  ');
fprintf(fid00,' %4.99s  & ',colnames{:});		
fprintf(fid00,'	\\\\ \\hline '); 

for i0 = 1:nrows
  i = list_t0(i0);
  fprintf(fid00,'\n %4.99s & ',para_names{i});
  if pshape0(i) == 1
    fprintf(fid00,'[0,1) & \\mbox{Beta} ');	
  elseif pshape0(i) == 2
    fprintf(fid00,'R^{+} & \\mbox{Gamma} ');	
  elseif pshape0(i) == 3
    fprintf(fid00,' R & \\mbox{\\cal N} ');	
  elseif pshape0(i) == 4
    fprintf(fid00,'R^{+} & \\mbox{InvGamma} ');	
  elseif pshape0(i) == 0
    fprintf(fid00,' &  ');	    
  end
  fprintf(fid00,FORMAT,outmat(i0,:));

end   
fprintf(fid00,' \\hline');
fprintf(fid00,'\n \\end{array} $$');        
fprintf(fid00,'\n \\end{document}');


fclose(fid00);		% close output file


