close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

run startdsswfed;

!hostname;
!ps -C matlab;
rand('state',sum(100*clock))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varspec;

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);


if isempty(mspec_ref)
  lmodel_ref = [];
  nobsdrat_ = [];
  lambdac_ = [];
  nlagsc_ = [];
  n_s_ = [];
  disp('NO reference model set in varspec.m')
else
  lmodel_ref = strcat('m',num2str(mspec_ref));
  nobsdrat_ = num2str(100*nobsdrat);
  lambdac_ = [];
  n_s_ = num2str(n_s);
  if nlagsc == 4
    nlagsc_ = [];
  else
    nlagsc_ = num2str(nlagsc);
  end  
  
end
stup_ = num2str(stup);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,mspec);

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat('pri',lprior)
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);

pmean = pmean.*(1-para_mask)+para_fix.*para_mask;

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setup for different parameters types
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set of parameters 
list_t0 = find(para_type == 0);
npara0 = length(list_t0);
para_mask0 = para_mask + (para_type ~= 0).*(1-para_mask);
pshape0 = pshape.*(1-para_mask0);
para_fix0 = para_fix;

list_t1 = find(para_type == 1);
npara1 = length(list_t1);
para_mask1 = para_mask + (para_type ~= 1).*(1-para_mask);
pshape1 = pshape.*(1-para_mask1);
para_fix1 = para_fix;

list_t2 = find(para_type == 2);
npara2 = length(list_t2);
para_mask2 = para_mask + (para_type ~= 2).*(1-para_mask);
pshape2 = pshape.*(1-para_mask2);
para_fix2 = para_fix;
%% recover the mean for type 0 and type 1 params
if ~isempty(list_t0)
  infile000 = strcat(spath,'/mhprioparat0',lmodel,lprior(1),dataset,'Mean');
  fid000 = fopen(infile000,'r');     % input file (set to read)   
  pmean0 = fread(fid000,[npara0,1],'single');
  fclose(fid000);
  
  para_fix2(list_t0) = pmean0;  
end
if ~isempty(list_t1)
  infile001 = strcat(spath,'/mhprioparat1',lmodel,lprior,dataset,lmodel_ref,stup_,'Mean');
  fid001 = fopen(infile001,'r');     % input file (set to read)   
  pmean1 = fread(fid001,[npara1,1],'single');
  fclose(fid001);
  
  para_fix2(list_t1) = pmean1;
end

%% put them all together
para_mask = [para_mask0 para_mask1 para_mask2 para_mask];
pshape = [pshape0 pshape1 pshape2 pshape];
para_fix = [para_fix0 para_fix1 para_fix2 para_fix];


%% moments from data
stmoms;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% doing rolling sample estimation?
ROLL = 0;
SAVEALL = 1;
DOFORECAST = 0;
fprintf(1,'\n\n ROLL: %2.0f, SAVEALL: %2.0f, DOFORECAST: %2.0f\n',[ROLL,SAVEALL,DOFORECAST]);
start_period = stime;                               % minimum # quarters to compute VAR estimates 

%mkdir(spath,'/2/');
%spath = strcat(spath,'/1/');
%start_period = nobsall-qahead+1-3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% CHAnGE

trial_period = nobsall-start_period-qahead+1;      % # quarters to compute RMSE


%%  Configure the Metropolis Algorithm
cc0     = 0.01;
cc      = 0.10;


Verbose = 0;


date_q = (1:1:qahead)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do Lambda loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fixparam = fixparamseq;          
L = 0;
for lambdas = lambdaseq     
  
  L = L+1;        
  

  Iseq = find(ti == dates);

  for I = Iseq;

    if DOFORECAST
      Fh = zeros(qahead,nvar);           % forecasts errors 
      Yh = zeros(qahead,nvar);           % forecasts 
    end

    fprintf(1,'\n\nRMSE computation: lambda = %2.3f; date: %2.2f ',[lambdas,ti(I)]);
    %	        fprintf(1,' %8.98s;',fixparam);
    
    if stime                           
      YY = YYall(I-stime+1:I,:);
      XX = XXall(I-stime+1:I,:);             
      tiI = ti(I-stime+1:I);
    else
      YY     = YYall(1:I,:);
      XX     = XXall(1:I,:);                   
      tiI = ti(1:I);
    end
    
    %% Define initial Y's and cointegration vector for DSGE model estimation
    %% ignore cointadd
    
    %% YY0: lagged growth rates
    YY0 = zeros(nlags,size(YY,2));
    for lagind = 1:nlags
      YY0(nlags-lagind+1,:) = XX(1,1+cointall+(lagind-1)*nvar+1:1+cointall+lagind*nvar);
    end
    
    %% YYcoint0: lagged cointegration vector
    if coint > 0            
      YYcoint0 = XX(1,1+cointadd:cointall);
      for lagind = 1:(nlags-1)
        YYcoint0 = YYcoint0-XX(1,1+cointall+(lagind-1)*nvar+1:1+cointall+lagind*nvar)*cointvec';
      end
    else
      YYcoint0 = [];               
    end
    
    if I+qahead <= nobsall
      YYfut  = YYall(I+1:I+qahead,:);
      XXpred = XXall(I+1,:);
    elseif I == nobsall
      YYfut  = NaN*zeros(qahead,nvar);
      XXpred = XXall(end,:);
    elseif (I < nobsall) & (I+qahead > nobsall)
      Ii = nobsall-I
      YYfut  = [YYall(I+1:I+Ii,:);NaN*zeros(qahead-Ii,nvar)];
      XXpred = XXall(end-Ii,:);        
    end
    
    
    nobs = size(YY,1);
    
    YYYY = YY'*YY;
    XXYY = XX'*YY;
    XXXX = XX'*XX;

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Reference Model for Prior Second Moments
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if ~isempty(mspec_ref)
      nobsd = nobsdrat*nobs;

      POPUL = 0
      lprior_ref = '';
      lmodel_ref    = strcat('m',num2str(mspec_ref));
      

      
      if POPUL 
        POP_ = '';
      else
        POP_ = 'P';
      end

      infile02 = strcat(spath,'/yyyy',lmodel_ref,lprior_ref,dataset,nlagsc_,n_s_,POP_,'Bench');
      fid02 = fopen(infile02,'r');		% input file (set to write)   
      infile03 = strcat(spath,'/xxyy',lmodel_ref,lprior_ref,dataset,nlagsc_,n_s_,POP_,'Bench');
      fid03 = fopen(infile03,'r');		% input file (set to write)   
      infile04 = strcat(spath,'/xxxx',lmodel_ref,lprior_ref,dataset,nlagsc_,n_s_,POP_,'Bench');
      fid04 = fopen(infile04,'r');		% input file (set to write)   

      k_ = nlagsc*nvar;

      YYYYd = fread(fid02,[nvar nvar],'single');
      XXYYd = fread(fid03,[k_  nvar],'single');
      XXXXd = fread(fid04,[k_ k_],'single');

      fclose(fid02);
      fclose(fid03);
      fclose(fid04);

      %lprior_bar    = num2str(2);
      %infile0 = strcat(spath,'/mhpriopara',lmodel,lprior_bar,dataset,[],[],'Mean');
      %fid0 = fopen(infile0,'r');     % input file (set to read)   
      %params_1bar = fread(fid0,[npara,1],'single');
      %fclose(fid0);  
      %params_1bar = params_1bar.*(1-para_mask)+para_fix.*para_mask;



      YYYYd = YYYYd*nobsd;
      XXYYd = XXYYd*nobsd;
      XXXXd = XXXXd*nobsd;

    elseif isempty(mspec_ref)
      
      nobsd = [];
      YYYYd = [];
      XXYYd = [];
      XXXXd = [];
      
    end
    
    
    lambda = lambdas*nobs;
    
    if isinf(lambdas)
      add = 'inf';
    elseif isnan(lambdas)
      add = 'dsge';
    else
      add = '';
    end

    addc = 'inf';
    
    marglh = 'marglh';

    
    if (lambdas > 0) | isinf(lambdas) | isnan(lambdas)
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% find posterior mode wrt dsge parameters  
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      if ~isnan(lambda) 
        disp('not programmed yet')
        return
        Ob = feval(strcat('objfcn',add),para,...
                   YYYY,XXYY,XXXX,nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,para_mask,...
                   para_fix,marglh,coint,cointadd,cointall,mspec_ref,params_1bar,YYYYd,XXYYd,XXXXd,nobsd,para_aux,lambdac,nlagsc,addc,bounds,0);
      elseif isnan(lambda) 

        Ob = feval(strcat('objfcn',add),para,YY,YY0,...
                   nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,para_mask,para_fix,marglh,...
                   coint,cointadd,cointall,YYcoint0,mspec_ref,YYYYd,XXYYd,XXXXd,nobsd,nlagsc,bounds,0);
        
      end
      format long g
      disp('Objective Fcn at Starting Value');
      disp(Ob)

      infile0 = strcat(spath,lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,'para',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),'Mode');

      lmodel0 = lmodel; %lmodel0 = strcat('m',num2str(105)); 
      stup_0 = stup_; nobsdrat_0= nobsdrat_;   lprior0 = lprior; lmodel_ref0 = lmodel_ref;     nlagsc_0 = nlagsc_;
      %stup_0 = num2str(stup);
      nobsdrat_0=num2str(10);  %lprior0 = num2str(1); lmodel_ref0 =  strcat('m',num2str(NaN));     nlagsc_0 = num2str(1);
      %stup_0 = ''; nobsdrat_0= '';   lprior0 = num2str(12); lmodel_ref0 =  '';     nlagsc_0 = '';         
      %infile0 = strcat(spath,lmodel0,lprior0,dataset,lmodel_ref0,stup_0,nobsdrat_0,lambdac_,nlagsc_0,'para',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),'Mode')
      
      if exist(infile0,'file');
        fid0 = fopen(infile0,'r');     % input file (set to read)   
        params = fread(fid0,[npara,1],'single');
        fclose(fid0);            
      else
        infile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,'Mean')
        if exist(infile0,'file');
          fid0 = fopen(infile0,'r');     % input file (set to read)   
          params = fread(fid0,[npara,1],'single');
          fclose(fid0);
        else 
          params = pmean;
        end
      end
      
      CH = 2%menu('Re -  Maximize the posterior density?','Yes','No');
      
      if CH == 1
        %%     Maximize the posterior density
        x0= invtrans(params,trspec);%+.0001*randn(npara,1);
        H0= eye(npara)*1E-4;
        nit = input('number of iterations?'); 
        crit= 1E-12;
        MIN = 1;
        if ~isnan(lambda) 
        elseif isnan(lambda) 
          [fh,xh,g,H,itct,fcount,retcode] = csminwel(strcat('objfcn',add),x0,H0,[],crit,nit,YY,YY0,...
                                                     nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,para_mask,para_fix,marglh,...
                                                     coint,cointadd,cointall,YYcoint0,mspec_ref,YYYYd,XXYYd,XXXXd,nobsd,nlagsc,bounds,MIN);
        end
        xh = real(xh);
        params = trans(xh,trspec);
        params = params.*(1-para_mask(:,end))+para_fix(:,end).*para_mask(:,end);
        disp('move on?');	pause;
        
      end
      
      CHK = 0; %% do you want to check para values, irfs, etc
      if CHK
        checkml;			
        if  CLOSE == 1
          return;
        end
      end %% id if CHK	
      
      outfile0 = strcat(spath,lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,'para',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),'Mode');

      fid0 = fopen(outfile0,'w');     % output file (set to write)   
      fwrite(fid0,params','single');
      fclose(fid0);
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% find Hessian corresponding to posterior mode  
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      

      infile01 = strcat(spath,lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,'hessian',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),'Mode');
      %lambdas00 = NaN;         infile01 = strcat(spath,lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,'hessian',num2str(cointall),st,num2str(100*lambdas00),num2str(100*ti(I)),num2str(nlags),'Mode');
      
      if exist(infile01,'file');
        fid01 = fopen(infile01,'r');     % input file (set to read)   
        hessian = fread(fid01,[npara,npara],'single');
        fclose(fid01);
      end
      CH = 2%menu('Re-Compute hessian?','Yes','No','just finish');        
      
      if CH == 1
        MIN = 0;
        if ~isnan(lambda) 
        elseif isnan(lambda) 
          hessian = hessizero(strcat('objfcn',add),[params,para_mask(:,end)],1,YY,YY0,...
                              nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,para_mask,para_fix,marglh,...
                              coint,cointadd,cointall,YYcoint0,mspec_ref,YYYYd,XXYYd,XXXXd,nobsd,nlagsc,bounds,MIN);
        end
        disp('proceed?');     
      elseif CH == 3
        return;
      end;
      outfile01 = strcat(spath,lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,'hessian',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),'Mode');
      fid01 = fopen(outfile01,'w');     % output file (set to write)   
      fwrite(fid01,hessian(:),'single');
      fclose(fid01);


      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% set up Metropolis Hastings
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      rankhhm = npara;
      % Create Inverse by Singular Value Decomposition
      [u,s,v] = svd(hessian);
      md = min(size(s)); 
      bigev = find(diag(s(1:md,1:md))>1e-12);
      sigpropdim = length(bigev);
      
      sigproplndet = 0;
      for i = 1:npara
        
        if i > sigpropdim
          s(i,i) = 0;
        else
          
          s(i,i)    = 1/s(i,i);
          sigproplndet = sigproplndet+log(s(i,i));  
          
        end
      end
      invhhm  = u*s*u';
      sigscale = u*sqrt(s);
      sigpropinv  =  hessian; 
      clear hessian*;
      
      if length(bigev) ~= (npara-length(find(para_mask(:,end))))

        length(bigev)
        npara
        length(find(para_mask(:,end)))
        disp('problem -- shutting down dimensions')
mm
      end

      statname = {'POST';'LIKE';'PROP';'PROPPOST';'REJECT'};
      
      pred = [];
      vpred = [];
      
      paranew = params;
      paranew = paranew.*(1-para_mask(:,end))+para_fix(:,end).*para_mask(:,end);

      %% check bounds
      ind1 = (paranew > bounds(:,2));
      ind1(find(para_mask(:,end))) = [];
      ind2 = (paranew < bounds(:,1));
      ind2(find(para_mask(:,end))) = [];
      if any(ind1) | any(ind2)
        lnpost0 = -1E6;
        return
      else
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% likelihood 
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        lnlik0 = 0; %% this is the likelihood part
        [TTT,RRR,valid] = dsgesolv(paranew,mspec);
        
        if valid < 1;
          lnlik0 = -10000; %% this is the dummy obs part
          
        else
          retcode = valid;
          [lnlik0,zend,ZZ,DD,QQ] = feval(strcat('likmh',add),paranew,bounds,YY,YY0,nobs,lambda,...
                                         nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape(:,end),TTT,RRR,valid,para_mask(:,end),...
                                         coint,cointadd,cointall,YYcoint0);    
        end


        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% Evaluate the Prior distribution
        %%% type 0 parameters
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        lnprio0 = 0;
        if any(pshape(:,1))
          %% this line is redundant
          para0 = paranew.*(1-para_mask(:,1))+para_fix(:,1).*para_mask(:,1);

          lnprio0 = priodens(para0,pmean,pstdd,pshape(:,1));
          
          %% computes moments from model
          M_vec = getparaM(para0,mspec);
          lnprioM = - sum( ( (M_vec-M_mean).^2 )./M_var );
          
          lnprio0 = lnprioM+lnprio0;
        end


        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% Evaluate the Prior distribution
        %%% type 1 parameters
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        lnprio1 = 0;
        if any(pshape(:,2))
          %% this line is redundant
          para1 = paranew.*(1-para_mask(:,2))+para_fix(:,2).*para_mask(:,2);
          lnprio1 = priodens(para1,pmean,pstdd,pshape(:,2));
        end

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% Evaluate the Prior distribution
        %%% type 2 parameters
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        lnprio2 = 0;
        if any(pshape(:,3))
          para2 = paranew.*(1-para_mask(:,3))+para_fix(:,3).*para_mask(:,3);

          lnprio2 = priodens(para2,pmean,pstdd,pshape(:,3));

          lnprioD = 0;
          if ~isempty(mspec_ref)

            %% Solve the model and evaluate the (dummy) likelihood
            [TTT,RRR,valid] = dsgesolv(para2,mspec);
            [yyyyd,xxyyd,xxxxd,retcode] = varprio_dmean(TTT,RRR,valid,para2,nvar,nlagsc,mspec,npara,coint,cointadd,cointall);
            lnprioD = feval('objfcnmhinf',para2,YYYYd,XXYYd,XXXXd,nobsd,...
                            nlagsc,nvar,mspec,npara,trspec,pmean,pstdd,pshape(:,3),yyyyd,xxyyd,xxxxd,retcode,para_mask(:,3),coint,cointadd,cointall);
          end
          lnprio2 = lnprioD+lnprio2;
        end



        lnpost0 =  lnlik0 + real(lnprio0) + real(lnprio1) + real(lnprio2);
      end

      nstate = length(TTT);
      nshocks = size(RRR,2);

      % Initialize PARANEW            
      valid0 = 0;
      while ~valid0
        
        paranew          = params + cc*(sigscale*randn(npara,1));
        paranew = paranew.*(1-para_mask(:,end))+para_fix(:,end).*para_mask(:,end);

        %% check bounds
        ind1 = (paranew > bounds(:,2));
        ind1(find(para_mask(:,end))) = [];
        ind2 = (paranew < bounds(:,1));
        ind2(find(para_mask(:,end))) = [];
        if any(ind1) | any(ind2)
          postnew = -1E6;
          return
        else
          
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          %%% likelihood 
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          likenew = 0; %% this is the likelihood part
          [TTT,RRR,valid] = dsgesolv(paranew,mspec);
          
          if valid < 1;
            likenew = -10000; %% this is the dummy obs part
          else
            retcode = valid;
            [likenew,zend,ZZ,DD,QQ] = feval(strcat('likmh',add),paranew,bounds,YY,YY0,nobs,lambda,...
                                            nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape(:,end),TTT,RRR,valid,para_mask(:,end),...
                                            coint,cointadd,cointall,YYcoint0);    
          end

          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          %%% Evaluate the Prior distribution
          %%% type 0 parameters
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          lnprio0 = 0;
          if any(pshape(:,1))
            %% this line is redundant
            para0 = paranew.*(1-para_mask(:,1))+para_fix(:,1).*para_mask(:,1);

            lnprio0 = priodens(para0,pmean,pstdd,pshape(:,1));
            
            %% computes moments from model
            M_vec = getparaM(para0,mspec);
            lnprioM = - sum( ( (M_vec-M_mean).^2 )./M_var );
            
            lnprio0 = lnprioM+lnprio0;
          end


          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          %%% Evaluate the Prior distribution
          %%% type 1 parameters
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          lnprio1 = 0;
          if any(pshape(:,2))
            %% this line is redundant
            para1 = paranew.*(1-para_mask(:,2))+para_fix(:,2).*para_mask(:,2);
            lnprio1 = priodens(para1,pmean,pstdd,pshape(:,2));
          end

          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          %%% Evaluate the Prior distribution
          %%% type 2 parameters
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          lnprio2 = 0;
          if any(pshape(:,3))
            para2 = paranew.*(1-para_mask(:,3))+para_fix(:,3).*para_mask(:,3);

            lnprio2 = priodens(para2,pmean,pstdd,pshape(:,3));

            lnprioD = 0;
            if ~isempty(mspec_ref)

              %% Solve the model and evaluate the (dummy) likelihood
              [TTT,RRR,valid] = dsgesolv(para2,mspec);
              [yyyyd,xxyyd,xxxxd,retcode] = varprio_dmean(TTT,RRR,valid,para2,nvar,nlagsc,mspec,npara,coint,cointadd,cointall);
              lnprioD = feval('objfcnmhinf',para2,YYYYd,XXYYd,XXXXd,nobsd,...
                              nlagsc,nvar,mspec,npara,trspec,pmean,pstdd,pshape(:,3),yyyyd,xxyyd,xxxxd,retcode,para_mask(:,3),coint,cointadd,cointall);
            end
            lnprio2 = lnprioD+lnprio2;
          end



          lnprionew = real(lnprio0) + real(lnprio1) + real(lnprio2);
          postnew =  likenew + lnprionew;
          
        end
        
        
        propdens  = -0.5*sigpropdim*log(2*pi) - 0.5*sigproplndet - 0.5*sigpropdim*log(cc0^2) ...
            -0.5*(paranew - params)'*sigpropinv*(paranew - params)/cc0^2;
        if postnew > -10000;
          valid0 = 1
        end
        disp('initializing')
        
      end  
    end
    
    tic
    fprintf(1,'\n Lambda = %2.3f',[lambdas]);
    fprintf(1,' model = %2.3f , becnhmark = %2.3f',[mspec,mspec_ref]);
    fprintf(1,'\n Time %3.2f ',[ti(I)]);
    fprintf(1,'\n Peak = %2.3f',[lnpost0]);
    
    if  (lambdas > 0 | isnan(lambda))

      infofile = strcat(spath,'/info',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));        
      if lambdas ~= 0;         save(infofile,'nstate','nshocks');        end;
      
      %% open write files
      %% params file
      outfile1 = strcat(spath,'/mhpara',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));      
      fid1 = fopen(outfile1,'w');     % output file (set to write)   
                                      %% Posterior file
      outfile4 = strcat(spath,'/post',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));        
      fid4 = fopen(outfile4,'w');     % output file (set to write)   
      
      outfile40 = strcat(spath,'/lik',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));        
      fid40 = fopen(outfile40,'w');     % output file (set to write)   
      
      outfile41 = strcat(spath,'/lnprioD',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));        
      fid41 = fopen(outfile41,'w');     % output file (set to write)   
      
      if SAVEALL

        %% MA representation: TTT (nstate x nstate)
        outfile11 = strcat(spath,'/TTT',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    
        fid11 = fopen(outfile11,'w');       % output file (set to write)   
        
        %% MA representation: RRR (nstate x nshocks)
        outfile12 = strcat(spath,'/RRR',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    
        fid12 = fopen(outfile12,'w');       % output file (set to write)   
      end
    end
    %%  set initial values for simulations
    Tim = 0;
    eT = 0;	        
    reje = 0;
    
    for iblock = 1:nblocks  

      if iblock >1; fprintf(1,' block: %2.0f %3.2f ;',[iblock,toc/(iblock-1)]); end;

      if  (lambdas > 0 | isnan(lambda))
        parasimm = zeros(nsim,npara);
        likesim = zeros(nsim,1);
        postsim = zeros(nsim,1);
        lnpriosim = zeros(nsim,1);
        rej     = zeros(nsim,1);
        propsim = zeros(nsim,1);
        proppostsim = zeros(nsim,1);
        TTTsim = zeros(nsim,nstate^2);
        RRRsim = zeros(nsim,nstate*nshocks);
        
      end
      
      for j = 1:nsim
        Tim = Tim+1;  	        
        if  (lambdas > 0 | isnan(lambda)) & ~fixparam
          
          
          if ( j == 1 ) & (iblock == 1)
            postold      = postnew;
            likeold      = likenew;
            lnprioold      = lnprionew;            
            paraold      = paranew;     
            rej(1) = 0;

            predold = pred;
            vpredold = vpred;
            zendold = zend;
            ZZold = ZZ;
            DDold = DD;
            QQold = QQ;
            
            TTTold = TTT;
            RRRold = RRR;
            validold = valid;
            
          end
          
          paranew          = paraold + cc*(sigscale*randn(npara,1));
          paranew = paranew.*(1-para_mask(:,end))+para_fix(:,end).*para_mask(:,end);

          %% check bounds
          ind1 = (paranew > bounds(:,2));
          ind1(find(para_mask(:,end))) = [];
          ind2 = (paranew < bounds(:,1));
          ind2(find(para_mask(:,end))) = [];      
          if any(ind1) | any(ind2)
            postnew = -1E6;            
          else
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% likelihood 
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            likenew = 0; %% this is the likelihood part
            [TTT,RRR,valid] = dsgesolv(paranew,mspec);
            
            if valid < 1;
              likenew = -10000; %% this is the dummy obs part
            else
              retcode = valid;
              [likenew,zend,ZZ,DD,QQ] = feval(strcat('likmh',add),paranew,bounds,YY,YY0,nobs,lambda,...
                                              nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape(:,end),TTT,RRR,valid,para_mask(:,end),...
                                              coint,cointadd,cointall,YYcoint0);    
            end

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% Evaluate the Prior distribution
            %%% type 0 parameters
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            lnprio0 = 0;
            if any(pshape(:,1))
              %% this line is redundant
              para0 = paranew.*(1-para_mask(:,1))+para_fix(:,1).*para_mask(:,1);

              lnprio0 = priodens(para0,pmean,pstdd,pshape(:,1));
              
              %% computes moments from model
              M_vec = getparaM(para0,mspec);
              lnprioM = - sum( ( (M_vec-M_mean).^2 )./M_var );
              
              lnprio0 = lnprioM+lnprio0;
            end


            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% Evaluate the Prior distribution
            %%% type 1 parameters
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            lnprio1 = 0;
            if any(pshape(:,2))
              %% this line is redundant
              para1 = paranew.*(1-para_mask(:,2))+para_fix(:,2).*para_mask(:,2);
              lnprio1 = priodens(para1,pmean,pstdd,pshape(:,2));
            end

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% Evaluate the Prior distribution
            %%% type 2 parameters
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            lnprio2 = 0;
            if any(pshape(:,3))
              para2 = paranew.*(1-para_mask(:,3))+para_fix(:,3).*para_mask(:,3);

              lnprio2 = priodens(para2,pmean,pstdd,pshape(:,3));

              lnprioD = 0;
              if ~isempty(mspec_ref)

                %% Solve the model and evaluate the (dummy) likelihood
                [TTT,RRR,valid] = dsgesolv(para2,mspec);
                [yyyyd,xxyyd,xxxxd,retcode] = varprio_dmean(TTT,RRR,valid,para2,nvar,nlagsc,mspec,npara,coint,cointadd,cointall);
                lnprioD = feval('objfcnmhinf',para2,YYYYd,XXYYd,XXXXd,nobsd,...
                                nlagsc,nvar,mspec,npara,trspec,pmean,pstdd,pshape(:,3),yyyyd,xxyyd,xxxxd,retcode,para_mask(:,3),coint,cointadd,cointall);
              end
              lnprio2 = lnprioD+lnprio2;
            end


            lnprionew = real(lnprio0) + real(lnprio1) + real(lnprio2);
            postnew =  likenew + lnprionew;
            
          end


          propdens         = -0.5*sigpropdim*log(2*pi) - 0.5*sigproplndet - 0.5*sigpropdim*log(cc^2) ...
              -0.5*(paranew - paraold)'*sigpropinv*(paranew - paraold)/cc^2;
          
          r = min([1 ; exp( postnew - postold)]);      
          
          if rand(1,1) < r;
            % Accept proposed jump
            
            parasim = paranew;
            
            postsim(j,1) = postnew;
            likesim(j,1) = likenew;
            lnpriosim(j,1)  = lnprionew;
            parasimm(j,:) = paranew';
            paraold = paranew;
            postold = postnew;
            likeold = likenew;
            lnprioold = lnprionew;
            
            predold = pred;
            vpredold = vpred;
            zendold = zend;
            ZZold = ZZ;
            DDold = DD;
            QQold = QQ;
            
            TTTold = TTT;
            RRRold = RRR;
            validold = valid;
            
            TTTsim(j,:) = TTT(:)';
            RRRsim(j,:) = RRR(:)';
            
          else;
            % Reject proposed jump
            parasim = paranew;
            
            likesim(j,1) = likeold;
            lnpriosim(j,1)  = lnprioold;
            postsim(j,1) = postold;
            parasimm(j,:) = paraold';
            rej(j) = 1;
            reje = reje+1;
            
            pred = predold;
            vpred = vpredold;
            zend = zendold;
            ZZ = ZZold;
            DD = DDold;
            QQ = QQold;
            
            TTT = TTTold;
            RRR = RRRold;
            valid = validold;
            
            TTTsim(j,:) = TTTold(:)';
            RRRsim(j,:) = RRRold(:)';
            
          end
          if Verbose
            fprintf(1,'\nBlock %2.2g of %2.2g',[iblock,nblocks]);
            fprintf(1,'\nSimulation step %2.0f of %2.0f',[j,nsim]);
            fprintf(1,'\nRejection r: %2.6f rej %2.2g perct %2.4f ',[r,rej(j),sum(rej)/j]);
            fprintf(1,'\nLikelihood: %4.6f Posterior: %4.6f Proposal Dens: %4.6f  ',[likesim(j,1),postsim(j,1),propsim(j,1)]);
            %fprintf(1,'\n Parameters: %2.6f',parasim);
            
          end                            
        end     % if  (lambdas > 0 | isnan(lambda)) & ~fixparam
        
      end % end j=1:nsim  
      fprintf(1,'\n block rejection rate = %2.4f, overall rejection rate = %2.4f',[sum(rej)/j reje/Tim]);

      if  (lambdas > 0 | isnan(lambda))
        fwrite(fid1,parasimm','single');
        fwrite(fid4,postsim','single');
        fwrite(fid40,likesim','single');
        fwrite(fid41,lnpriosim','single');
        if SAVEALL
          fwrite(fid11,TTTsim','single');
          fwrite(fid12,RRRsim','single');
        end		
      end
      
    end;    % end simulation - given lambda and time period
    toc
    fprintf(1,'rejection rate = %2.4f',reje/Tim);

    if  (lambdas > 0 | isnan(lambda))
      num1 = ftell(fid1);
      fclose(fid1);
      num4 = ftell(fid4);
      fclose(fid4);
      num40 = ftell(fid40);
      fclose(fid40);
      num41 = ftell(fid41);
      fclose(fid41);
      if SAVEALL
        num11 = ftell(fid11);
        fclose(fid11);
        num12 = ftell(fid12);
        fclose(fid12);
      end
      
      
      outfile = strcat(spath,'/mhvarnums',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));        
      %		        save(outfile,'num1');   		        save(outfile,'num4','-append');             		
    end
    
  end %% end rolling sample

end         % end lambdaseq



exit;
