close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

run startdsswfed;

!hostname;
!ps -C matlab;
rand('state',sum(100*clock))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para, ...
 varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode, ...
 stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix,para_aux,mspec_ref,nobsdrat,lambdac] = varspec(chdata);

%[fpath,spath,gpath] = pathspecmh(num2str(floor(dataset)),chdata,coint);
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);

fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);

if isempty(mspec_ref)
  lmodel_ref = [];
  nobsdrat_ = [];
  lambdac_ = [];
else
  lmodel_ref = strcat('m',num2str(mspec_ref));
  nobsdrat_ = num2str(nobsdrat);
  if isinf(lambdac)
      lambdac_ = [];
  else
      lambdac_ = num2str(100*lambdac);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);
pmean = pmean.*(1-para_mask)+para_fix.*para_mask;
pstdd = pstdd.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reference Model for Prior Second Moments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(mspec_ref)
  nobsd = (1/nobsdrat)*stime;

  if ~isnan(mspec_ref)

    lmodel_ref    = strcat('m',num2str(mspec_ref));

    %% WANT POPULATION OR SAMPLE MOMENTS?
    POPUL = 1
    if POPUL 
      POP_ = '';
    else
      POP_ = 'P';
    end

    infile02 = strcat(spath,'/yyyy',lmodel_ref,lprior,dataset,POP_,'ThMean');
    fid02 = fopen(infile02,'r');		% input file (set to write)   
    infile03 = strcat(spath,'/xxyy',lmodel_ref,lprior,dataset,POP_,'ThMean');		
    fid03 = fopen(infile03,'r');		% input file (set to write)   
    infile04 = strcat(spath,'/xxxx',lmodel_ref,lprior,dataset,POP_,'ThMean');		
    fid04 = fopen(infile04,'r');		% input file (set to write)   

    k_ = nlags*nvar;

    YYYYd = fread(fid02,[nvar nvar],'single');
    XXYYd = fread(fid03,[k_  nvar],'single');
    XXXXd = fread(fid04,[k_ k_],'single');
    
    fclose(fid02);
    fclose(fid03);
    fclose(fid04);

  elseif isnan(mspec_ref)

    nobs_p = 94;
    k_ = nlags*nvar;

    infile02 = strcat(spath,'/yyyy',dataset,num2str(nobs_p),'PreSample');
    fid02 = fopen(infile02,'r');		% output file (set to write)   
    infile03 = strcat(spath,'/xxyy',dataset,num2str(nobs_p),'PreSample');		
    fid03 = fopen(infile03,'r');		% output file (set to write)   
    infile04 = strcat(spath,'/xxxx',dataset,num2str(nobs_p),'PreSample');		
    fid04 = fopen(infile04,'r');		% output file (set to write)   

    YYYYd = fread(fid02,[nvar nvar],'single');
    XXYYd = fread(fid03,[k_  nvar],'single');
    XXXXd = fread(fid04,[k_ k_],'single');
    
    fclose(fid02);
    fclose(fid03);
    fclose(fid04);

  end

  infile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,[],[],'Mean');
  fid0 = fopen(infile0,'r');     % input file (set to read)   
  params_1bar = fread(fid0,[npara,1],'single');
  fclose(fid0);  
  params_1bar = params_1bar.*(1-para_mask)+para_fix.*para_mask;


  YYYYd = YYYYd*nobsd;
  XXYYd = XXYYd*nobsd;
  XXXXd = XXXXd*nobsd;

elseif isempty(mspec_ref)
  
  params_1bar = [];
  nobsd = [];
  YYYYd = [];
  XXYYd = [];
  XXXXd = [];
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% doing rolling sample estimation?
ROLL = 0;
SAVEALL = 1;
DOFORECAST = 0;
if ROLL 
  DOFORECAST = 1;
  SAVEALL = 0;
end
fprintf(1,'\n\n ROLL: %2.0f, SAVEALL: %2.0f, DOFORECAST: %2.0f\n',[ROLL,SAVEALL,DOFORECAST]);
start_period = stime;                               % minimum # quarters to compute VAR estimates 

%mkdir(spath,'/2/');
%spath = strcat(spath,'/1/');
%start_period = nobsall-qahead+1-3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% CHAnGE

trial_period = nobsall-start_period-qahead+1;      % # quarters to compute RMSE


%%  Configure the Metropolis Algorithm
cc0     = 0.01;
cc      = 0.12;


Verbose = 0;


date_q = (1:1:qahead)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do Lambda loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fixparam = fixparamseq;          
L = 0;
for lambdas = lambdaseq     
  
  L = L+1;        
  

  if ROLL        
    Iseq = start_period+1:start_period+trial_period;

    Iseq = start_period+1:start_period+trial_period;
    %% change

    fprintf(1,'\n beginning: %2.4f ; end:  %2.4f \n',[ti(Iseq(1)),ti(Iseq(end))]);
  else
    Iseq = find(ti == dates);
  end
  for I = Iseq;
    if ~ROLL
      start_period = I-1;
    end
    if DOFORECAST
      Fh = zeros(qahead,nvar);           % forecasts errors 
      Yh = zeros(qahead,nvar);           % forecasts 
    end

    fprintf(1,'\n\nRMSE computation: lambda = %2.3f; date: %2.2f ',[lambdas,ti(I)]);
    %	        fprintf(1,' %8.98s;',fixparam);
    
    if stime                           
      YY = YYall(I-stime+1:I,:);
      XX = XXall(I-stime+1:I,:);             
      tiI = ti(I-stime+1:I);
    else
      YY     = YYall(1:I,:);
      XX     = XXall(1:I,:);                   
      tiI = ti(1:I);
    end
    
    %% Define initial Y's and cointegration vector for DSGE model estimation
    %% ignore cointadd
    
    %% YY0: lagged growth rates
    YY0 = zeros(nlags,size(YY,2));
    for lagind = 1:nlags
      YY0(nlags-lagind+1,:) = XX(1,1+cointall+(lagind-1)*nvar+1:1+cointall+lagind*nvar);
    end
    
    %% YYcoint0: lagged cointegration vector
    if coint > 0            
      YYcoint0 = XX(1,1+cointadd:cointall);
      for lagind = 1:(nlags-1)
        YYcoint0 = YYcoint0-XX(1,1+cointall+(lagind-1)*nvar+1:1+cointall+lagind*nvar)*cointvec';
      end
    else
      YYcoint0 = [];               
    end
    
    if I+qahead <= nobsall
      YYfut  = YYall(I+1:I+qahead,:);
      XXpred = XXall(I+1,:);
    elseif I == nobsall
      YYfut  = NaN*zeros(qahead,nvar);
      XXpred = XXall(end,:);
    elseif (I < nobsall) & (I+qahead > nobsall)
      Ii = nobsall-I
      YYfut  = [YYall(I+1:I+Ii,:);NaN*zeros(qahead-Ii,nvar)];
      XXpred = XXall(end-Ii,:);        
    end
    
    
    nobs = size(YY,1);
    
    YYYY = YY'*YY;
    XXYY = XX'*YY;
    XXXX = XX'*XX;
    
    if JustMode

      lmodel_ref2 = strcat('m',num2str(106));
      
      nobs = 2*nobs
      %% WANT POPULATION OR SAMPLE MOMENTS?
      POPUL = 1
      if POPUL 
        POP_ = '';
      else
        POP_ = 'P';
      end

      infile02 = strcat(spath,'/yyyy',lmodel_ref2,lprior,dataset,POP_,'ThMeanND');
      fid02 = fopen(infile02,'r');		% input file (set to write)   
      infile03 = strcat(spath,'/xxyy',lmodel_ref2,lprior,dataset,POP_,'ThMeanND');		
      fid03 = fopen(infile03,'r');		% input file (set to write)   
      infile04 = strcat(spath,'/xxxx',lmodel_ref2,lprior,dataset,POP_,'ThMeanND');		
      fid04 = fopen(infile04,'r');		% input file (set to write)   

      k_ = 1+nlags*nvar;

      YYYYd_ = fread(fid02,[nvar nvar],'single');
      XXYYd_ = fread(fid03,[k_  nvar],'single');
      XXXXd_ = fread(fid04,[k_ k_],'single');
      
      fclose(fid02);
      fclose(fid03);
      fclose(fid04);
      
      YYYY = YYYYd_*nobs;
      XXYY = XXYYd_*nobs;
      XXXX = XXXXd_*nobs;
      
    end

    
    
    
    lambda = lambdas*nobs;
    
    if isinf(lambdas)
      add = 'inf';
    elseif isnan(lambdas)
      add = 'dsge';
    else
      add = '';
    end

    lambdacs = lambdac;
    lambdac = lambdac*nobsd;
    if isinf(lambdac)
      addc = 'inf';
    else
      addc = '';
    end
    
    marglh = 'marglh';
    
    if (lambdas > 0) | isinf(lambdas) | isnan(lambdas)
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% find posterior mode wrt dsge parameters  
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


      if ~isnan(lambda) 
        Ob = feval(strcat('objfcn',add),para,...
                   YYYY,XXYY,XXXX,nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,para_mask,...
                   para_fix,marglh,coint,cointadd,cointall,mspec_ref,params_1bar,YYYYd,XXYYd,XXXXd,nobsd,para_aux,lambdac,addc,bounds,0);
      elseif isnan(lambda) 
        Ob = feval(strcat('objfcn',add),para,YY,YY0,...
                   nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,para_mask,para_fix,marglh,...
                   coint,cointadd,cointall,YYcoint0,mspec_ref,params_1bar,YYYYd,XXYYd,XXXXd,nobsd,para_aux,lambdac,addc,bounds,0);
      end
      format long g
      disp('Objective Fcn at Starting Value');
      disp(Ob)

      if ~fixparam
        
        if ROLL

          infile0 = strcat(spath,lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,'para',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(end)),num2str(nlags),'Mode');          
          fid0 = fopen(infile0,'r');     % input file (set to read)   
          params = fread(fid0,[npara,1],'single');
          fclose(fid0);

        else %% if ~ROLL
          infile0 = strcat(spath,lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,'para',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),'Mode')
          %          lmodel0 = strcat('m',num2str(105)); infile0 = strcat(spath,lmodel0,lprior,'para',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(end)),num2str(nlags),'Mode');          
          
          if ~exist(infile0,'file');
            infile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,'Mean')
            if ~exist(infile0,'file');
              infile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,lmodel_ref,num2str(4),'Mean')
            end
          end
          
          
          fid0 = fopen(infile0,'r');     % input file (set to read)   
          params = fread(fid0,[npara,1],'single');
          fclose(fid0);


          CH = 2%menu('Re -  Maximize the posterior density?','Yes','No');
          
          if CH == 1
            %%     Maximize the posterior density
            x0= invtrans(params,trspec)+.0001*randn(npara,1);
            H0= eye(npara)*1E-4;
            nit = 999%input('number of iterations?'); 
            crit= 1E-12;
            MIN = 1;
            if ~isnan(lambda) 
              [fh,xh,g,H,itct,fcount,retcode] = csminwel(strcat('objfcn',add),x0,H0,[],crit,nit,...
                                                         YYYY,XXYY,XXXX,nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,para_mask,...
                                                         para_fix,marglh,coint,cointadd,cointall,mspec_ref,params_1bar,YYYYd,XXYYd,XXXXd,nobsd,para_aux,lambdac,addc,bounds,MIN);
            elseif isnan(lambda) 
              [fh,xh,g,H,itct,fcount,retcode] = csminwel(strcat('objfcn',add),x0,H0,[],crit,nit,YY,YY0,...
                                                         nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,para_mask,para_fix,marglh,...
                                                         coint,cointadd,cointall,YYcoint0,mspec_ref,params_1bar,YYYYd,XXYYd,XXXXd,nobsd,para_aux,lambdac,addc,bounds,MIN);
            end
            xh = real(xh);
            params = trans(xh,trspec);
            params = params.*(1-para_mask)+para_fix.*para_mask;
            %disp('move on?');	pause;
            
          end
          
          CHK = 0; %% do you want to check para values, irfs, etc
          if CHK
            checkml;			
            if  CLOSE == 1
              return;
            end
          end %% id if CHK	
          
          outfile0 = strcat(spath,lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,'para',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),'Mode');

          fid0 = fopen(outfile0,'w');     % output file (set to write)   
          fwrite(fid0,params','single');
          fclose(fid0);
        end %% end if ROLL	            
        
      elseif fixparam
        %% default values for FIXED DSGE parameters
        params = para;                       
      end;
      

      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% find Hessian corresponding to posterior mode  
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      if ROLL
        infile01 = strcat(spath,lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,'hessian',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(end)),num2str(nlags),'Mode');
        
        fid01 = fopen(infile01,'r');     % input file (set to read)   
        hessian = fread(fid01,[npara,npara],'single');
        fclose(fid01);


      elseif ~ROLL

        infile01 = strcat(spath,lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,'hessian',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),'Mode');
        
        if exist(infile01,'file');
          fid01 = fopen(infile01,'r');     % input file (set to read)   
          hessian = fread(fid01,[npara,npara],'single');
          fclose(fid01);
        end
        CH = 2%menu('Re-Compute hessian?','Yes','No','just finish');        
        
        if CH == 1
          MIN = 0;
          if ~isnan(lambda) 
            hessian = hessizero(strcat('objfcn',add),[params,para_mask],1,...
                                YYYY,XXYY,XXXX,nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,para_mask,...
                                para_fix,marglh,coint,cointadd,cointall,mspec_ref,params_1bar,YYYYd,XXYYd,XXXXd,nobsd,para_aux,lambdac,addc,bounds,MIN);
          elseif isnan(lambda) 
            hessian = hessizero(strcat('objfcn',add),[params,para_mask],1,YY,YY0,...
                                nobs,lambda,nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,para_mask,para_fix,marglh,...
                                coint,cointadd,cointall,YYcoint0,mspec_ref,params_1bar,YYYYd,XXYYd,XXXXd,nobsd,para_aux,lambdac,addc,bounds,MIN);
          end
          disp('proceed?');     
        elseif CH == 3
          return;
        end;
        outfile01 = strcat(spath,lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,'hessian',num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),'Mode');
        fid01 = fopen(outfile01,'w');     % output file (set to write)   
        fwrite(fid01,hessian(:),'single');
        fclose(fid01);


      end; %% end if ROLL	
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% set up Metropolis Hastings
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      rankhhm = npara;
      % Create Inverse by Singular Value Decomposition
      [u,s,v] = svd(hessian);
      md = min(size(s)); 
      bigev = find(diag(s(1:md,1:md))>1e-6);
      sigpropdim = length(bigev);
      
      sigproplndet = 0;
      for i = 1:npara
	
        if i > sigpropdim
          s(i,i) = 0;
        else
          
          s(i,i)    = 1/s(i,i);
          sigproplndet = sigproplndet+log(s(i,i));  
          
        end
      end
      invhhm  = u*s*u';
      sigscale = u*sqrt(s);
      sigpropinv  =  hessian; 
      clear hessian*;
      
      
      statname = {'POST';'LIKE';'PROP';'PROPPOST';'REJECT'};
      
      pred = [];
      vpred = [];
      
      paranew = params;
      [TTT,RRR,valid] = dsgesolv(paranew,mspec);
      nstate = length(TTT);
      nshocks = size(RRR,2);

      if ~isnan(lambda)
        [yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,paranew,nvar,nlags,mspec,npara,coint,cointadd,cointall);
        [lnpy0] = feval(strcat('likmh',add),paranew,bounds,YYYY,XXYY,XXXX,nobs,lambda,...
                        nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd,xxyyd,xxxxd,retcode,para_mask,coint,cointadd,cointall);
      elseif isnan(lambda)
        retcode = valid;
        [lnpy0,zend0,ZZ0,DD0,QQ0] = feval(strcat('likmh',add),paranew,bounds,YY,YY0,nobs,lambda,...
                                          nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,TTT,RRR,valid,para_mask,coint,cointadd,cointall,YYcoint0);    
      end
      if isempty(mspec_ref)
        lnprio0  = priodens(paranew,pmean,pstdd,pshape);
      elseif ~isempty(mspec_ref)
        %% Fix non-auxiliary parameters
        paranew_ = paranew.*para_aux+params_1bar.*(1-para_aux);
        [TTT_,RRR_,valid_] = dsgesolv(paranew_,mspec);

        [yyyyd_,xxyyd_,xxxxd_,retcode] = varprio_dmean(TTT_,RRR_,valid_,paranew_,nvar,nlags,mspec,npara,coint,cointadd,cointall);
        [lnprio0,likenew_] = feval(strcat('objfcnmh',addc),paranew,bounds,YYYYd,XXYYd,XXXXd,nobsd,lambdac,...
                                   nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd_,xxyyd_,xxxxd_,retcode,para_mask,coint,cointadd,cointall);
      end
      lnpost0  = lnpy0 + real(lnprio0);


      % Initialize PARANEW            
      valid0 = 0;
      while ~valid0
        
        paranew   = params + cc0*(sigscale*randn(npara,1));
        paranew = paranew.*(1-para_mask)+para_fix.*para_mask;
        [TTT,RRR,valid] = dsgesolv(paranew,mspec);
        if ~isnan(lambda)
          [yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,paranew,nvar,nlags,mspec,npara,coint,cointadd,cointall);
          [likenew] = feval(strcat('likmh',add),paranew,bounds,YYYY,XXYY,XXXX,nobs,lambda,...
                            nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd,xxyyd,xxxxd,retcode,para_mask,coint,cointadd,cointall);
        elseif isnan(lambda)
          retcode = valid;
          [likenew,zend,ZZ,DD,QQ] = feval(strcat('likmh',add),paranew,bounds,YY,YY0,nobs,lambda,...
                                          nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,TTT,RRR,valid,para_mask,coint,cointadd,cointall,YYcoint0);    
        end
        if isempty(mspec_ref)
          lnprio  = priodens(paranew,pmean,pstdd,pshape);
        elseif ~isempty(mspec_ref)
          %% Fix non-auxiliary parameters
          paranew_ = paranew.*para_aux+params_1bar.*(1-para_aux);
          [TTT_,RRR_,valid_] = dsgesolv(paranew_,mspec);

          [yyyyd_,xxyyd_,xxxxd_,retcode] = varprio_dmean(TTT_,RRR_,valid_,paranew_,nvar,nlags,mspec,npara,coint,cointadd,cointall);
          [lnprio,likenew_] = feval(strcat('objfcnmh',addc),paranew,bounds,YYYYd,XXYYd,XXXXd,nobsd,lambdac,...
                                    nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd_,xxyyd_,xxxxd_,retcode,para_mask,coint,cointadd,cointall);
        end
        
        postnew  = likenew + real(lnprio);
        
        propdens  = -0.5*sigpropdim*log(2*pi) - 0.5*sigproplndet - 0.5*sigpropdim*log(cc0^2) ...
            -0.5*(paranew - params)'*sigpropinv*(paranew - params)/cc0^2;
        if postnew > -10000;
          valid0 = 1
        end
        disp('initializing')
	
      end  
    end
    
    tic
    fprintf(1,'\n Lambda = %2.3f',[lambdas]);
    fprintf(1,' model = %2.3f , becnhmark = %2.3f',[mspec,mspec_ref]);
    fprintf(1,'\n Time %3.2f ',[ti(I)]);
    fprintf(1,'\n Peak = %2.3f',[lnpost0]);
    
    if  (lambdas > 0 | isnan(lambda))

      infofile = strcat(spath,'/info',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));        
      if lambdas ~= 0;         save(infofile,'nstate','nshocks');        end;
      
      %% open write files
      %% params file
      outfile1 = strcat(spath,'/mhpara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));      
      fid1 = fopen(outfile1,'w');     % output file (set to write)   
                                      %% Posterior file
      outfile4 = strcat(spath,'/post',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));        
      fid4 = fopen(outfile4,'w');     % output file (set to write)   
      
      if SAVEALL
        %% VAR file
        outfile2 = strcat(spath,'/varpara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));     
        fid2 = fopen(outfile2,'w');     % output file (set to write)   
                                        %% Forecasts file
        outfile3 = strcat(spath,'/pred',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));        
        fid3 = fopen(outfile3,'w');     % output file (set to write)   
                                        %% Rotation file
        outfile5 = strcat(spath,'/rot',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));     
        fid5 = fopen(outfile5,'w');     % output file (set to write)   
        
        %% MA representation: TTT (nstate x nstate)
        outfile11 = strcat(spath,'/TTT',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    
        fid11 = fopen(outfile11,'w');       % output file (set to write)   
        
        %% MA representation: RRR (nstate x nshocks)
        outfile12 = strcat(spath,'/RRR',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    
        fid12 = fopen(outfile12,'w');       % output file (set to write)   
      end
    end
    %%  set initial values for simulations
    Tim = 0;
    eT = 0;	        
    reje = 0;
    
    for iblock = 1:nblocks  

      if iblock >1; fprintf(1,' block: %2.0f %3.2f ;',[iblock,toc/(iblock-1)]); end;

      if  (lambdas > 0 | isnan(lambda))
        parasimm = zeros(nsim,npara);
        likesim = zeros(nsim,1);
        postsim = zeros(nsim,1);
        rej     = zeros(nsim,1);
        propsim = zeros(nsim,1);
        proppostsim = zeros(nsim,1);
        TTTsim = zeros(nsim,nstate^2);
        RRRsim = zeros(nsim,nstate*nshocks);
        
        varpara = zeros(nsim,nvar*(1+nlags*nvar+coint)+nvar^2);
        pred = zeros(nsim,nvar*qahead);
        rotpara = zeros(nsim,nvar^2);
      end
      
      for j = 1:nsim
        Tim = Tim+1;  	        
        if  (lambdas > 0 | isnan(lambda)) & ~fixparam
          
          
          if ( j == 1 ) & (iblock == 1)
            postold      = postnew;
            likeold      = likenew;
            paraold      = paranew;     
            rej(1) = 0;
            if ~isnan(lambda)
              yyyydold = yyyyd; 
              xxyydold = xxyyd; 
              xxxxdold = xxxxd; 
              
            elseif isnan(lambda)
              predold = pred;
              vpredold = vpred;
              zendold = zend;
              ZZold = ZZ;
              DDold = DD;
              QQold = QQ;
              
            end
            TTTold = TTT;
            RRRold = RRR;
            validold = valid;
            
          end
          
          paranew          = paraold + cc*(sigscale*randn(npara,1));
          paranew = paranew.*(1-para_mask)+para_fix.*para_mask;
          [TTT,RRR,valid] = dsgesolv(paranew,mspec);
          if ~isnan(lambda)
            [yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,paranew,nvar,nlags,mspec,npara,coint,cointadd,cointall);
            [likenew] = feval(strcat('likmh',add),paranew,bounds,YYYY,XXYY,XXXX,nobs,lambda,...
                              nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd,xxyyd,xxxxd,retcode,para_mask,coint,cointadd,cointall);
          elseif isnan(lambda)
            retcode = valid;
            [likenew,zend,ZZ,DD,QQ] = feval(strcat('likmh',add),paranew,bounds,YY,YY0,nobs,lambda,...
                                            nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,TTT,RRR,valid,para_mask,coint,cointadd,cointall,YYcoint0);    
          end
          if isempty(mspec_ref)
            lnprio  = priodens(paranew,pmean,pstdd,pshape);
          elseif ~isempty(mspec_ref)
            %% Fix non-auxiliary parameters
            paranew_ = paranew.*para_aux+params_1bar.*(1-para_aux);
            [TTT_,RRR_,valid_] = dsgesolv(paranew_,mspec);

            [yyyyd_,xxyyd_,xxxxd_,retcode] = varprio_dmean(TTT_,RRR_,valid_,paranew_,nvar,nlags,mspec,npara,coint,cointadd,cointall);
            [lnprio,likenew_] = feval(strcat('objfcnmh',addc),paranew,bounds,YYYYd,XXYYd,XXXXd,nobsd,lambdac,...
                                      nlags,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd_,xxyyd_,xxxxd_,retcode,para_mask,coint,cointadd,cointall);
          end

          postnew  = likenew + real(lnprio);

          propdens         = -0.5*sigpropdim*log(2*pi) - 0.5*sigproplndet - 0.5*sigpropdim*log(cc^2) ...
              -0.5*(paranew - paraold)'*sigpropinv*(paranew - paraold)/cc^2;
          
          r = min([1 ; exp( postnew - postold)]);      
          
          if rand(1,1) < r;
            % Accept proposed jump
            
            parasim = paranew;
            
            postsim(j,1) = postnew;
            likesim(j,1) = likenew;
            parasimm(j,:) = paranew';
            paraold = paranew;
            postold = postnew;
            likeold = likenew;
            
            if ~isnan(lambda)
              yyyydold = yyyyd; 
              xxyydold = xxyyd; 
              xxxxdold = xxxxd;                                 
            elseif isnan(lambda)
              predold = pred;
              vpredold = vpred;
              zendold = zend;
              ZZold = ZZ;
              DDold = DD;
              QQold = QQ;
            end
            
            TTTold = TTT;
            RRRold = RRR;
            validold = valid;
            
            TTTsim(j,:) = TTT(:)';
            RRRsim(j,:) = RRR(:)';
            
          else;
            % Reject proposed jump
            parasim = paranew;
            
            likesim(j,1) = likeold;
            postsim(j,1) = postold;
            parasimm(j,:) = paraold';
            rej(j) = 1;
            reje = reje+1;
            
            if ~isnan(lambda)
              yyyyd = yyyydold; 
              xxyyd = xxyydold; 
              xxxxd = xxxxdold; 
            elseif isnan(lambda)
              pred = predold;
              vpred = vpredold;
              zend = zendold;
              ZZ = ZZold;
              DD = DDold;
              QQ = QQold;
            end
            
            TTT = TTTold;
            RRR = RRRold;
            valid = validold;
            
            TTTsim(j,:) = TTTold(:)';
            RRRsim(j,:) = RRRold(:)';
            
          end
          if Verbose
            fprintf(1,'\nBlock %2.2g of %2.2g',[iblock,nblocks]);
            fprintf(1,'\nSimulation step %2.0f of %2.0f',[j,nsim]);
            fprintf(1,'\nRejection r: %2.6f rej %2.2g perct %2.4f ',[r,rej(j),sum(rej)/j]);
            fprintf(1,'\nLikelihood: %4.6f Posterior: %4.6f Proposal Dens: %4.6f Estimated Score %4.6f ',[likesim(j,1),postsim(j,1),propsim(j,1),lnpy0+log(1/(sum(1/exp(likesim(1:j,1)-lnpy0))/j))]);
            fprintf(1,'\n Parameters: %2.6f',parasim);
            
          end                            
        end     % if  (lambdas > 0 | isnan(lambda)) & ~fixparam
        
        if DOFORECAST

          if ~isnan(lambda)           
            %%%%%%%%%%%%%%%%%%%%%%%%% For each para, draw a CC and Sig for the VAR
            
            if (lambdas > 0) & ~isinf(lambdas)
              
              yyyyd = YYYY + lambda*yyyyd;
              xxyyd = XXYY + lambda*xxyyd;
              xxxxd = XXXX + lambda*xxxxd;
              
            elseif isinf(lambdas)
              
              yyyyd = yyyyd;
              xxyyd = xxyyd;
              xxxxd = xxxxd;
              
            elseif lambdas == 0
              
              yyyyd = YYYY;
              xxyyd = XXYY;
              xxxxd = XXXX;
              
            end
            
            nobsc = nobs + lambda;    
            k     = 1+nlags*nvar+coint;
            
            % Compute OLS estimates
            
            invxxxxd = inv(xxxxd);
            c_hat0  = invxxxxd*xxyyd;
            c_hat   = c_hat0(:);
            
            if ~isinf(lambdas)
              %                   sig_hat = (yyyyd -xxyyd'*invxxxxd*xxyyd)/nobsc;
              sig_hat = (yyyyd -xxyyd'*c_hat0)/nobsc;
              
              % draw from the marginal posterior of sig
              
              h = nobsc*sig_hat;           
              z = chol(inv(h))'*randn(nvar,nobsc-k);
              sig_sim = inv(z*z');
              
              clear z h;
              
              % draw from the conditional posterior of C
              
              vc     = kron(sig_sim,invxxxxd);
              cvc = chol(vc)';
              
              val = 0;
              while ~val;
                
		
		
                c_sim  = c_hat + cvc*randn(nvar*k,1);
                cct_sim= reshape(c_sim,k,nvar)';
                %clear xxxxd ;
                
                if ~coint
                  % Check for stationarity            
                  cc_tildes =  [cct_sim(:,(coint+1+1):k) ; ...
                                [eye(nvar*(nlags-1)),zeros(nvar*(nlags-1),nvar)]];
                  if max(abs(eig(cc_tildes))) < 1; 
                    val = 1;
                  end;
                elseif coint 
                  val = 1;
                end
                %clear cct_sim;
                
              end; 
              clear sig_hat;
            elseif isinf(lambdas)
              sig_hat = (yyyyd -xxyyd'*invxxxxd*xxyyd);
              c_sim = c_hat;
              sig_sim = sig_hat;
              cct_sim= reshape(c_sim,k,nvar)';
            end
            
            clear sig_hat;
            varpara(j,:) = [c_sim',sig_sim(:)'];
            
            %%%%%%%%%%%%%%%%%%%%%%%% Get rotation
            if (lambda > 0) & ~ROLL
              aairf = irfdsge(TTT,RRR,parasimm(j,:)',nvar,mspec,npara,1,cum_irf,valid,nlags,coint,cointadd,cointall);

              a0_m = [];
              for ii=1:nvar
                a0_m = [a0_m,aairf(1,nvar*(ii-1)+1:nvar*ii)'];
              end; 
              
              [phimat,r_a] = qr(a0_m');         
              phimat = phimat';
              %% phimat is normalized (row signs are changed) so that the lower diagonal matrix (r_a') has positive diagonal elements  
              phimat = repmat(sign(diag(r_a)),[1 nvar]).*phimat;
              
              rotpara(j,:) = [phimat(:)'];
            else  %% if lambda == 0 | ROLL
              phimat = eye(nvar);
            end;
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%% For each cc and Sigma and rotation, get distribution of predicted values
            
            yypred  = zeros(qahead,nvar);
            if ~ROLL
              sig_chol = chol(sig_sim)'*phimat;
              Shocks = randn(qahead,nvar);

              xxpred =  XXpred;
              if coint
                for t = 1:qahead;       
                  out = (cct_sim*xxpred'+sig_chol*Shocks(t,:)')';
                  yypred(t,:) = out;
                  xxl = xxpred(coint+1+1:k-nvar);
                  addcoint = xxpred(1:coint)+out*cointvec';				    
                  xxpred = [addcoint,1,out,xxl];           
                end;
              elseif ~coint
                for t = 1:qahead;       
                  out = (cct_sim*xxpred'+sig_chol*Shocks(t,:)')';
                  yypred(t,:) = out;
                  xxl = xxpred(1+1:k-nvar);
                  xxpred = [1,out,xxl];           
                end;
              end
              

            elseif ROLL



              if coint
                yypred  = zeros(qahead,nvar);
                xxpred =  XXpred;
                for t = 1:qahead;       
                  out = (cct_sim*xxpred')';
                  yypred(t,:) = out;
                  xxl = xxpred(coint+1+1:k-nvar);
                  addcoint = xxpred(1:coint)+out*cointvec';				    
                  xxpred = [addcoint,1,out,xxl];           
                end;
              elseif ~coint
                %			                    xxpred =  XXpred;
                %				            for t = 1:qahead;       
                %			        	            out = (cct_sim*xxpred')';
                %			                	    yypred(t,:) = out;
                %				                    xxl = xxpred(1+1:k-nvar);
                %				                    xxpred = [1,out,xxl];           
                %				            end;	

                yypred = yypredmex(XXpred,cct_sim,qahead,nvar,k); 
              end


              
            end	
            
            if Tim > nburn
              eT = eT+1; 			
              Fh(:,ICUM) = ((eT-1)/eT)*Fh(:,ICUM)+...
                  (1/eT)*cumsum(yypred(:,ICUM)-YYfut(:,ICUM)); 
              Yh(:,ICUM) = ((eT-1)/eT)*Yh(:,ICUM)+(1/eT)*cumsum(yypred(:,ICUM)); 

              Fh(:,INCUM) = ((eT-1)/eT)*Fh(:,INCUM)+(1/eT)*(yypred(:,INCUM)-YYfut(:,INCUM)); 
              Yh(:,INCUM) = ((eT-1)/eT)*Yh(:,INCUM)+(1/eT)*yypred(:,INCUM); 
            end			                            

            pred(j,:) = [yypred(:)'];
            
            
            
            
          elseif isnan(lambda)  %%%%%%%%%%%%%%%%%%%%%%%%%%%%% isnan(lambda)
            
            yypred  = zeros(qahead,nvar);
            if ~ROLL
              Shocks = randn(qahead,nshocks);
              
              %% careful : we are assuming iid shocks
              RRR1 = RRR.*repmat(sqrt(diag(QQ)'),[size(RRR,1) 1]);
              z = zend;
              
              for t = 1:qahead;       
                z = TTT*z+RRR1*Shocks(t,:)';
                yypred(t,:) = (DD+ZZ*z)';
              end;

            elseif ROLL
              
              z = zend;
              for t = 1:qahead;       
                z = TTT*z;
                yypred(t,:) = (DD+ZZ*z)';
              end;
            end
            
            if Tim > nburn
              eT = eT+1; 			
              Fh(:,ICUM) = ((eT-1)/eT)*Fh(:,ICUM)+...
                  (1/eT)*cumsum(yypred(:,ICUM)-YYfut(:,ICUM)); 
              Yh(:,ICUM) = ((eT-1)/eT)*Yh(:,ICUM)+(1/eT)*cumsum(yypred(:,ICUM)); 

              Fh(:,INCUM) = ((eT-1)/eT)*Fh(:,INCUM)+(1/eT)*(yypred(:,INCUM)-YYfut(:,INCUM)); 
              Yh(:,INCUM) = ((eT-1)/eT)*Yh(:,INCUM)+(1/eT)*yypred(:,INCUM); 
            end			                            
            pred(j,:) = [yypred(:)'];
            
          end;    % end: if ~isnan(lambda)
        end % end if SAVEALL		            
      end % end j=1:nsim  

      if  (lambdas > 0 | isnan(lambda))
        fwrite(fid1,parasimm','single');
        fwrite(fid4,postsim','single');
        if SAVEALL
          if ~isnan(lambda)           
            fwrite(fid2,varpara','single');
          end
          fwrite(fid3,pred','single');
          fwrite(fid5,rotpara','single');
          fwrite(fid11,TTTsim','single');
          fwrite(fid12,RRRsim','single');
        end		
      end
      
    end;    % end simulation - given lambda and time period
    toc
    fprintf(1,'rejection rate = %2.4f',reje/Tim);

    if  (lambdas > 0 | isnan(lambda))
      num1 = ftell(fid1);
      fclose(fid1);
      num4 = ftell(fid4);
      fclose(fid4);
      if SAVEALL
        num2 = ftell(fid2);
        fclose(fid2);
        num3 = ftell(fid3);
        fclose(fid3);
        num5 = ftell(fid5);
        fclose(fid5);
        num11 = ftell(fid11);
        fclose(fid11);
        num12 = ftell(fid12);
        fclose(fid12);
      end
      
      
      outfile = strcat(spath,'/mhvarnums',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));        
      %		        save(outfile,'num1');   		        save(outfile,'num4','-append');             		
    end
    
    % Write Forecast Output
    if DOFORECAST
      save(strcat(spath,'Fh',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(fixparam),num2str(JustMode),num2str(100*lambdas),num2str(100*ti(I))),'Fh');
      save(strcat(spath,'Yh',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(fixparam),num2str(JustMode),num2str(100*lambdas),num2str(100*ti(I))),'Yh');
    end
    
    
  end %% end rolling sample

end         % end lambdaseq



exit;
