close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
run startdsswfed;
!hostname;
!ps -C matlab;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varspec;

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);


if isempty(mspec_ref)
  lmodel_ref = [];
  nobsdrat_ = []
  lambdac_ = [];
  nlagsc_ = [];
  n_s_ = [];
  disp('NO reference model set in varspec.m')
else
  lmodel_ref = strcat('m',num2str(mspec_ref));
  nobsdrat_ = num2str(100*nobsdrat);
  lambdac_ = [];
  n_s_ = num2str(n_s);
  if nlagsc == 4
    nlagsc_ = [];
  else
    nlagsc_ = num2str(nlagsc);
  end  
  
end
stup_ = num2str(stup);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,mspec);

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat('pri',lprior);
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set date
I = find(ti == dates);

Verbose = 0;

fixparam = fixparamseq(1);
if fixparam ~= 0 
  fixparam
  break
end

fpath2 = strcat(fpath,'/',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,'/');
mkdir(fpath2);

outfile00 = strcat(fpath2,lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(coint),st,num2str(100*ti(I)),num2str(nlags),'-',num2str(JustMode),'Mom.tex');		% output file name 
fid00 = fopen(outfile00,'w');		% output file (set to write)

fprintf(fid00,'\n \\documentclass[12pt]{article} \n \\usepackage[dvips]{color}\n ');
fprintf(fid00,'\n \\begin{document}');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do Lambda loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
percent = .90

Theta1 =[pmean];
Theta2 =[pmean,pstdd];

L = 0;
for lambdas = lambdaseq     
  
  L = L+1;

  
  

  fprintf(1,'\n Lambda = %2.3f',[lambdas]);
  fprintf(1,'\n Time %3.2f ',[ti(I)]);


  %% open read files
  %% params file
  if isnan(lambdas)
    coint0 = coint;
  else
    coint0 = cointall;
  end
  infile1 = strcat(spath,'/mhpara',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(coint0),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
  num1 = nblocks*nsim*npara*4;            % number of bites per file
  numb1 = nburn*npara*4;                  % number of bites to dicard


  %% get means and st dev of parameters
  fid1 = fopen(infile1,'r');		% input file (set to read)               
  status = fseek(fid1,numb1,'bof');

  %% Posterior file
  infile2 = strcat(spath,'/post',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
  num2 = nblocks*nsim*1*4;            % number of bites per file
  numb2 = nburn*1*4;                  % number of bites to dicard

  fid2 = fopen(infile2,'r');		% input file (set to read)               
  status = fseek(fid2,numb2,'bof');

  theta = [];
  post = [];
  while ( ftell(fid1) < num1 ) 
    % Read blocks of size nsim
    thetadd = fread(fid1,[npara,nsim],'single')';
    postadd = fread(fid2,[nsim,1],'single');
    if all(size(thetadd,2) == [npara])
      theta = [theta;thetadd];
      post = [post;postadd];
    else
      fclose(fid1);
      fclose(fid2);
    end
    clear thetadd;
    clear postadd;
    
  end;
  fclose(fid1);
  fclose(fid2);
  ndraws = size(theta,1)
  
  Likplot=1          
  if Likplot

    %% Lik file
    infile40 = strcat(spath,'/lik',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
    num40 = nblocks*nsim*1*4;            % number of bites per file
    numb40 = nburn*1*4;                  % number of bites to dicard

    fid40 = fopen(infile40,'r');		% input file (set to read)               
    status = fseek(fid40,numb40,'bof');

    %% prior
    infile41 = strcat(spath,'/lnprioD',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
    num41 = nblocks*nsim*1*4;            % number of bites per file
    numb41 = nburn*1*4;                  % number of bites to dicard

    fid41 = fopen(infile41,'r');		% input file (set to read)               
    status = fseek(fid41,numb41,'bof');

    lik = [];
    lnprioD = [];
    while ( ftell(fid40) < num40 ) 
      % Read blocks of size nsim
      likadd = fread(fid40,[nsim,1],'single');
      lnprioDadd = fread(fid41,[nsim,1],'single');
      lik = [lik;likadd];
      lnprioD = [lnprioD;lnprioDadd];

      clear likadd;
      clear lnprioDadd;

    end
    
    fclose(fid40);
    fclose(fid41);
  end      


  PLOTDRAWS = 1
  if PLOTDRAWS
    datec = strcat('Post/',date);
    mkdir(fpath2,datec);
    gpath_2 = strcat(fpath2,datec,'/');

    Postplot=1          

    Ip = 0;
    sirf = (1:length(theta));
    for p = find(para_mask == 0)'
      Ip = Ip+1;


      
      figure(p)
      ax1 = gca;
      

      title(para_names{p});

      P = line(sirf,theta(:,p));
      set(P,'Color',[0 0 0],'Parent',ax1,'LineStyle','-');
      set(P,'LineWidth',2.2);
      set(ax1,'YAxisLocation','left')
      set(ax1,'XLim',[min(sirf),max(sirf)]);
      set(ax1,'FontSize',8);    
      if Postplot
        ax2 = axes('Position',get(ax1,'Position'),'Color','none');
        P2 = line(sirf,post);
        set(P2,'Color',[0.5 0.5 0.5],'Parent',ax2,'LineStyle','-');
        set(P2,'LineWidth',.5);
        set(ax2,'YAxisLocation','right')
        set(ax2,'XLim',[min(sirf),max(sirf)]);
        set(ax2,'FontSize',8);    
      end
      if Likplot
        ax3 = axes('Position',get(ax1,'Position'),'Color','none');
        P3 = line(sirf,lik);
        set(P3,'Color',[0 0.5 0.5],'Parent',ax3,'LineStyle',':');
        set(P3,'LineWidth',.5);
        set(ax3,'YAxisLocation','right')
        set(ax3,'XLim',[min(sirf),max(sirf)]);
        set(ax3,'XTickLabel',[]);
        set(ax3,'YTick',[]);
        set(ax3,'YTickLabel',[]);
        set(ax3,'FontSize',8);    
      end



      ng=strcat(gpath_2,num2str(p));
      figure(p)
      print(char(strcat(ng,'.pdf')),'-dpdf');   % print(char(strcat(ng,'.eps')),'-depsc2');             
    end
  end


  %%%%%%%%%%% Compute posterior moments of the parameters
  thetahat = mean(theta,1)';
  thetasig = cov(theta,1);

  thetabands = hpdint(theta,percent,1)';

  %% save the mean
  outfile0 = strcat(spath,lmodel,lprior,'para',num2str(coint0),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),'Mean');

  fid0 = fopen(outfile0,'w');     % output file (set to write)   
  fwrite(fid0,thetahat','single');
  fclose(fid0);

  colnames = {'Parameter ' 'Prior Mean '  'Prior Stdd '  'Post Mean ' strcat(num2str(100*percent),'\% {\tiny Lower Band}') ...
              strcat(num2str(100*percent),'\% {\tiny Upper Band}') };
  outmat = [pmean,pstdd,thetahat,thetabands];

  Theta1 = [Theta1,thetahat];
  Theta2 = [Theta2,thetahat,thetabands];


  fprintf(1,'\n');
  fprintf(1,'%4.99s  ',colnames{:});
  for i = 1:length(para_names)
    fprintf(1,'\n $%4.99s$ & ',para_names{i});
    fprintf(1,' %8.3f & ',outmat(i,:));
  end    


  fprintf(fid00,'\n \\begin{table}[h] \\centering');        
  fprintf(fid00,'\n \\caption{Parameter Estimates, $\\lambda$ = %2.1f } \n\\vspace*{.5cm}',lambdas);
  fprintf(fid00,'\n \\begin{tabular}{lllllll}\\hline \n');

  fprintf(fid00,' %4.99s  & ',colnames{:});
  for i = 1:length(para_names)
    fprintf(fid00,'\\\\ \n $%4.99s$ &  ',para_names{i});
    fprintf(fid00,' %8.3f & ',outmat(i,:));
  end   
  fprintf(fid00,'\\\\  \\hline');		    
  fprintf(fid00,'\n \\end{tabular}');
  fprintf(fid00,'\n \\end{table} \n\n');

  clear parasimm postsim;





end         % end lambdaseq
fprintf(1,'\n');



fprintf(fid00,'\n \\begin{table}[h] \\centering');        
fprintf(fid00,'\n \\caption{Parameter Estimates: Prior and Posterior Mean} \n\\vspace*{.5cm}',lambdas);
fprintf(fid00,'\n \\begin{tabular}{cl');
for ic = 1:length(lambdaseq)
  fprintf(fid00,'l');
end
fprintf(fid00,'}\\hline \n');
fprintf(fid00,' Paramter & Prior ');
for ic = 1:length(lambdaseq)
  fprintf(fid00,' & $\\lambda$ = %2.1f  ',lambdaseq(ic));
end
fprintf(fid00,'\\\\ \\hline \n');

for i = 1:length(para_names)
  fprintf(fid00,' \n $%4.99s$   ',para_names{i});
  fprintf(fid00,' & %8.3f  ',Theta1(i,:));
  fprintf(fid00,'\\\\ ');
end   
fprintf(fid00,'\\\\  \\hline');		    
fprintf(fid00,'\n \\end{tabular}');
fprintf(fid00,'\n \\end{table} \n\n');

%% compute moments
[TTT,RRR,valid] = dsgesolv(thetahat,mspec);
%valid = 1;
%[yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint,cointadd,cointall);

[ZZ,DD,DDcointadd,QQ,EE,MM,retcode] = measur(TTT,RRR,valid,thetahat,nvar,nlags,mspec,npara,coint,cointadd,cointall);

if retcode == 0
  disp('trouble');mm
  return;
end;

HH = EE+MM*QQ*MM';
VV = QQ*MM';

% Compute nlags autocovariances

% Initialize Autocovariances

GAMM0 = zeros(nvar^2,nlags);

GA0 = dlyap(TTT,RRR*QQ*RRR');

Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
GAMM0(:,1) = Gl(:);

TTl = TTT;
for l = 1:nlags;
  Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);
  GAMM0(:,l+1) = Gl(:);
  TTl = TTl*TTT;
end

%% Create limit cross product matrices
yyyyd0 = reshape(GAMM0(:,1),nvar,nvar);
yyyyd1 = reshape(GAMM0(:,2),nvar,nvar);
yyyyd  = yyyyd0(1:nvar,1:nvar);

yyxxd = zeros(nvar,nlags*nvar);
xxxxd = zeros(nlags*nvar,nlags*nvar);

for rr = 1:nlags;
  %% E[yy,x(lag rr)]
  yyxxd(:,nvar*(rr-1)+1:nvar*rr) = reshape(GAMM0(:,rr+1),nvar,nvar);

  %% E[x(lag rr),x(lag ll)]
  for ll = rr:nlags;
    yyyydrrll = reshape(GAMM0(:,ll-rr+1),nvar,nvar);
    xxxxd(nvar*(rr-1)+1:nvar*rr,nvar*(ll-1)+1:nvar*ll) = yyyydrrll(1:nvar,1:nvar);
    xxxxd(nvar*(ll-1)+1:nvar*ll,nvar*(rr-1)+1:nvar*rr) = yyyydrrll(1:nvar,1:nvar)';
  end
end

xxyyd = yyxxd';


MeY = DD
VaY = yyyyd;
stdY = sqrt(diag(VaY))
CorY = VaY./(stdY*stdY');
CoY = xxyyd(1:nvar,:);
ACorY = diag(CoY)./diag(VaY)


stats = [MeY' stdY' ACorY' CorY(:)'];

Fortab = '\n \\begin{tabular}{l';
Fornum = ' ';
Heads = 'Variables ';
Heads2 = ' ';

%% mean (model, data)
DatOut = [MeY];
Fortab = strcat(Fortab,'|l');
Fornum = strcat(Fornum,' & %2.2f ');
Heads = strcat(Heads,'&  \\multicolumn{1}{c}{Mean}  ');

%% st dev (model, data)
DatOut = [DatOut stdY ];
Fortab = strcat(Fortab,'|l');
Fornum = strcat(Fornum,' & %2.2f ');
Heads = strcat(Heads,'&  \\multicolumn{1}{c}{St. Dev.}  ');

%% autocorr (model, data)
DatOut = [DatOut ACorY];
Fortab = strcat(Fortab,'|l');
Fornum = strcat(Fornum,' & %2.2f ');
Heads = strcat(Heads,'&  \\multicolumn{1}{c}{Autocorr.}  ');


%% contemp corr (model, data)
CorMix = tril(CorY,-1)+triu(CorY);
DatOut2 = [CorMix];
Heads = strcat(Heads,' & \\multicolumn{',num2str(nvar),'}{c}{Contemporanous Corr.}  ');

Fortab = strcat(Fortab,'|l');
Fornum2 = ' ';
for jj = 1:nvar
  Fortab = strcat(Fortab,'l');
  Fornum2 = strcat(Fornum2,' & %2.2f ');
end


%% finish up
Fortab = strcat(Fortab,'}\\hline \n');
Heads = strcat(Heads,'\\\\ \n');


fprintf(fid00,'\n \\begin{table}[h] \\centering');
fprintf(fid00,'\n \\caption{Model %2.0f Statistics at Posterior Mean} \n\\vspace*{.5cm}',mspec);

%% setting is: mean (model, data) , variances (model, data), autocorrelations (model, data), correlations (model, data)
fprintf(fid00,Fortab);

fprintf(fid00,Heads);
for i = 1:size(DatOut,1);
  fprintf(fid00,'\\\\ \n $%4.99s$ ',varnames{i});
  fprintf(fid00,Fornum,DatOut(i,:));
  fprintf(fid00,' & ');
  fprintf(fid00,Fornum2,DatOut2(i,:));
end

fprintf(fid00,'\\\\  \\hline');
fprintf(fid00,'\n \\end{tabular}');
fprintf(fid00,'\n \\end{table} \n\n');
fprintf(fid00,'\n \\end{document}');

fclose(fid00);		% close output file

i_aux = [2 12];%find(~para_mask);%[3 29]%
thetastd = sqrt(diag(thetasig(i_aux,i_aux)));
thetacorr = thetasig(i_aux,i_aux)./(thetastd*thetastd');
dm(thetacorr);
