close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

run startdsgeprioeuro;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para, ...
 varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode, ...
 stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix] = varspec(chdata);


%[fpath,spath,gpath] = pathspecmh(num2str(floor(dataset)),chdata,coint);
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f',[mspec,dataset,mprior]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% doing rolling sample estimation?
ROLL = 1
SAVEALL = 0
DOFORECAST = 1
if ROLL 
  DOFORECAST = 1;
  SAVEALL = 0;
end
start_period = stime;                               % minimum # quarters to compute VAR estimates 


%mkdir(spath,'/2/');
%spath = strcat(spath,'/1/');
%start_period = nobsall-qahead+1-3

trial_period = nobsall-start_period-qahead+1;      % # quarters to compute RMSE

Verbose = 0;

fixparam = fixparamseq(1);
if fixparam ~= 0 
  fixparam
  break
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do Lambda loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mh = zeros(1,length(lambdaseq));
percent = .90;

L = 0;
for lambdas = lambdaseq     
  
  L = L+1;

  if ROLL        

    Iseq = start_period+1:start_period+trial_period;
%    Iseq = start_period+31:start_period+31;
    
    fprintf(1,'\n beginning: %2.4f ; end:  %2.4f \n',[ti(Iseq(1)),ti(Iseq(end))]);
    outfile00 = strcat(fpath,lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(start_period+1)),num2str(100*ti(start_period+trial_period)),num2str(nlags),'rollMOM.txt');	% output file name 

  else
    Iseq = find(ti == dates);

    outfile00 = strcat(fpath,lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(Iseq)),num2str(nlags),'MOM.txt');	% output file name 

  end

  fid00 = fopen(outfile00,'w');		% output file (set to write)
  fprintf(fid00,'\n Lambda = %2.3f',[lambdas]);
  for I = Iseq;
    if ~ROLL
      start_period = I-1;
    end


    
    
    %% set params for MH evaluation 
    nfnpara = npara-length(find(para_mask));

    hmax = 5;
    p = (.1:.1:(9-1)*.1)';
    pcrit = chi2inv(p,nfnpara);
    densfac = -1000;
    
    
    fprintf(1,'\n Lambda = %2.3f',[lambdas]);
    fprintf(1,'\n Time %3.2f ',[ti(I)]);
    

    
    %% open read files
    %% params file
    infile1 = strcat(spath,'/mhpara',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode))

    num1 = nblocks*nsim*npara*4;            % number of bites per file
    numb1 = nburn*npara*4;                  % number of bites to dicard
    
    
    %% get means and st dev of parameters
    fid1 = fopen(infile1,'r');		% input file (set to read)               
    status = fseek(fid1,numb1,'bof');
    %infile1
    %fid1		
    theta = [];
    it =0;
    it0=-inf;
    while ( ftell(fid1) < num1 ) & (it > it0)
      it0 = it;
      it=ftell(fid1);
      % Read blocks of size nsim
      thetadd = fread(fid1,[nsim,npara],'single');
      if all(size(thetadd,2) == [npara])
        theta = [theta;thetadd];
      else
        %		fclose(fid1);
      end
      clear thetadd;
      
    end;
    fclose(fid1);
    ndraws = size(theta,1)
    

    
    
    %% delete rows for fixed parameters
    theta(:,find(para_mask)) = [];
    
    %%%%%%%%%%% Compute posterior moments of the parameters
    thetahat = mean(theta,1)';
    thetasig = cov(theta,1);
    thetastd = sqrt(diag(thetasig));
    
    fprintf(fid00,'\n Time %3.2f (%2.0f) ',[ti(I),I-start_period]);
    i0 = 0;
    for i = find(~para_mask)'
      i0 = i0+1;
      fprintf(fid00,' \n $%4.99s$   ',para_names{i});
      fprintf(fid00,'  %2.3f (%2.3f) ',[thetahat(i0),thetastd(i0)]);
    end   
    

  end %% end rolling sample
  fclose(fid00);		% close output file  
end         % end lambdaseq


