close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

run startdsgeprioeuro;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para, ...
 varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode, ...
 stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix,para_aux,mspec_ref] = varspec(chdata);

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set date
I = find(ti == dates);

%% if JustMode: do not do simulations and compute forecasts at mode
if JustMode
  JustMode
  break;
end

Verbose = 0;

fixparam = fixparamseq(1);
if fixparam ~= 0 
  fixparam
  break
end

outfile00 = strcat(fpath,lmodel,lprior,dataset,num2str(coint),st,num2str(100*ti(I)),num2str(nlags),'Mom2.tex');		% output file name 
fid00 = fopen(outfile00,'w');		% output file (set to write)

fprintf(fid00,'\n \\documentclass[12pt]{article} \n \\usepackage[dvips]{color}\n ');
fprintf(fid00,'\n \\begin{document}');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do Lambda loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
percent = .90

Theta1 =[pmean];
Theta2 =[pmean,pstdd];

L = 0;
for lambdas = lambdaseq     
  
  L = L+1;
  
  
  fprintf(1,'\n Lambda = %2.3f',[lambdas]);
  fprintf(1,'\n Time %3.2f ',[ti(I)]);
  
  
  %% open read files
  %% params file
  if isnan(lambdas)
    coint0 = coint;
  else
    coint0 = cointall;
  end
  infile1 = strcat(spath,'/mhpara',lmodel,lprior,dataset,num2str(coint0),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
  num1 = nblocks*nsim*npara*4;            % number of bites per file
  numb1 = nburn*npara*4;                  % number of bites to dicard
  
  %% get means and st dev of parameters
  fid1 = fopen(infile1,'r');		% input file (set to read)               
  status = fseek(fid1,numb1,'bof');
  
  %% prior params file (first block has already been thrown out)
  infile3 = strcat(spath,'/mhNIpriopara',lmodel,lprior,dataset);   
  fid3 = fopen(infile3,'r');		% input file (set to read)               
  
  %% Posterior file
  infile4 = strcat(spath,'/post',lmodel,lprior,dataset,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));        

  num4 = nblocks*nsim*1*4;            % number of bites per file
  numb4 = nburn*1*4;                  % number of bites to dicard

  fid4 = fopen(infile4,'r');		% input file (set to read)               
  status = fseek(fid4,numb4,'bof');

  theta = [];
  priotheta = [];
  post = [];

  while ( ftell(fid1) < num1 ) 
    % Read blocks of size nsim

    thetadd = fread(fid1,[nsim,npara],'single');
    theta = [theta;thetadd];
    clear thetadd;

    priothetadd = fread(fid3,[npara,nsim],'single')';
    priotheta = [priotheta;priothetadd];
    clear priothetadd;
    
    postadd = fread(fid4,[nsim,1],'single');
    post = [post;postadd];
    clear postadd;

  end;
  fclose(fid1);
  fclose(fid3);
  fclose(fid4);
  ndraws = size(theta,1)
  
  
  PLOTDRAWS = 1
  if PLOTDRAWS
    datec = strcat(date,'-params-lambda-eq-',num2str(100*lambdas));
    mkdir(gpath,datec);
    gpath_2 = strcat(gpath,datec,'/');
    datec3 = strcat(date,'-params-lambda-eq-',num2str(100*lambdas),'PrioPost');
    mkdir(gpath,datec3);
    gpath_3 = strcat(gpath,datec3,'/');

    outfile01 = strcat(gpath_2,'paralist.tex');		% output file name 
    fid01 = fopen(outfile01,'w');		% output file (set to write)

    sirf = (1:length(theta));
    Legend(L) = {strcat('Lambda = ',num2str(lambdas))};

    Ip = 0;
    for p = find(para_mask == 0)'
      Ip = Ip+1;


      
      fprintf(fid01,'\n %2.9s ',para_names{p});
      fprintf(fid01,'equal %2.09f ',p);
      
      if length(lambdaseq) > 1
        figure(p)
        ax1 = gca;
        

        title(para_names{p});

        P = line(sirf,theta(:,p));
        if isnan(lambdas)
          set(P,'Color',[0 0 0],'Parent',ax1,'LineStyle','-');
        else
          set(P,'Color',[.3 .3 .3],'Parent',ax1,'LineStyle','-');
        end
        legend(Legend);
        set(P,'LineWidth',2.2);
        set(ax1,'YAxisLocation','left')
        set(ax1,'XLim',[min(sirf),max(sirf)]);
        %  set(gca,'XTickLabel',intx);
        set(ax1,'FontSize',8);    


        ng=strcat(gpath_2,strcat(num2str(p),'-PostAllLambdas'));
        %print(char(strcat(ng,'.pdf')),'-dpdf');   % print(char(strcat(ng,'.eps')),'-depsc2');             

      end      
      

      figure(200+p)
      title(para_names{p});

      minth = min(min(theta(:,p)),min(priotheta(:,p)));
      maxth = max(max(theta(:,p)),max(priotheta(:,p)));
      
      %subplot(1,2,1);
      ax1 = gca;
      P = line(sirf,priotheta(:,p));
      if isnan(lambdas)
        set(P,'Color',[.5 .5 .5],'Parent',ax1,'LineStyle','-');
      else
        set(P,'Color',[.5 .5 .5],'Parent',ax1,'LineStyle','-');
      end
      set(P,'LineWidth',2.2);
      set(ax1,'YAxisLocation','left')
      set(ax1,'XLim',[min(sirf),max(sirf)]);
      set(ax1,'YLim',[minth,maxth]);
      %  set(gca,'XTickLabel',intx);
      set(ax1,'FontSize',8);    

      %subplot(2,1,2); ax2 = gca;
      ax2 = axes('Position',get(ax1,'Position'),'Color','none');
      P = line(sirf,theta(:,p));
      if isnan(lambdas)
        set(P,'Color',[0  0 0]);
      else
        set(P,'Color',[0  0 0]);
      end        
      set(P,'LineWidth',1,'Parent',ax2,'LineStyle','-');
      set(ax2,'YAxisLocation','right')
      set(ax2,'YLim',[minth,maxth]);
      %set(ax2,'YTick',[(0:1:max(plgr))]);
      set(ax2,'XLim',[min(sirf),max(sirf)]);
      set(ax2,'FontSize',8);    


      figure(200+p);
      ng=strcat(gpath_3,strcat(num2str(p),'-PrioPost'));
      print(char(strcat(ng,'.eps')),'-depsc2');   
      print(char(strcat(ng,'.pdf')),'-dpdf');   
      close(figure(200+p))

      thp = theta(:,p);
      pthp = priotheta(:,p);

      if p >= 33
        thp(isnan(pthp) ) = [];
        pthp(isnan(pthp) ) = [];
        mean(pthp)
        std(pthp)

        Indt = ( abs(thp-mean(thp)) > 2*std(thp) );
        Indpt = ( abs(pthp-mean(pthp)) > 2*std(pthp) );
        thp(Indt|Indpt) = [];
        pthp(Indt|Indpt) = [];
      end      
      figure(1000+p)
      [N1,X1] = hist([thp,pthp],20);
      N1 = N1./repmat(sum(N1,1),[size(N1,1) 1]);
      bar(X1,N1);
      colormap([[.3 .3 .3];[.8 .8 .8]]);
      title(para_names{p});
      set(gca,'YLim',[0 max(max(N1))]);
      ng=strcat(gpath_2,strcat(num2str(p),'-Hist'));
      figure(1000+p);
      print(char(strcat(ng,'.pdf')),'-dpdf');   % print(char(strcat(ng,'.eps')),'-depsc2');             

      figure(1000)
      subplot(round(length(find(para_mask == 0))/2),2,Ip);
      bar(X1,N1);
      colormap([[.3 .3 .3];[.8 .8 .8]]);
      set(gca,'Fontsize',6);
      set(gca,'YLim',[0 max(max(N1))]);
      %set(gca,'XLim',[min(X1) max(X1)]);
      Ob = ylabel(para_names{p});
      set(Ob,'Fontsize',10);
      
      
      figure(2000+p)
      ind = ((nsim/40):(nsim/40):(nblocks-1)*nsim);
      thetasc = theta(ind,p);
      priothetasc = priotheta(ind,p);
      Iq = 0;
      p_List = [10 16 26 32];%find(para_mask == 0)'; make sure the list does not have parameters that are being fixed
      for q = p_List
        Iq = Iq+1;
        p_thetasc = theta(ind,q);
        p_priothetasc = priotheta(ind,q);

        subplot(ceil(length(p_List)/2),2,Iq)

        xminth = min(min(thetasc),min(priothetasc));
        xmaxth = max(max(thetasc),max(priothetasc));
        yminth = min(min(p_thetasc),min(p_priothetasc));
        ymaxth = max(max(p_thetasc),max(p_priothetasc));
        ax1 = gca;
        P = line(thetasc,p_thetasc);
        set(P,'Color',[0 0 0]);
        set(ax1,'XLim',[xminth,xmaxth]);
        set(ax1,'YLim',[yminth,ymaxth]);
        set(P,'Marker','.','MarkerSize',7,'LineStyle','none','Parent',ax1);

        ax2 = axes('Position',get(ax1,'Position'),'Color','none');
        P = line(priothetasc,p_priothetasc);
        set(P,'Color',[.5 .5 .5]);
        set(ax2,'XLim',[xminth,xmaxth]);
        set(ax2,'YLim',[yminth,ymaxth]);
        set(P,'Marker','.','MarkerSize',7,'LineStyle','none','Parent',ax1);

        title(strcat('param = ',para_names{q}));
        
      end
      subtitle(para_names{p});
      figure(2000+p);
      ng=strcat(gpath_2,strcat(num2str(p),'-Scat'));
      print(char(strcat(ng,'.eps')),'-depsc2');             
      print(char(strcat(ng,'.pdf')),'-dpdf');   
    end
    fclose(fid01);		% close output file

    figure(1000);
    ng=strcat(gpath_2,'AllHist');
    print(char(strcat(ng,'.pdf')),'-dpdf');   % print(char(strcat(ng,'.eps')),'-depsc2');             
    
  end

  
  %%%%%%%%%%% Compute posterior moments of the parameters
  thetahat = mean(theta,1)';
  thetasig = cov(theta,1);

  thetabands = hpdint(theta,percent,1)';

  %% save the mean
  outfile0 = strcat(spath,lmodel,lprior,'para',num2str(coint0),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),'Mean');

  fid0 = fopen(outfile0,'w');     % output file (set to write)   
  fwrite(fid0,thetahat','single');
  fclose(fid0);
  
  colnames = {'Parameter ' 'Prior Mean '  'Prior Stdd '  'Post Mean ' strcat(num2str(100*percent),'\% {\tiny Lower Band}') ...
              strcat(num2str(100*percent),'\% {\tiny Upper Band}') };
  outmat = [pmean,pstdd,thetahat,thetabands];

  Theta1 = [Theta1,thetahat];
  Theta2 = [Theta2,thetahat,thetabands];

  
  fprintf(1,'\n');
  fprintf(1,'%4.99s  ',colnames{:});
  for i = 1:length(para_names)
    fprintf(1,'\n $%4.99s$ & ',para_names{i});
    fprintf(1,' %8.3f & ',outmat(i,:));
  end    
  

  fprintf(fid00,'\n \\begin{table}[h] \\centering');        
  fprintf(fid00,'\n \\caption{Parameter Estimates, $\\lambda$ = %2.1f } \n\\vspace*{.5cm}',lambdas);
  fprintf(fid00,'\n \\begin{tabular}{lllllll}\\hline \n');

  fprintf(fid00,' %4.99s  & ',colnames{:});
  for i = 1:length(para_names)
    fprintf(fid00,'\\\\ \n $%4.99s$ &  ',para_names{i});
    fprintf(fid00,' %8.3f & ',outmat(i,:));
  end   
  fprintf(fid00,'\\\\  \\hline');		    
  fprintf(fid00,'\n \\end{tabular}');
  fprintf(fid00,'\n \\end{table} \n\n');
  
  clear parasimm postsim;

  
  
  
  
end         % end lambdaseq
fprintf(1,'\n');



fprintf(fid00,'\n \\begin{table}[h] \\centering');        
fprintf(fid00,'\n \\caption{Parameter Estimates: Prior and Posterior Mean} \n\\vspace*{.5cm}',lambdas);
fprintf(fid00,'\n \\begin{tabular}{cl');
for ic = 1:length(lambdaseq)
  fprintf(fid00,'l');
end
fprintf(fid00,'}\\hline \n');
fprintf(fid00,' Paramter & Prior ');
for ic = 1:length(lambdaseq)
  fprintf(fid00,' & $\\lambda$ = %2.1f  ',lambdaseq(ic));
end
fprintf(fid00,'\\\\ \\hline \n');

for i = 1:length(para_names)
  fprintf(fid00,' \n $%4.99s$   ',para_names{i});
  fprintf(fid00,' & %8.3f  ',Theta1(i,:));
  fprintf(fid00,'\\\\ ');
end   
fprintf(fid00,'\\\\  \\hline');		    
fprintf(fid00,'\n \\end{tabular}');
fprintf(fid00,'\n \\end{table} \n\n');


fprintf(fid00,'\n \\end{document}');

fclose(fid00);		% close output file
