function obj = objfcninf(para,YYYY,XXYY,XXXX,nobs,lambda,nlags,nvar,mspec,npara,...
   trspec,pmean,pstdd,pshape,para_mask,para_fix,marglh,coint,cointadd,cointall,mspec_ref,params_1bar,YYYYd,XXYYd,XXXXd,nobsd,para_aux,lambdac,nlagsc,addc,bounds,MIN);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% obj evaluates the log  posterior = log likelihood + log prior
%% MIN takes into account that csminwel minimizes as opposed to max, and that the parameters are rescaled
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% likelihood 
if MIN
    para = trans(para,trspec);
end
para = para.*(1-para_mask)+para_fix.*para_mask;

% Evaluate the Prior distribution

if isempty(mspec_ref)
  lnprio  = priodens(para,pmean,pstdd,pshape);
elseif ~isempty(mspec_ref)
  %% Fix non-auxiliary parameters
  para_ = para.*para_aux+params_1bar.*(1-para_aux);
  [TTT_,RRR_,valid_] = dsgesolv(para_,mspec);

  [yyyyd_,xxyyd_,xxxxd_,retcode] = varprio_dmean(TTT_,RRR_,valid_,para_,nvar,nlagsc,mspec,npara,coint,cointadd,cointall);
  [lnprio,likenew_] = feval(strcat('objfcnmh',addc),para,bounds,YYYYd,XXYYd,XXXXd,nobsd,lambdac,...
                            nlagsc,nvar,mspec,npara,trspec,pmean,pstdd,pshape,yyyyd_,xxyyd_,xxxxd_,retcode,para_mask,coint,cointadd,cointall);
end

% Solve the model and evaluate the likelihood

[TTT,RRR,valid] = dsgesolv(para,mspec);
[yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint,cointadd,cointall);

if retcode == 0

    % dummy observations are invalid
    lnpy = -1E11;

else

    % dummy observations are valid
    
    % remove cointadd component
    xxxxd = xxxxd(1+cointadd:end,1+cointadd:end);
    XXXX  = XXXX(1+cointadd:end,1+cointadd:end);
    
    xxyyd = xxyyd(1+cointadd:end,:);
    XXYY  = XXYY(1+cointadd:end,:);
    
    % evaluate restricted GAUSSIAN likelihood
    phis = xxxxd \ xxyyd;
    sigs = yyyyd -xxyyd'*phis;
    lnpy = -(nvar*(nobs)/2)*log(2*pi)-(nobs/2)*log(det(sigs)) ...
           -.5*sum(diag( sigs \ (YYYY-phis'*XXYY-XXYY'*phis+phis'*XXXX*phis)));

%    sighat = (YYYY - XXYY'*inv(XXXX)*XXYY)/nobs;
    
end

if MIN 
    obj = real(-lnpy-lnprio);  % We minize the inverse of the likelihood fcn 
else 
    obj = real(lnpy+lnprio);  
end
