function obj = objfcnprio(para,YYYY,XXYY,XXXX,nobs,nlags,nvar,mspec,npara,...
                          trspec,pmean,pstdd,pshape,para_mask,para_fix,marglh,coint,cointadd,cointall,MIN);
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% obj evaluates the log  posterior = log likelihood + log prior
  %% MIN takes into account that csminwel minimizes as opposed to max, and that the parameters are rescaled
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  %% likelihood 
  if MIN
    para = trans(para,trspec);
  end
  para = para.*(1-para_mask)+para_fix.*para_mask;

  %% Evaluate the Prior distribution

  lnprio = priodens(para,pmean,pstdd,pshape);



  %% moments from data
  stmoms;

  %% computes moments from model
  M_vec = getparaM(para,mspec);
 
  lnprioM = - sum( ( (M_vec-M_mean).^2 )./M_var );
  
  
  lnpy = 0; %% this is the dummy obs part
  if ~isnan(nobs)

    %% Solve the model and evaluate the (dummy) likelihood
    
    [TTT,RRR,valid] = dsgesolv(para,mspec);
    [yyyyd,xxyyd,xxxxd,retcode] = varprio_dmean(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint,cointadd,cointall);


    % remove cointadd component
    %xxxxd = xxxxd(1+cointadd:end,1+cointadd:end);
    %XXXX  = XXXX(1+cointadd:end,1+cointadd:end,:);

    if retcode == 0

      % dummy observations are invalid
      lnpy = -1E11;

    else

      % dummy observations are valid
      
      for i_s = 1:size(YYYY,3);
        
        
        if nlags == 0
          yyyyd = diag(diag(yyyyd));
          YYYY = diag(diag(YYYY));

          sigs = yyyyd;
      
          fnfix = find(~para_mask);
          deter = zeros(nvar,nvar);
          i_0 = 0;
          for i_d = fnfix(end-nvar+1:end)'
            i_0 = i_0+1;
            para_ = para;
            para_(33:end) = 0;
            para_(i_d) = 1;
            [yyyyd_,xxyyd_,xxxxd_,retcode_] = varprio_dmean(TTT,RRR,valid,para_,nvar,nlags,mspec,npara,coint,cointadd,cointall);
            deter(:,i_0) = diag(yyyyd_);
          end

          lnpy =  lnpy -(nvar*(nobs)/2)*log(2*pi)-(nobs/2)*log(det(sigs)) ...
                 -.5*sum(diag( sigs \ YYYY(:,:,i_s)))+log(det(2*deter));

          
        else
          
          
          
          xxyyd = xxyyd(1+cointadd:end,:);
          XXYY  = XXYY(1+cointadd:end,:,:);
          
          % evaluate restricted GAUSSIAN likelihood
          phis = xxxxd \ xxyyd;
          sigs = yyyyd -xxyyd'*phis;

          lnpy = lnpy -(nvar*(nobs)/2)*log(2*pi)-(nobs/2)*log(det(sigs)) ...
                 -.5*sum(diag( sigs \ (YYYY(:,:,i_s)-phis'*XXYY(:,:,i_s)-XXYY(:,:,i_s)'*phis+phis'*XXXX(:,:,i_s)*phis)));

          %    sighat = (YYYY - XXYY'*inv(XXXX)*XXYY)/nobs;

        end
        
      end
      
    end
  end
  

  if MIN 
    obj = real(-lnpy-lnprio-lnprioM);  % We minize the inverse of the likelihood fcn 
  else 
    obj = real(lnpy+lnprio+lnprioM);  
  end
