function obj = objfcnprio(para,yyyy,xxyy,xxxx,nobs,nlags,nvar,mspec,npara,...
                          trspec,pmean,pstdd,pshape,para_mask,para_fix,marglh,coint,cointadd,cointall,MIN);
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% obj evaluates the log  posterior = log likelihood + log prior
  %% MIN takes into account that csminwel minimizes as opposed to max, and that the parameters are rescaled
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  %% likelihood 
  if MIN
    para = trans(para,trspec);
  end
  para = para.*(1-para_mask)+para_fix.*para_mask;

  %% Evaluate the Prior distribution
  lnprio = priodens(para,pmean,pstdd,pshape);


  %% moments from data
  stmoms;
  
  %% computes moments from model
  M_vec = getparaM(para,mspec);
  lnprioM = - sum( ( (M_vec-M_mean).^2 )./M_var );
  
  
  lnpy = 0; %% this is the dummy obs part
  if ~isnan(nobs)
    
    %% Solve the model and evaluate the (dummy) likelihood
    
    [TTT,RRR,valid] = dsgesolv(para,mspec);
    [yyyyd,xxyyd,xxxxd,retcode] = varprio_dmean(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint,cointadd,cointall);


    % remove cointadd component
    %xxxxd = xxxxd(1+cointadd:end,1+cointadd:end);
    %xxxx  = xxxx(1+cointadd:end,1+cointadd:end,:);

    if retcode == 0

      % dummy observations are invalid
      lnpy = -1E11;

    else

      % dummy observations are valid
      
      for i_s = 1:size(yyyy,3);
        
        
        if nlags == 0
          %yyyyd = diag(diag(yyyyd));
          %yyyy = diag(diag(yyyy));

          sigs = yyyyd;
      
          lnpy =  lnpy -(nvar*(nobs)/2)*log(2*pi)-(nobs/2)*log(det(sigs)) ...
                 -.5*sum(diag( sigs \ yyyy(:,:,i_s)));

          
        else
          
          
          
          xxyyd = xxyyd(1+cointadd:end,:);
          xxyy  = xxyy(1+cointadd:end,:,:);
          
          % evaluate restricted GAUSSIAN likelihood
          phis = xxxxd \ xxyyd;
          sigs = yyyyd -xxyyd'*phis;

          lnpy = lnpy -(nvar*(nobs)/2)*log(2*pi)-(nobs/2)*log(det(sigs)) ...
                 -.5*sum(diag( sigs \ (yyyy(:,:,i_s)-phis'*xxyy(:,:,i_s)-xxyy(:,:,i_s)'*phis+phis'*xxxx(:,:,i_s)*phis)));

          %    sighat = (yyyy - xxyy'*inv(xxxx)*xxyy)/nobs;

        end
        
        %para(26:end)
        
      end
      
    end
  end
  

  if MIN 
    obj = real(-lnpy-lnprio-lnprioM);  % We minize the inverse of the likelihood fcn 
  else 
    obj = real(lnpy+lnprio+lnprioM);  
  end
