close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
run startdsswfed;
!hostname;
!ps -C matlab;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para, ...
 varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode, ...
 stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix,para_aux,mspec_ref,nobsdrat,lambdac,nlagsc,diago] = varspec(chdata);

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);

Iseq = find(ti == dates);
I = Iseq;
if stime                           
  YY = YYall(I-stime+1:I,:);
  XX = XXall(I-stime+1:I,:);             
  tiI = ti(I-stime+1:I);
  
  YY_p = YYall(1:I-stime,:);
  XX_p = XXall(1:I-stime,:);             
  nobs_p = size(YY_p,1)
  
else
  YY     = YYall(1:I,:);
  XX     = XXall(1:I,:);                   
  tiI = ti(1:I);
end
nobs = size(YY,1);    

k     = nlags*nvar;

YYYY = YY'*YY;
XXYY = XX'*YY;
XXXX = XX'*XX;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
percent = .90;

dataset = num2str(10*dataset);

mspecno = [3 1];
mspeclist = [106,105,106];
mspec_ref_list = cell(length(mspeclist),1);
mspec_ref_list(1) = {[]};
mspec_ref_list(2) = {[]};
mspec_ref_list(3) = {[]};

nobsdrat_list = cell(length(mspeclist),1);
nobsdrat_list(1) = {[10/100]};
nobsdrat_list(2) = {[10/100]};
nobsdrat_list(3) = {[10/100]};

lambdac_list = cell(length(mspeclist),1);
lambdac_list(1) = {[Inf]};
lambdac_list(2) = {[Inf]};
lambdac_list(3) = {[Inf]};

nlagsc_list = cell(length(mspeclist),1);
nlagsc_list(1) = {[1]};
nlagsc_list(2) = {[1]};
nlagsc_list(3) = {[1]};

diago_list = cell(length(mspeclist),1);
diago_list(1) = {[0]};
diago_list(2) = {[0]};
diago_list(3) = {[0]};

mprior_list = cell(length(mspeclist),1);
mprior_list(1) = {[2]};
mprior_list(2) = {[1]};
mprior_list(3) = {[1]};

%% if = 1 use prior, = 0 use posterior
mPostPrio_list = cell(length(mspeclist),1);
mPostPrio_list(1) = {[0]};
mPostPrio_list(2) = {[0]};
mPostPrio_list(3) = {[1]};

modls = 'm';
for ispec = mspecno 
  mspec = mspeclist(ispec);
  modls = strcat(modls,num2str(mspec));
  modls = strcat(modls,num2str(mprior_list{ispec}));  
  if ~isempty(mspec_ref_list{ispec})
    modls = strcat(modls,'-',num2str(mspec_ref_list{ispec}));
    modls = strcat(modls,'nobsd',num2str(100*nobsdrat_list{ispec}));
    if ~isinf(lambdac_list{ispec})
      modls = strcat(modls,'lam',num2str(lambdac_list{ispec}));
    end  
    modls = strcat(modls,'l',num2str(nlagsc_list{ispec}));
    if diago_list{ispec}
      modls = strcat(modls,'d',num2str(diago_list{ispec}));
    end
  end    
  modls = strcat(modls,'---');  
end

Acolor = zeros(length(mspeclist),3);
Lstyle = cell(length(mspeclist),1);
Mstyle = cell(length(mspeclist),1);
Lstyle(1) = {'-'};
Mstyle(1) = {'+'};
Acolor(2,:) = [0 .2 .2];
Lstyle(2) = {'--'};
Mstyle(2) = {'^'};
Acolor(3,:) = [0 .5 .5];
Lstyle(3) = {'-.'};
Mstyle(3) = {'o'};


outfile00 = strcat(fpath,modls,lprior,'PrioCorr.tex');		% output file name 
fid00 = fopen(outfile00,'w');		% output file (set to write)
fprintf(fid00,'\n \\documentclass[12pt]{article} \n \\usepackage[dvips]{color}\n ');
fprintf(fid00,'\n \\begin{document}');

whichlist = 2
if whichlist == 1
  Listnames = {'ZSh','PhiSh','GSh','LaFSh'}


  Listall1(1) = {26};
  Listall2(1) = {33};

  Listall1(2) = {27};
  Listall2(2) = {34};

  Listall1(3) = {32};
  Listall2(3) = {39};
  
  Listall1(4) = {29};
  Listall2(4) = {36};
elseif whichlist == 2
  Listnames = {'CorrIndPiRhoLaFSh','CorrIndWRhoLaFSh','CorrIndPIndW'}


  Listall1(1) = {29};
  Listall2(1) = {3};

  Listall1(2) = {29};
  Listall2(2) = {13};

  Listall1(3) = {3};
  Listall2(3) = {13};

end


xminth = zeros(10,length(Listnames)); 
xmaxth = zeros(10,length(Listnames));
yminth = zeros(10,length(Listnames)); 
ymaxth = zeros(10,length(Listnames));

xminth_ = zeros(10,length(Listnames)); 
xmaxth_ = zeros(10,length(Listnames));
yminth_ = zeros(10,length(Listnames)); 
ymaxth_ = zeros(10,length(Listnames));

xminth_(:,1) = -inf;
xmaxth_(:,1) = inf;
yminth_(:,1) = -inf;
ymaxth_(:,1) = inf;
xminth_(:,2) = -inf;
xmaxth_(1:3,2) = inf;
yminth_(:,2) = -inf;
ymaxth_(1:3,2) = inf;
xminth_(:,3) = -inf;
xmaxth_(:,3) = inf;
yminth_(:,3) = -inf;
ymaxth_(:,3) = inf;



for ispec = mspecno 
  mspec = mspeclist(ispec)
  lmodel    = strcat('m',num2str(mspec));

  mprior = mprior_list{ispec}
  lprior    = num2str(mprior);
  
  mspec_ref = mspec_ref_list{ispec}
  nobsdrat = nobsdrat_list{ispec}
  lambdac = lambdac_list{ispec}
  nlagsc = nlagsc_list{ispec}
  diago = diago_list{ispec}
  if isempty(mspec_ref)
    lmodel_ref = [];
    nobsdrat_ = [];
    lambdac_ = [];
    nlagsc_ = [];
    diago_ = [];
    
    lprior    = num2str(2);
    
  else
    lmodel_ref = strcat('m',num2str(mspec_ref));
    nobsdrat_ = num2str(100*nobsdrat);
    if isinf(lambdac)
      lambdac_ = [];
    else
      lambdac_ = num2str(100*lambdac);
    end
    if nlagsc == 4
      nlagsc_ = [];
    else
      nlagsc_ = num2str(nlagsc);
    end  
    if ~diago
      diago_ = [];
    else
      diago_ = num2str(diago);
    end
    
    lprior    = num2str(1);
    
  end

  if mPostPrio_list{ispec} == 1 %% use prior

    if isempty(mspec_ref)

      %% w/o correcting model - already discarded nburns
      infile1 = strcat(spath,'/mhNIpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc_,diago_)
      fid1 = fopen(infile1,'r');		% input file (set to read)   
      para = fread(fid1,[npara,inf],'single')';
      fclose(fid1);

      infile2 = strcat(spath,'/priodet',lmodel,lprior,dataset);
      fid2 = fopen(infile2,'r');		% input file (set to read)   
      detsim = fread(fid2,[1,inf],'int8')';
      fclose(fid2);

      %% discard indeterminacy
      ndraws = size(para,1)
      para(find(detsim),:) = [];
      fprintf(1,'\n\n  ratio of determinacy draws %2.2f\n',size(para,1)/ndraws);
      ndraws = size(para,1)

    else
      %% w/ correcting model -  have to  discard nburns
      
      infile1 = strcat(spath,'/mhNIpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc_,diago_)
      fid1 = fopen(infile1,'r');		% input file (set to read)   
      numb1 = nburn*npara*4;                  % number of bites to discard
      status = fseek(fid1,numb1,'bof');       % discard nburn 
      para = fread(fid1,[npara,inf],'single')';
      fclose(fid1);

      
      ndraws = size(para,1)


    end
    
  elseif mPostPrio_list{ispec} == 0 %% use posterior
    
    lambdas = lambdaseq     
    if isnan(lambdas)
      coint0 = coint;
    else
      coint0 = cointall;
    end
    fixparam = fixparamseq(1);

    
    infile1 = strcat(spath,'/mhpara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc_,diago_,num2str(coint0),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		

    fid1 = fopen(infile1,'r');		% input file (set to read)   
    numb1 = nburn*npara*4;                  % number of bites to discard
    status = fseek(fid1,numb1,'bof');       % discard nburn 
    para = fread(fid1,[npara,inf],'single')';
    fclose(fid1);

    
    ndraws = size(para,1)

  end
  
  stepp = 40;
  ind = ((nsim/stepp):(nsim/stepp):size(para,1));
  statssc = para(ind,:);
  %clear stats;






  for ll = 1:length(Listall1)

    q1_List = Listall1{ll};
    q2_List = Listall2{ll};

    tott = length(q1_List);
    if any(ll == [1,2,3])
      siz1 = tott;
      siz2 = 1;
    end
    
    for Iq = 1:tott

      q1 = q1_List(Iq);
      q1_statssc = statssc(:,q1);

      q2 = q2_List(Iq);
      q2_statssc = statssc(:,q2);


      figure(2000+ll)
      Z = subplot(siz1,siz2,Iq);

      if ispec == mspecno(1)
        xminth(Iq,ll) = .99*min(q1_statssc);
        xmaxth(Iq,ll) = 1.01*max(q1_statssc);
      end

      xminth0 = .99*min(q1_statssc);
      xmaxth0 = 1.01*max(q1_statssc);
      xminth(Iq,ll) = min(xminth(Iq,ll),xminth0);
      xmaxth(Iq,ll) = max(xmaxth(Iq,ll),xmaxth0);

      if ispec == mspecno(1)
        yminth(Iq,ll) = .99*min(q2_statssc);
        ymaxth(Iq,ll) = 1.01*max(q2_statssc);
      end

      yminth0 = .99*min(q2_statssc);
      ymaxth0 = 1.01*max(q2_statssc);
      yminth(Iq,ll) = min(yminth(Iq,ll),yminth0);
      ymaxth(Iq,ll) = max(ymaxth(Iq,ll),ymaxth0);
      
      
      ax1 = gca;
      P = line(q1_statssc,q2_statssc);
      set(P,'Color',Acolor(ispec,:));
      set(P,'Marker',Mstyle{ispec},'MarkerSize',7,'LineStyle','none','Parent',ax1);
      
      set(ax1,'XLim',[xminth(Iq,ll),xmaxth(Iq,ll)]);
      set(ax1,'YLim',[yminth(Iq,ll),ymaxth(Iq,ll)]);

      
      T = xlabel(para_names{q1});
      set(T,'FontSize',16);
      T = ylabel(para_names{q2});
      set(T,'FontSize',16);
      if whichlist == 1

        T = title(names_shocks{ll});
        set(T,'FontSize',12);
      end

      
      
      hold off;
      

    end
  end

end %% end mspec list


for ll = 1:length(Listall1)
  figure(2000+ll);
  ng=strcat(gpath,modls,'PrioCheck',Listnames{ll});
  print(char(strcat(ng,'.eps')),'-depsc2');             
  %print(char(strcat(ng,'.pdf')),'-dpdf');   
end



fprintf(fid00,'\n \\end{document}');

fclose(fid00);		% close output file
