% filename:    prio.m
% description: generates draws from the prior and computes prior moments for the parameters of interest
%              marginal log likelihood
%              with respect to the DSGE model parameters
close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
run startdsswfed;
!hostname;
!ps -C matlab;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varspec;

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);

if isempty(mspec_ref)
  lmodel_ref = [];
else
  lmodel_ref = strcat('m',num2str(mspec_ref));   
end
stup_ = num2str(stup);

%% set of parameters that are subject to this procedure
list_t1 = find(para_type == 1);
npara1 = length(list_t1)

para_names{list_t1}

if isempty(list_t1)
  disp('No params of type 1');
  return;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,mspec);
Iseq = find(ti == dates);
I = Iseq;

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
if isempty(mspec_ref)
  priorfile = strcat('pri',lprior)
elseif ~isempty(mspec_ref)
  priorfile = strcat('pri',lprior)
end
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);

pmean = pmean.*(1-para_mask)+para_fix.*para_mask;

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% doing rolling sample estimation?
ROLL = 0;
SAVEALL = 0;
DOFORECAST = 0;
if ROLL 
  DOFORECAST = 1;
  SAVEALL = 0;
end
fprintf(1,'\n\n ROLL: %2.0f, SAVEALL: %2.0f, DOFORECAST: %2.0f\n',[ROLL,SAVEALL,DOFORECAST]);
start_period = stime;                               % minimum # quarters to compute VAR estimates 


%mkdir(spath,'/2/');
%spath = strcat(spath,'/1/');
%start_period = nobsall-qahead+1-3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% CHAnGE

trial_period = nobsall-start_period-qahead+1;      % # quarters to compute RMSE


I = find(ti == dates);

%% if JustMode: do not do simulations and compute forecasts at mode
if JustMode
  JustMode
  break;
end

Verbose = 0;

fixparam = fixparamseq(1);
if fixparam ~= 0 
  fixparam
  break
end

outfile00 = strcat(fpath,lmodel,lprior,dataset,lmodel_ref,stup_,'PrioMomt1.tex');		% output file name 
fid00 = fopen(outfile00,'w');		% output file (set to write)

fprintf(fid00,'\n \\documentclass[12pt]{article} \n \\usepackage[dvips]{color}\n ');
fprintf(fid00,'\n \\begin{document}');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do Lambda loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% open write files
%% params file
outfile1 = strcat(spath,'/mhBprioparat1',lmodel,lprior,dataset,lmodel_ref,stup_);		
outfile2 = strcat(spath,'/priobound',lmodel,lprior,dataset,lmodel_ref,stup_);		
outfile3 = strcat(spath,'/mhprioparat1',lmodel,lprior,dataset,lmodel_ref,stup_);   


addblocks = 0;
DOD = 1%input('\n\n Skip draws from prior? (0 = just skip, else produce draws)');
if DOD
  fid1 = fopen(outfile1,'w');		% output file (set to write)   
  for iblock = 1:(nblocks+addblocks)  

    parasim = zeros(nsim,npara1);
    
    for i0 = 1:npara1

      i = list_t1(i0);

      if pshape(i) == 1 %% BETA Prior  
        a = (1-pmean(i))*pmean(i)^2/pstdd(i)^2 - pmean(i);
        b = a*(1/pmean(i) - 1);
        parasim(:,i0) = betarnd(a,b,nsim,1);
        
      elseif pshape(i) == 2 %% GAMMA PRIOR  
        b = pstdd(i)^2/pmean(i);
        a = pmean(i)/b;
        parasim(:,i0) = b*gamrnd(a,1,nsim,1);

      elseif pshape(i) == 3 %% GAUSSIAN PRIOR  
        a = pmean(i);
        b = pstdd(i);
        parasim(:,i0) = a + b*randn(nsim,1);

      elseif pshape(i) == 4 %% INVGAMMA PRIOR  
        a = pmean(i);
        b = pstdd(i);
        parasim(:,i0) = sqrt((b*a^2)./sum(randn(nsim,b).^2,2));

      elseif pshape(i) == 5 %% uniform prior  
        a = pmean(i);
        b = pstdd(i);
        parasim(:,i0) = a + (b-a)*rand(nsim,1);

      elseif pshape(i) == 0 %% No Prior - Fixed  
        a = pmean(i);
        parasim(:,i0) = a*ones(nsim,1);

      end
      
    end
    
    fwrite(fid1,parasim','single');
  end;		
  num1 = ftell(fid1);
  fclose(fid1);


  infile10 = outfile1;
  fid10 = fopen(infile10,'r');		% input file (set to read)   
  num10 = (nblocks+addblocks)*nsim*npara1*4;            % number of bites per file
  numb10 = nburn*npara1*4;                  % number of bites to discard
                                           %status = fseek(fid10,numb10,'bof');       % discard nburn 


  fid2 = fopen(outfile2,'w');		% output file (set to write)   
  fid3 = fopen(outfile3,'w');		% output file (set to write)   

  Tot = 0;
  Tot_all = nsim*nblocks - nburn
  
  iblock = 0;
  while ( Tot < Tot_all )
    % Read blocks of size nsim
    iblock = iblock+1
    needd = Tot_all - Tot
    
    
    if ftell(fid10) >= num10; disp('problem -- need more draws'); mm; end;
    
    parasim = fread(fid10,[npara1,nsim],'single')';
    boundsim      = zeros(nsim,1);

    for j = 1:nsim
      paradraw = parasim(j,:)';

      %% check whether draws are out of boundaries
      ind1 = (paradraw > bounds(list_t1,2));
      ind2 = (paradraw < bounds(list_t1,1));
      if any(ind1) | any(ind2)
        boundsim(j) = 1;
      end

    end

    fprintf(1,'\n\n  Block, %2.0f; ratio of out of bound draws %2.2f\n',[iblock,length(find(boundsim))/nsim]);

    parasim(find(boundsim),:) = [];%NaN; 
    
    if needd < sum(~boundsim) 
      parasim = parasim(1:needd,:);
      Tot = Tot_all;    
    else
      Tot = Tot+sum(~boundsim);
    end
    
    
    fwrite(fid3,parasim','single');
    fwrite(fid2,boundsim,'int8');
    
  end;
  fclose(fid10);
  fclose(fid3);
  fclose(fid2);
  
end

delete(outfile1);

infile30 = outfile3;
fid30 = fopen(infile30,'r');		% input file (set to read)   
para1 = fread(fid30,[npara1,inf],'single')';
fclose(fid30);

infile20 = outfile2;
fid20 = fopen(infile20,'r');		% input file (set to read)   
boundsim = fread(fid20,[1,inf],'int8')';
fclose(fid20);

ndraws = size(para1,1)

%% discard indeterminacy
%para(find(boundsim),:) = [];


para1hat = mean(para1,1)';
para1sig = cov(para1,1);
para1bands = hpdint(para1,percent,1)';


%% save mean
outfile0 = strcat(spath,'/mhprioparat1',lmodel,lprior,dataset,lmodel_ref,stup_,'Mean')
fid0 = fopen(outfile0,'w');     % output file (set to write)   
fwrite(fid0,para1hat','single');
fclose(fid0);


fprintf(fid00,'\n \\documentclass{article}');
fprintf(fid00,'\n \\begin{document}');
fprintf(fid00,'\n \\setlength{\\tabcolsep}{.7mm}');

fprintf(1,'\n\n  ratio of out of bounds draws %2.2f\n',size(find(boundsim))/size(boundsim))
fprintf(fid00,'\n\n  ratio of out of bounds draws %2.2f\n',size(find(boundsim))/size(boundsim))


fprintf(1,'%4.99s  ','Mean');
for i0 = 1:npara1;
  i = list_t1(i0);
  fprintf(1,'\n %4.10s  ',para_names{i});
  fprintf(1,' %8.4f  ',para1hat(i0,:));
end    

outmat = [para1hat,sqrt(diag(para1sig)),para1bands];

fprintf(fid00,'\n');


colnames = {'Parameter' 'Support '  'Density '  'Prior Mean ' 'Prior Stdd ' strcat(num2str(100*percent),'\% LB') ...
            strcat(num2str(100*percent),'\% UB') };

[nrows,ncols] = size(outmat);

fprintf(fid00,'PARAMETERS \n $$');
fprintf(fid00,'\n \\vspace{.2cm} \n \\begin{array}{|@{\\hspace*{.5cm}} l @{\\hspace*{.5cm}}|@{\\hspace*{-.2cm}}c@{\\hspace*{-.2cm}}|@{\\hspace*{-.2cm}}c@{\\hspace*{-.2cm}}|');
FORMAT = '  ';
for j = 1:ncols
  fprintf(fid00,' @{\\hspace*{.5cm}} c'); 
  if j == 1
    FORMAT = strcat(FORMAT,' &  %8.3f  '); 
    % elseif (j == ncols) | (j == (ncols-1))
    %	FORMAT = strcat(FORMAT,' & \\mbox{{\\scriptsize $%8.3f$ }} '); 
  else
    FORMAT = strcat(FORMAT,' & %8.3f '); 
  end

end

fprintf(fid00,'|}\\hline');
FORMAT = strcat(FORMAT,' \\\\');  

fprintf(fid00,'\n  ');
fprintf(fid00,' %4.99s  & ',colnames{:});		
fprintf(fid00,'	\\\\ \\hline '); 

for i0 = 1:nrows
  i = list_t1(i0);
  fprintf(fid00,'\n %4.99s & ',para_names{i});
  if pshape(i) == 1
    fprintf(fid00,'[0,1) & \\mbox{Beta} ');	
  elseif pshape(i) == 2
    fprintf(fid00,'R^{+} & \\mbox{Gamma} ');	
  elseif pshape(i) == 3
    fprintf(fid00,' R & \\mbox{\\cal N} ');	
  elseif pshape(i) == 4
    fprintf(fid00,'R^{+} & \\mbox{InvGamma} ');	
  elseif pshape(i) == 0
    fprintf(fid00,' &  ');	    
  end
  fprintf(fid00,FORMAT,outmat(i0,:));

end   
fprintf(fid00,' \\hline');
fprintf(fid00,'\n \\end{array} $$');        
fprintf(fid00,'\n \\end{document}');


fclose(fid00);		% close output file


