% filename:    prio.m
% description: generates draws from the prior and computes prior moments for the parameters of interest
%              marginal log likelihood
%              with respect to the DSGE model parameters
close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
run startdsswfed;
!hostname;
!ps -C matlab;
rand('state',sum(100*clock))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para, ...
 varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode, ...
 stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix,para_aux,mspec_ref,nobsdrat,lambdac,nlagsc,diago] = varspec(chdata);

%[fpath,spath,gpath] = pathspecmh(num2str(floor(dataset)),chdata,coint);
[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);

if isempty(mspec_ref)
  lmodel_ref = [];
  nobsdrat_ = [];
  nobsdrat_2 = [];
  lambdac_ = [];
  nlagsc_ = [];
  diago_ = [];

  nlagsc2 = nlagsc;
  if nlagsc2 == 4
    nlagsc2_ = [];
  else
    nlagsc2_ = num2str(nlagsc2);
  end  
  
else
  lmodel_ref = strcat('m',num2str(mspec_ref));
  nobsdrat_ = num2str(nobsdrat);
  nobsdrat_2 = num2str(100/nobsdrat);
if isinf(lambdac)
    lambdac_ = [];
  else
    lambdac_ = num2str(100*lambdac);
  end
  if nlagsc == 4
    nlagsc_ = [];
  else
    nlagsc_ = num2str(nlagsc);
  end  
  if ~diago
    diago_ = [];
  else
    diago_ = num2str(diago);
  end

  nlagsc2 = nlagsc;
  if nlagsc2 == 4
    nlagsc2_ = [];
  else
    nlagsc2_ = num2str(nlagsc2);
  end  
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);
pmean = pmean.*(1-para_mask)+para_fix.*para_mask;
pstdd = pstdd.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);

percent = .90

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% doing rolling sample estimation?
ROLL = 0;
SAVEALL = 0;
DOFORECAST = 0;
if ROLL 
  DOFORECAST = 1;
  SAVEALL = 0;
end
fprintf(1,'\n\n ROLL: %2.0f, SAVEALL: %2.0f, DOFORECAST: %2.0f\n',[ROLL,SAVEALL,DOFORECAST]);
start_period = stime;                               % minimum # quarters to compute VAR estimates 


%mkdir(spath,'/2/');
%spath = strcat(spath,'/1/');
%start_period = nobsall-qahead+1-3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% CHAnGE

trial_period = nobsall-start_period-qahead+1;      % # quarters to compute RMSE


I = find(ti == dates);


Verbose = 0;

fixparam = fixparamseq(1);
if fixparam ~= 0 
  fixparam
  break
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reference Model for Prior Second Moments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isempty(mspec_ref)

  %% w/o correcting model - already discarded nburns
  infile1 = strcat(spath,'/mhNIpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc_,diago_)
  fid1 = fopen(infile1,'r');		% input file (set to read)   
  para = fread(fid1,[npara,inf],'single')';
  fclose(fid1);

  infile2 = strcat(spath,'/priodet',lmodel,lprior,dataset);
  fid2 = fopen(infile2,'r');		% input file (set to read)   
  detsim = fread(fid2,[1,inf],'int8')';
  fclose(fid2);

  %% discard indeterminacy
  ndraws = size(para,1)
  para(find(detsim),:) = [];
  fprintf(1,'\n\n  ratio of determinacy draws %2.2f\n',size(para,1)/ndraws);
  ndraws = size(para,1)

else
  %% w/ correcting model -  have to  discard nburns
  
  infile1 = strcat(spath,'/mhNIpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc_,diago_)
  fid1 = fopen(infile1,'r');		% input file (set to read)   
  numb1 = nburn*npara*4;                  % number of bites to discard
  status = fseek(fid1,numb1,'bof');       % discard nburn 
  para = fread(fid1,[npara,inf],'single')';
  fclose(fid1);

  
  ndraws = size(para,1)


end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% read draws of  parameters and discard those for which the model has no solution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Compute posterior moments of the parameters (after IND)
outfile00 = strcat(fpath,lmodel,lprior,dataset,lmodel_ref,nobsdrat_2,lambdac_,nlagsc_,diago_,'PrioMom.tex');		% output file name 
fid00 = fopen(outfile00,'w');		% output file (set to write)

parahat = mean(para,1)';
paramed = median(para,1)';
parasig = cov(para,1);

parabands = hpdint(para,percent,1)';

%% save mean
outfile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc_,diago_,'Mean')
fid0 = fopen(outfile0,'w');     % output file (set to write)   
fwrite(fid0,parahat','single');
fclose(fid0);

%% save median
outfile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc_,diago_,'Median')
fid0 = fopen(outfile0,'w');     % output file (set to write)   
fwrite(fid0,paramed','single');
fclose(fid0);

%% save cov matrix
outfile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc_,diago_,'Cov')
fid0 = fopen(outfile0,'w');     % output file (set to write)   
fwrite(fid0,parasig(:),'single');
fclose(fid0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% Mean
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

colnames = {  ' Mean ', 'St. Dev. ',strcat(num2str(100*percent),'\% Lower Band'),strcat(num2str(100*percent),'\% Upper Band') };
outmat = [parahat,sqrt(diag(parasig)),parabands];

fprintf(1,'\n');
fprintf(1,'%4.99s  ',colnames{:});
for i = 1:length(para_names)
  fprintf(1,'\n %4.10s  ',para_names{i});
  fprintf(1,' %8.4f  ',outmat(i,:));
end    

fprintf(fid00,'\n');


colnames = {'Parameter' 'Support '  'Density '  'Prior Mean ' 'Prior Stdd ' strcat(num2str(100*percent),'\% LB') ...
            strcat(num2str(100*percent),'\% UB') };
%outmat = [parahat,sqrt(diag(parasig)),parabands];

rownames = para_names;

[nrows,ncols] = size(outmat);
fprintf(fid00,'\n \\documentclass{article}');
fprintf(fid00,'\n \\begin{document}');
fprintf(fid00,'\n \\setlength{\\tabcolsep}{.7mm}');

fprintf(fid00,'PARAMETERS (FILTERNG OUT INDETERMINACY AND NON EX)\n $$');
fprintf(fid00,'\n \\vspace{.2cm} \n \\begin{array}{@{\\hspace*{.5cm}} l @{\\hspace*{.5cm}}@{\\hspace*{-.2cm}}c@{\\hspace*{-.2cm}}@{\\hspace*{-.2cm}}c@{\\hspace*{-.2cm}}');
FORMAT = '  ';
for j = 1:ncols
  fprintf(fid00,' @{\\hspace*{.5cm}} c'); 
  if j == 1
    FORMAT = strcat(FORMAT,' &  %8.3f  '); 
    % elseif (j == ncols) | (j == (ncols-1))
    %	FORMAT = strcat(FORMAT,' & \\mbox{{\\scriptsize $%8.3f$ }} '); 
  else
    FORMAT = strcat(FORMAT,' & %8.3f '); 
  end

end

fprintf(fid00,'}\\hline');
FORMAT = strcat(FORMAT,' \\\\');  

fprintf(fid00,'\n  ');
fprintf(fid00,' %4.99s   ',colnames{1});		
fprintf(fid00,' & %4.99s ',colnames{2:end});		
fprintf(fid00,'	\\\\ \\hline '); 

for i = 1:nrows
  fprintf(fid00,'\n %4.99s & ',rownames{i});
  if pshape(i) == 1
    fprintf(fid00,'[0,1) & \\mbox{Beta} ');	
  elseif pshape(i) == 2
    fprintf(fid00,'R^{+} & \\mbox{Gamma} ');	
  elseif pshape(i) == 3
    fprintf(fid00,' R & \\mbox{\\cal N} ');	
  elseif pshape(i) == 4
    fprintf(fid00,'R^{+} & \\mbox{InvGamma} ');	
  elseif pshape(i) == 0
    fprintf(fid00,' &  ');	    
  end
  fprintf(fid00,FORMAT,outmat(i,:));

end   
fprintf(fid00,' \\hline');
fprintf(fid00,'\n \\end{array} $$');        
fprintf(fid00,'\n  ratio of determinacy draws %2.2f\n',size(para,1)/ndraws);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% Mean
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% compute moments
[TTT,RRR,valid] = dsgesolv(parahat,mspec);
%valid = 1;
%[yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,para,nvar,nlagsc2,mspec,npara,coint,cointadd,cointall);

[ZZ,DD,DDcointadd,QQ,EE,MM,retcode] = measur(TTT,RRR,valid,parahat,nvar,nlagsc2,mspec,npara,coint,cointadd,cointall);

if retcode == 0
  disp('trouble');mm
  return;
end;

HH = EE+MM*QQ*MM';
VV = QQ*MM';

% Compute nlags autocovariances

% Initialize Autocovariances

GAMM0 = zeros(nvar^2,nlagsc2);

GA0 = dlyap(TTT,RRR*QQ*RRR');

Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
GAMM0(:,1) = Gl(:);

TTl = TTT;
for l = 1:nlagsc2;
  Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);
  GAMM0(:,l+1) = Gl(:);            
  TTl = TTl*TTT;
end

%% Create limit cross product matrices
yyyyd0 = reshape(GAMM0(:,1),nvar,nvar);
yyyyd1 = reshape(GAMM0(:,2),nvar,nvar);

parahat
yyyyd  = yyyyd0(1:nvar,1:nvar)
mm

yyxxd = zeros(nvar,nlagsc2*nvar);
xxxxd = zeros(nlagsc2*nvar,nlagsc2*nvar);

for rr = 1:nlagsc2;
  %% E[yy,x(lag rr)]
  yyxxd(:,nvar*(rr-1)+1:nvar*rr) = reshape(GAMM0(:,rr+1),nvar,nvar);
  
  %% E[x(lag rr),x(lag ll)]
  for ll = rr:nlagsc2;  
    yyyydrrll = reshape(GAMM0(:,ll-rr+1),nvar,nvar);
    xxxxd(nvar*(rr-1)+1:nvar*rr,nvar*(ll-1)+1:nvar*ll) = yyyydrrll(1:nvar,1:nvar);
    xxxxd(nvar*(ll-1)+1:nvar*ll,nvar*(rr-1)+1:nvar*rr) = yyyydrrll(1:nvar,1:nvar)';
  end
end

xxyyd = yyxxd';


MeY = DD
VaY = yyyyd;
stdY = sqrt(diag(VaY))
CorY = VaY./(stdY*stdY');
CoY = xxyyd(1:nvar,:);
ACorY = diag(CoY)./diag(VaY)


stats = [MeY' stdY' ACorY' CorY(:)'];

if isempty(mspec_ref)

  
  POP_ = '';

  outfile2 = strcat(spath,'/yyyy',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc2_,POP_,'ThMean');		
  fid2 = fopen(outfile2,'w');		% output file (set to write)   
  outfile3 = strcat(spath,'/xxyy',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc2_,POP_,'ThMean');		
  fid3 = fopen(outfile3,'w');		% output file (set to write)   
  outfile4 = strcat(spath,'/xxxx',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc2_,POP_,'ThMean');		
  fid4 = fopen(outfile4,'w');		% output file (set to write)   

  dm(yyyyd);
  dm(xxyyd);
  dm(xxxxd);
  
  fwrite(fid2,yyyyd,'single');
  fwrite(fid3,xxyyd,'single');
  fwrite(fid4,xxxxd,'single');

  fclose(fid2);
  fclose(fid3);
  fclose(fid4);
  
  POP_ = '';

  outfile200 = strcat(spath,'/stats',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc2_,POP_,'ThMean');		
  fid200 = fopen(outfile200,'w');		% output file (set to write)   
  fwrite(fid200,stats,'single');
  fclose(fid200);


  %%non demeaned matrices
  GAMM0 = zeros(nvar^2,nlagsc2+1);

  % GA0 = RRR*QQ*RRR';
  % GA0 =  GA0(:);
  % GA0  = inv(eye(nstate^2) - kron(TTT,TTT))*GA0;
  % GA0  = reshape(GA0',nstate,nstate)';
  % GA0_ = d2lyap(TTT,RRR*QQ*RRR'); %% this is the .m version of dlyap.mexglx
  GA0 = dlyap(TTT,RRR*QQ*RRR');


  Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
  Gl = Gl';
  GAMM0(:,1) = Gl(:);

  TTl = TTT;
  for l = 1:nlagsc2;
    Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);
    Gl = Gl';
    GAMM0(:,l+1) = Gl(:);            %%%????
    TTl = TTl*TTT;
  end

  % Create limit cross product matrices

  yyyyd = reshape(GAMM0(:,1)',nvar,nvar)' + DD*DD';

  xxyyd = DD;
  for l = 1:nlagsc2;
    xxyyd = [xxyyd , (reshape(GAMM0(:,l+1)',nvar,nvar)' + DD*DD')];
  end
  xxyyd = xxyyd';

  xxxxd = zeros(1+nlagsc2*nvar,1+nlagsc2*nvar);
  xxxxd(1,1) = 1;
  xxxxd(1,2:1+nlagsc2*nvar) = kron(ones(1,nlagsc2),DD');
  xxxxd(2:1+nlagsc2*nvar,1) = kron(ones(nlagsc2,1),DD);

  for r = 1:nlagsc2;
    for l = r:nlagsc2;	
      xxxxd(1+nvar*(r-1)+1:1+nvar*r,1+nvar*(l-1)+1:1+nvar*l) = reshape(GAMM0(:,l-r+1)',nvar,nvar)'+DD*DD';
      xxxxd(1+nvar*(l-1)+1:1+nvar*l,1+nvar*(r-1)+1:1+nvar*r) = reshape(GAMM0(:,l-r+1)',nvar,nvar)+DD*DD';
    end
  end

  POP_ = '';

  outfile2 = strcat(spath,'/yyyy',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc2_,POP_,'ThMeanND');		
  fid2 = fopen(outfile2,'w');		% output file (set to write)   
  outfile3 = strcat(spath,'/xxyy',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc2_,POP_,'ThMeanND');		
  fid3 = fopen(outfile3,'w');		% output file (set to write)   
  outfile4 = strcat(spath,'/xxxx',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc2_,POP_,'ThMeanND');		
  fid4 = fopen(outfile4,'w');		% output file (set to write)   

  dm(yyyyd);
  dm(xxyyd);
  dm(xxxxd);

  fwrite(fid2,yyyyd,'single');
  fwrite(fid3,xxyyd,'single');
  fwrite(fid4,xxxxd,'single');

  fclose(fid2);
  fclose(fid3);
  fclose(fid4);
end  

Fortab = '\n \\begin{tabular}{l';
Fornum = ' ';
Heads = 'Variables ';
Heads2 = ' ';

%% mean (model, data)
DatOut = [MeY];
Fortab = strcat(Fortab,'|l');
Fornum = strcat(Fornum,' & %2.2f ');
Heads = strcat(Heads,'&  \\multicolumn{1}{c}{Mean}  ');

%% st dev (model, data)
DatOut = [DatOut stdY ];
Fortab = strcat(Fortab,'|l');
Fornum = strcat(Fornum,' & %2.2f ');
Heads = strcat(Heads,'&  \\multicolumn{1}{c}{St. Dev.}  ');

%% autocorr (model, data)
DatOut = [DatOut ACorY];
Fortab = strcat(Fortab,'|l');
Fornum = strcat(Fornum,' & %2.2f ');
Heads = strcat(Heads,'&  \\multicolumn{1}{c}{Autocorr.}  ');


%% contemp corr (model, data)
CorMix = tril(CorY,-1)+triu(CorY);
DatOut2 = [CorMix];
Heads = strcat(Heads,' & \\multicolumn{',num2str(nvar),'}{c}{Contemporanous Corr.}  ');

Fortab = strcat(Fortab,'|l');
Fornum2 = ' ';
for jj = 1:nvar
  Fortab = strcat(Fortab,'l');
  Fornum2 = strcat(Fornum2,' & %2.2f ');
end


%% finish up
Fortab = strcat(Fortab,'}\\hline \n');
Heads = strcat(Heads,'\\\\ \n');


fprintf(fid00,'\n \\begin{table}[h] \\centering');        
fprintf(fid00,'\n \\caption{Model %2.0f Statistics at Posterior Mean} \n\\vspace*{.5cm}',mspec);

%% setting is: mean (model, data) , variances (model, data), autocorrelations (model, data), correlations (model, data)
fprintf(fid00,Fortab);

fprintf(fid00,Heads);
for i = 1:size(DatOut,1);
  fprintf(fid00,'\\\\ \n $%4.99s$ ',varnames{i});
  fprintf(fid00,Fornum,DatOut(i,:));
  fprintf(fid00,' & ');  
  fprintf(fid00,Fornum2,DatOut2(i,:));
end   
fprintf(fid00,'\\\\  \\hline');		    
fprintf(fid00,'\n \\end{tabular}');
fprintf(fid00,'\n \\end{table} \n\n');


fprintf(fid00,'\n \\end{document}');
fclose(fid00);          % close output file

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% Median
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
colnames = {  ' Mean ', 'St. Dev. ',strcat(num2str(100*percent),'\% Lower Band'),strcat(num2str(100*percent),'\% Upper Band') };
outmat = [paramed,sqrt(diag(parasig)),parabands];

fprintf(1,'\n');
fprintf(1,'%4.99s  ',colnames{:});
for i = 1:length(para_names)
  fprintf(1,'\n %4.10s  ',para_names{i});
  fprintf(1,' %8.4f  ',outmat(i,:));
end    

fprintf(fid00,'\n');


colnames = {'Parameter' 'Support '  'Density '  'Prior Median ' 'Prior Stdd ' strcat(num2str(100*percent),'\% LB') ...
            strcat(num2str(100*percent),'\% UB') };
%outmat = [parahat,sqrt(diag(parasig)),parabands];

rownames = para_names;

[nrows,ncols] = size(outmat);

fprintf(fid00,'PARAMETERS (FILTERNG OUT INDETERMINACY AND NON EX)\n $$');
fprintf(fid00,'\n \\vspace{.2cm} \n \\begin{array}{@{\\hspace*{.5cm}} l @{\\hspace*{.5cm}}@{\\hspace*{-.2cm}}c@{\\hspace*{-.2cm}}@{\\hspace*{-.2cm}}c@{\\hspace*{-.2cm}}');
FORMAT = '  ';
for j = 1:ncols
  fprintf(fid00,' @{\\hspace*{.5cm}} c'); 
  if j == 1
    FORMAT = strcat(FORMAT,' &  %8.3f  '); 
    % elseif (j == ncols) | (j == (ncols-1))
    %	FORMAT = strcat(FORMAT,' & \\mbox{{\\scriptsize $%8.3f$ }} '); 
  else
    FORMAT = strcat(FORMAT,' & %8.3f '); 
  end

end

fprintf(fid00,'}\\hline');
FORMAT = strcat(FORMAT,' \\\\');  

fprintf(fid00,'\n  ');
fprintf(fid00,' %4.99s   ',colnames{1});		
fprintf(fid00,' & %4.99s ',colnames{2:end});		
fprintf(fid00,'	\\\\ \\hline '); 

for i = 1:nrows
  fprintf(fid00,'\n %4.99s & ',rownames{i});
  if pshape(i) == 1
    fprintf(fid00,'[0,1) & \\mbox{Beta} ');	
  elseif pshape(i) == 2
    fprintf(fid00,'R^{+} & \\mbox{Gamma} ');	
  elseif pshape(i) == 3
    fprintf(fid00,' R & \\mbox{\\cal N} ');	
  elseif pshape(i) == 4
    fprintf(fid00,'R^{+} & \\mbox{InvGamma} ');	
  elseif pshape(i) == 0
    fprintf(fid00,' &  ');	    
  end
  fprintf(fid00,FORMAT,outmat(i,:));

end   
fprintf(fid00,' \\hline');
fprintf(fid00,'\n \\end{array} $$');        
fprintf(fid00,'\n  ratio of determinacy draws %2.2f\n',size(para,1)/ndraws);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% Median
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% compute moments
[TTT,RRR,valid] = dsgesolv(paramed,mspec);
%valid = 1;
%[yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint,cointadd,cointall);

[ZZ,DD,DDcointadd,QQ,EE,MM,retcode] = measur(TTT,RRR,valid,paramed,nvar,nlagsc2,mspec,npara,coint,cointadd,cointall);

if retcode == 0
  disp('trouble');mm
  return;
end;

HH = EE+MM*QQ*MM';
VV = QQ*MM';

% Compute nlags autocovariances

% Initialize Autocovariances

GAMM0 = zeros(nvar^2,nlagsc2);

GA0 = dlyap(TTT,RRR*QQ*RRR');

Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
GAMM0(:,1) = Gl(:);

TTl = TTT;
for l = 1:nlagsc2;
  Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);
  GAMM0(:,l+1) = Gl(:);            
  TTl = TTl*TTT;
end

%% Create limit cross product matrices
yyyyd0 = reshape(GAMM0(:,1),nvar,nvar);
yyyyd1 = reshape(GAMM0(:,2),nvar,nvar);
yyyyd  = yyyyd0(1:nvar,1:nvar);

yyxxd = zeros(nvar,nlagsc2*nvar);
xxxxd = zeros(nlagsc2*nvar,nlagsc2*nvar);

for rr = 1:nlagsc2;
  %% E[yy,x(lag rr)]
  yyxxd(:,nvar*(rr-1)+1:nvar*rr) = reshape(GAMM0(:,rr+1),nvar,nvar);
  
  %% E[x(lag rr),x(lag ll)]
  for ll = rr:nlagsc2;  
    yyyydrrll = reshape(GAMM0(:,ll-rr+1),nvar,nvar);
    xxxxd(nvar*(rr-1)+1:nvar*rr,nvar*(ll-1)+1:nvar*ll) = yyyydrrll(1:nvar,1:nvar);
    xxxxd(nvar*(ll-1)+1:nvar*ll,nvar*(rr-1)+1:nvar*rr) = yyyydrrll(1:nvar,1:nvar)';
  end
end

xxyyd = yyxxd';


MeY = DD
VaY = yyyyd;
stdY = sqrt(diag(VaY))
CorY = VaY./(stdY*stdY');
CoY = xxyyd(1:nvar,:);
ACorY = diag(CoY)./diag(VaY)


stats = [MeY' stdY' ACorY' CorY(:)'];

if isempty(mspec_ref)

  
  POP_ = '';

  outfile2 = strcat(spath,'/yyyy',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc2_,POP_,'ThMedian');		
  fid2 = fopen(outfile2,'w');		% output file (set to write)   
  outfile3 = strcat(spath,'/xxyy',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc2_,POP_,'ThMedian');		
  fid3 = fopen(outfile3,'w');		% output file (set to write)   
  outfile4 = strcat(spath,'/xxxx',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc2_,POP_,'ThMedian');		
  fid4 = fopen(outfile4,'w');		% output file (set to write)   

  dm(yyyyd);
  dm(xxyyd);
  dm(xxxxd);

  fwrite(fid2,yyyyd,'single');
  fwrite(fid3,xxyyd,'single');
  fwrite(fid4,xxxxd,'single');

  fclose(fid2);
  fclose(fid3);
  fclose(fid4);
  
  POP_ = '';

  outfile200 = strcat(spath,'/stats',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,nlagsc_,diago_,POP_,'ThMedian');		
  fid200 = fopen(outfile200,'w');		% output file (set to write)   
  fwrite(fid200,stats,'single');
  fclose(fid200);

  

end  

Fortab = '\n \\begin{tabular}{l';
Fornum = ' ';
Heads = 'Variables ';
Heads2 = ' ';

%% mean (model, data)
DatOut = [MeY];
Fortab = strcat(Fortab,'|l');
Fornum = strcat(Fornum,' & %2.2f ');
Heads = strcat(Heads,'&  \\multicolumn{1}{c}{Median}  ');

%% st dev (model, data)
DatOut = [DatOut stdY ];
Fortab = strcat(Fortab,'|l');
Fornum = strcat(Fornum,' & %2.2f ');
Heads = strcat(Heads,'&  \\multicolumn{1}{c}{St. Dev.}  ');

%% autocorr (model, data)
DatOut = [DatOut ACorY];
Fortab = strcat(Fortab,'|l');
Fornum = strcat(Fornum,' & %2.2f ');
Heads = strcat(Heads,'&  \\multicolumn{1}{c}{Autocorr.}  ');


%% contemp corr (model, data)
CorMix = tril(CorY,-1)+triu(CorY);
DatOut2 = [CorMix];
Heads = strcat(Heads,' & \\multicolumn{',num2str(nvar),'}{c}{Contemporanous Corr.}  ');

Fortab = strcat(Fortab,'|l');
Fornum2 = ' ';
for jj = 1:nvar
  Fortab = strcat(Fortab,'l');
  Fornum2 = strcat(Fornum2,' & %2.2f ');
end


%% finish up
Fortab = strcat(Fortab,'}\\hline \n');
Heads = strcat(Heads,'\\\\ \n');




fprintf(fid00,'\n \\begin{table}[h] \\centering');        
fprintf(fid00,'\n \\caption{Model %2.0f Statistics at Posterior Median} \n\\vspace*{.5cm}',mspec);

%% setting is: mean (model, data) , variances (model, data), autocorrelations (model, data), correlations (model, data)
fprintf(fid00,Fortab);

fprintf(fid00,Heads);

for i = 1:size(DatOut,1);
  fprintf(fid00,'\\\\ \n $%4.99s$ ',varnames{i});
  fprintf(fid00,Fornum,DatOut(i,:));
  fprintf(fid00,' & ');  
  fprintf(fid00,Fornum2,DatOut2(i,:));
end   
fprintf(fid00,'\\\\  \\hline');		    
fprintf(fid00,'\n \\end{tabular}');
fprintf(fid00,'\n \\end{table} \n\n');


fprintf(fid00,'\n \\end{document}');
fclose(fid00);		% close output file
