% filename:    prio.m
% description: generates draws from the prior and computes prior moments for the parameters of interest
%              marginal log likelihood
%              with respect to the DSGE model parameters
close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
run startdsswfed;
!hostname;
!ps -C matlab;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varspec;

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);

if isempty(mspec_ref)
  lmodel_ref = [];
  nobsdrat_ = [];
  lambdac_ = [];
  nlagsc_ = [];
  n_s_ = [];

else
  lmodel_ref = strcat('m',num2str(mspec_ref));
  nobsdrat_ = num2str(100*nobsdrat);
  lambdac_ = [];
  n_s_ = num2str(n_s);
  if nlagsc == 4
    nlagsc_ = [];
  else
    nlagsc_ = num2str(nlagsc);
  end  
  
end
stup_ = num2str(stup);

%% set of parameters 
list_t2 = find(para_type == 2);
npara2 = length(list_t2)
list_t0 = find(para_type == 0);
npara0 = length(list_t0)
list_t1 = find(para_type == 1);
npara1 = length(list_t1)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,mspec);
Iseq = find(ti == dates);
I = Iseq;

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
if isempty(mspec_ref)
  priorfile = strcat('pri',lprior)
elseif ~isempty(mspec_ref)
  priorfile = strcat('pri',lprior)
end
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);

pmean = pmean.*(1-para_mask)+para_fix.*para_mask;

nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


infile100 = strcat(spath,'/mhprioparat0',lmodel,lprior(1),dataset);
fid100 = fopen(infile100,'r');		% input file (set to read)   
num100 = nblocks*nsim*npara0*4;            % number of bites per file
numb100 = nburn*npara0*4;                  % number of bites to discard
status = fseek(fid100,numb100,'bof');       % discard nburn 

if ~isempty(list_t1)
  infile101 = strcat(spath,'/mhprioparat1',lmodel,lprior,dataset,lmodel_ref,stup_);  
  fid101 = fopen(infile101,'r');		% input file (set to read)   
  num101 = (nblocks*nsim-nburn)*npara1*4;            % number of bites per file
                                                     % no need to discard
end

if ~isempty(list_t2)
  infile102 = strcat(spath,'/mhprioparat2',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);
  fid102 = fopen(infile102,'r');		% input file (set to read)   
  num102 = nblocks*nsim*npara2*4;            % number of bites per file
  numb102 = nburn*npara2*4;                  % number of bites to discard
  status = fseek(fid102,numb102,'bof');       % discard nburn 
end

%% open write files
%% params file
outfile1 =  strcat(spath,'/mhNIpriopara',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);

%% determinacy flag
outfile2 =  strcat(spath,'/priodet',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);

%% MA representation: TTT (nstate x nstate) and RRR (nstate x nshocks)
outfile11 = strcat(spath,'/TTTprio',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);    
outfile12 = strcat(spath,'/RRRprio',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);    

fid1 = fopen(outfile1,'w');		% output file (set to write)   
fid2 = fopen(outfile2,'w');		% output file (set to write)   
fid11 = fopen(outfile11,'w');       % output file (set to write)   
fid12 = fopen(outfile12,'w');       % output file (set to write)   

[TTT,RRR,valid] = dsgesolv(para,mspec);
nstate = length(TTT);
nshocks = size(RRR,2);

iblock = 0;
while ( ftell(fid100) < num100 ) 
  % Read blocks of size nsim
  iblock = iblock+1

  
  parasim = zeros([nsim,npara]);
  detsim      = zeros(nsim,1);
  TTTsim = zeros(nsim,nstate^2);
  RRRsim = zeros(nsim,nstate*nshocks);

  parasim(:,find(para_mask)) = repmat(para_fix(find(para_mask))',[nsim 1]);

  para0sim = fread(fid100,[npara0,nsim],'single')';
  parasim(:,list_t0) = para0sim;
  if ~isempty(list_t1)
    para1sim = fread(fid101,[npara1,nsim],'single')';
    parasim(:,list_t1) = para1sim;
  end

  if ~isempty(list_t2)
    para2sim = fread(fid102,[npara2,nsim],'single')';
    parasim(:,list_t2) = para2sim;
  end
  for j = 1:nsim
    paradraw = parasim(j,:)';

    [TTT,RRR,valid] = dsgesolv(paradraw,mspec);

    if valid ~= 1 | any(isnan(TTT))
      %% indeterminacy or non-existence
      detsim(j) = 1;
      
    else

      TTTsim(j,:) = TTT(:)';
      RRRsim(j,:) = RRR(:)';
      
    end

  end
  fprintf(1,'\n\n  Block, %2.0f; ratio of INdeterminacy draws %2.2f\n',[iblock,length(find(detsim))/nsim]);
  
  parasim(find(detsim),:) = [];%NaN; 
  TTTsim(find(detsim),:)  = [];%NaN; 
  RRRsim(find(detsim),:)  = [];%NaN; 
  
  fwrite(fid1,parasim','single');
  fwrite(fid2,detsim,'int8');
   
  %% note prime
  fwrite(fid11,TTTsim','single');
  fwrite(fid12,RRRsim','single');
  
end;
fclose(fid1);
fclose(fid2);
fclose(fid11);
fclose(fid12);

fclose(fid100);
if ~isempty(list_t1)
  fclose(fid101);
end
if ~isempty(list_t2)
  fclose(fid102);
end
