
close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
run startdsswfed;
!hostname;
!ps -C matlab;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varspec;

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)));


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);


if isempty(mspec_ref)
  lmodel_ref = [];
  nobsdrat_ = [];
  lambdac_ = [];
  nlagsc_ = [];
  n_s_ = [];
  disp('NO reference model set in varspec.m')
else
  lmodel_ref = strcat('m',num2str(mspec_ref));
  nobsdrat_ = num2str(100*nobsdrat);
  lambdac_ = [];
  n_s_ = num2str(n_s);
  if nlagsc == 4
    nlagsc_ = [];
  else
    nlagsc_ = num2str(nlagsc);
  end  
  
end
stup_ = num2str(stup);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,mspec);
Iseq = find(ti == dates);
I = Iseq;
if stime                           
  YY = YYall(I-stime+1:I,:);
  XX = XXall(I-stime+1:I,:);             
  tiI = ti(I-stime+1:I);
else
  YY     = YYall(1:I,:);
  XX     = XXall(1:I,:);                   
  tiI = ti(1:I);
end
nobs = size(YY,1)


YYYY = YY'*YY;
XXYY = XX'*YY;
XXXX = XX'*XX;

k = 1+nlags*nvar+coint;

MeY_0 = mean(YY,1)';
VaY_0 = cov(YY,1);
stdY_0 = sqrt(diag(VaY_0));
CorY_0 = VaY_0./(stdY_0*stdY_0');
ACorY_0 = zeros(nvar,1);
for ii = 1:nvar;
  ACorY_0(ii) = corr(YY(:,ii),XX(:,1+cointall+ii));
end

stats_0 = [MeY_0' stdY_0' ACorY_0' CorY_0(:)'];


dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat('pri',lprior);
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set date
I = find(ti == dates);

%% WANT POPULATION OR SAMPLE MOMENTS?
POPUL = 0
if POPUL 
  POP_ = '';
else
  POP_ = 'P';
end


percent = .90;

[TTT,RRR,valid] = dsgesolv(para,mspec);
nstate = length(TTT);
nshocks = size(RRR,2);


PRIO = 1
if PRIO

  %% prior params file (first block has already been thrown out)
  infile3 = strcat(spath,'/mhNIpriopara',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);
  numb3 = nburn*npara*4;  % number of bites per file
  fid3 = fopen(infile3,'r');		% input file (set to read)               
                                        %% note nburn has already been discarded
  status = fseek(fid3,0,1);       % measure how many bytes in the file
  tot_ = ftell(fid3);
  status = fseek(fid3,0,-1);       % back to the beginning of the file


  %% MA representation: TTT (nstate x nstate) and RRR (nstate x nshocks)
  infile11 = strcat(spath,'/TTTprio',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);    
  numb11 = nburn*nstate^2*4;  % number of bites per file
  fid11 = fopen(infile11,'r');		% input file (set to read) 

  infile12 = strcat(spath,'/RRRprio',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);  
  numb12 = nburn*nstate*nshocks*4;  % number of bites per file
  fid12 = fopen(infile12,'r');		% input file (set to read) 

  outfile01 = strcat(spath,'/priostats',lmodel,lprior,dataset,POP_,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);
  fid01 = fopen(outfile01,'w');		% output file (set to write)   

  outfile011 = strcat(spath,'/CorLsPi',lmodel,lprior,dataset,POP_,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);		
  fid011 = fopen(outfile011,'w');		% output file (set to write)   
  outfile012 = strcat(spath,'/CorLsY',lmodel,lprior,dataset,POP_,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);		
  fid012 = fopen(outfile012,'w');		% output file (set to write)   
  outfile013 = strcat(spath,'/AcorLs',lmodel,lprior,dataset,POP_,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);		
  fid013 = fopen(outfile013,'w');		% output file (set to write)   
  outfile014 = strcat(spath,'/AcorPi',lmodel,lprior,dataset,POP_,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);		
  fid014 = fopen(outfile014,'w');		% output file (set to write)   


else
  coint0 = coint;
  lambdas = NaN;   
  fixparam = fixparamseq(1);

  %% posterior
  infile3 = strcat(spath,'/mhpara',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(coint0),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
  num3 = nblocks*nsim*npara*4;            % number of bites per file
  numb3 = nburn*npara*4;                  % number of bites to dicard

  fid3 = fopen(infile3,'r');		% input file (set to read)               
  status = fseek(fid3,0,1);       % measure how many bytes in the file
  tot_ = ftell(fid3);
  status = fseek(fid3,0,-1);       % back to the beginning of the file
                                   %  status = fseek(fid3,numb3,'bof');

  
  infile11 = strcat(spath,'/TTT',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    
  numb11 = nburn*nstate^2*4;  % number of bites per file
  
  fid11 = fopen(infile11,'r');		% input file (set to read)               
                                        %  status = fseek(fid11,numb11,'bof');

  infile12 = strcat(spath,'/RRR',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    
  numb12 = nburn*nstate*nshocks*4;  % number of bites per file
  
  fid12 = fopen(infile12,'r');		% input file (set to read)               
                                        %  status = fseek(fid12,numb12,'bof');
  

  outfile01 = strcat(spath,'/priostats',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(coint0),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));
  fid01 = fopen(outfile01,'w');		% output file (set to write)   

  outfile011 = strcat(spath,'/CorLsPi',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(coint0),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
  fid011 = fopen(outfile011,'w');		% output file (set to write)   
  outfile012 = strcat(spath,'/CorLsY',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(coint0),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
  fid012 = fopen(outfile012,'w');		% output file (set to write)   
  outfile013 = strcat(spath,'/AcorLs',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(coint0),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
  fid013 = fopen(outfile013,'w');		% output file (set to write)   
  outfile014 = strcat(spath,'/AcorPi',lmodel,lprior,dataset,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_,num2str(coint0),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));		
  fid014 = fopen(outfile014,'w');		% output file (set to write)   

end


%outfile02 = strcat(spath,'/yyyy',lmodel,lprior,dataset,POP_,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);		
%fid02 = fopen(outfile02,'w');		% output file (set to write)   
%outfile03 = strcat(spath,'/xxyy',lmodel,lprior,dataset,POP_,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);		
%fid03 = fopen(outfile03,'w');		% output file (set to write)   
%outfile04 = strcat(spath,'/xxxx',lmodel,lprior,dataset,POP_,lmodel_ref,stup_,nobsdrat_,nlagsc_,n_s_);		
%fid04 = fopen(outfile04,'w');		% output file (set to write)   

priotheta = [];
iblock = 0;
while ( ftell(fid3) < tot_ ) 
  % Read blocks of size nsim

  iblock = iblock+1;
  fprintf(1,'\n block: %2.0f  ;',iblock);
  priotheta = fread(fid3,[npara,nsim],'single')';
  TTTsim = fread(fid11,[nstate^2,nsim],'single')';
  RRRsim = fread(fid12,[nstate*nshocks,nsim],'single')';


  nsim_ = size(priotheta,1);
  fprintf(1,' block size: %2.0f  ;',nsim_);

  statssim = zeros(nsim_,length(stats_0));


  CorLsPisim = zeros(nsim_,2*k_l+1);
  CorLsYsim = zeros(nsim_,2*k_l+1);
  AcorLssim = zeros(nsim_,k_l);
  AcorPisim = zeros(nsim_,k_l);
  for j = 1:nsim_

    params = priotheta(j,:)';          
    if any(isnan(params))
      statssim(j,:) = NaN;

    else
      
      TTT = reshape(TTTsim(j,:),nstate,nstate);
      RRR = reshape(RRRsim(j,:),nstate,nshocks);
      if ~PRIO
        [TTT,RRR,valid] = dsgesolv(params,mspec);
      end

      %%%%%%%%%%%%%% CHANGE!!!!!!!!!
      CHP = 0;
      if CHP
        infile0 = strcat(spath,'/mhpriopara',lmodel,lprior,dataset,[],[],'Mean');
        fid0 = fopen(infile0,'r');     % input file (set to read)   
        params_1bar = fread(fid0,[npara,1],'single');
        fclose(fid0);  
        params_1bar = params_1bar.*(1-para_mask)+para_fix.*para_mask;
        %params
        
        %% Fix non-auxiliary parameters
        params = params.*para_aux+params_1bar.*(1-para_aux);
        %% Solve the model and evaluate the likelihood

        [TTT,RRR,valid] = dsgesolv(params,mspec);
      end
      %%%%%%%%%%%%%% CHANGE!!!!!!!!!
      
      %valid = 1;
      %[yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,params,nvar,nlags,mspec,npara,coint,cointadd,cointall);
      
      [ZZ,DD,DDcointadd,QQ,EE,MM,retcode] = measur(TTT,RRR,valid,params,nvar,nlags,mspec,npara,coint,cointadd,cointall);
      %% generate data and computes moments from that
      
      DD_coint = DD(nvar+1:nvar+coint,:);
      ZZ_ = ZZ;
      DD_ = DD;
      %% Remove cointegration component from measurement equation	                
      ZZ = ZZ(1:nvar,:);
      DD = DD(1:nvar,:);
      EE = EE(1:nvar,1:nvar);
      MM = MM(1:nvar,:);


      %% set sample length
      nobsd = nobs+nlags;

      Shocks = randn(nobsd,nshocks);
      Shocks0 = Shocks*(QQ^.5);

      z = zeros(nstate,1);
      series = zeros(nobsd,nvar+coint);

      for t = 1:nobsd
        z = TTT*z+RRR*Shocks0(t,:)';
        series(t,:) = (DD_+ZZ_*z)';
      end;

      series_coint = series(:,nvar+1:nvar+coint);
      series = series(:,1:nvar);


      %% construct VAR data

      YYd = series(1+nlags:nobsd,:);
      XXd = ones(nobsd-nlags, coint+1+nvar*nlags);
      for i = 1:1:nlags;
        XXd(:,1+coint+(i-1)*nvar+1:1+coint+i*nvar) = series(nlags-i+1:nobsd-i,:);
      end

      if coint > 0
        XXd(:,1:coint) = series_coint(nlags:nobsd-1,:);
      end
      
      % YYYYd = YYd'*YYd/nobs;
      % XXYYd = XXd'*YYd/nobs;
      % XXXXd = XXd'*XXd/nobs;
      

      MeY = mean(YYd,1)';
      VaY = cov(YYd,1);
      stdY = sqrt(diag(VaY));
      CorY = VaY./(stdY*stdY');
      ACorY = zeros(nvar,1);
      for ii = 1:nvar;
        ACorY(ii) = corr(YYd(:,ii),XXd(:,1+cointall+ii));
      end
      %if ACorY(4)<0; disp([j, ACorY(4)]); end
      statssim(j,:) = [MeY' stdY' ACorY' CorY(:)'];


      for il = -k_l:k_l;
        if il <= 0
          CorLsPisim(j,k_l+1+il) = corr(YYd(1:end+il,4),YYd(1-il:end,3));
          CorLsYsim(j,k_l+1+il) = corr(YYd(1:end+il,3),YYd(1-il:end,1));
        else
          CorLsPisim(j,k_l+1+il) = corr(YYd(1+il:end,4),YYd(1:end-il,3));
          CorLsYsim(j,k_l+1+il) = corr(YYd(1+il:end,3),YYd(1:end-il,1));
          AcorLssim(j,il) = corr(YYd(1:end-il,3),YYd(1+il:end,3));
          AcorPisim(j,il) = corr(YYd(1:end-il,4),YYd(1+il:end,4));
        end
      end

    end
    
    
  end
  fwrite(fid01,statssim','single');

  fwrite(fid011,CorLsPisim','single');
  fwrite(fid012,CorLsYsim','single');
  fwrite(fid013,AcorLssim','single');
  fwrite(fid014,AcorPisim','single');

  
end;
fclose(fid3);
fclose(fid11);
fclose(fid12);

fclose(fid01);

fclose(fid011);
fclose(fid012);
fclose(fid013);
fclose(fid014);



