close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
run startdsswfed;
!hostname;
!ps -C matlab;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para, ...
 varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode, ...
 stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix,para_aux,mspec_ref,nobsdrat,lambdac] = varspec(chdata);

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);


fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);

POST = 1;

if isempty(mspec_ref)
  lmodel_ref = [];
  nobsdrat_ = [];
  lambdac_ = [];
else
  lmodel_ref = strcat('m',num2str(mspec_ref));
  nobsdrat_ = num2str(nobsdrat);
  if isinf(lambdac)
      lambdac_ = [];
  else
      lambdac_ = num2str(100*lambdac);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);
Iseq = find(ti == dates);
I = Iseq;
if stime                           
  YY = YYall(I-stime+1:I,:);
  XX = XXall(I-stime+1:I,:);             
  tiI = ti(I-stime+1:I);
else
  YY     = YYall(1:I,:);
  XX     = XXall(1:I,:);                   
  tiI = ti(1:I);
end
nobs = size(YY,1)


YYYY = YY'*YY;
XXYY = XX'*YY;
XXXX = XX'*XX;

k = 1+nlags*nvar+coint;

MeY_0 = mean(YY,1)';
VaY_0 = cov(YY,1);
stdY_0 = sqrt(diag(VaY_0));
CorY_0 = VaY_0./(stdY_0*stdY_0');
ACorY_0 = zeros(nvar,1);
for ii = 1:nvar;
  ACorY_0(ii) = corr(YY(:,ii),XX(:,1+cointall+ii));
end

stats_0 = [MeY_0' stdY_0' ACorY_0' CorY_0(:)'];

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set date
I = find(ti == dates);

%% WANT POPULATION OR SAMPLE MOMENTS?
POPUL = 0
if POPUL 
  POP_ = '';
else
  POP_ = 'P';
end


percent = .90;

[TTT,RRR,valid] = dsgesolv(para,mspec);
nstate = length(TTT);
nshocks = size(RRR,2);


if ~POST
  
  %% INFILES
  %% prior params file (first block has already been thrown out)
  infile3 = strcat(spath,'/mhNIpriopara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_);   
  %% MA representation: TTT (nstate x nstate) and RRR (nstate x nshocks)
  infile11 = strcat(spath,'/TTTprio',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_);    
  infile12 = strcat(spath,'/RRRprio',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_);  

  %% OUTFILES
  outfile01 = strcat(spath,'/priostats',lmodel,lprior,dataset,POP_,lmodel_ref,nobsdrat_,lambdac_);
  outfile02 = strcat(spath,'/yyyy',lmodel,lprior,dataset,POP_,lmodel_ref,nobsdrat_,lambdac_);		
  outfile03 = strcat(spath,'/xxyy',lmodel,lprior,dataset,POP_,lmodel_ref,nobsdrat_,lambdac_);		
  outfile04 = strcat(spath,'/xxxx',lmodel,lprior,dataset,POP_,lmodel_ref,nobsdrat_,lambdac_);		

elseif POST
  %% INFILES
  %% params file
  infile3 = strcat(spath,'/mhpara',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));      
  %% MA representation: TTT (nstate x nstate) and RRR (nstate x nshocks)
  infile11 = strcat(spath,'/TTT',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    
  infile12 = strcat(spath,'/RRR',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    

  %% OUTFILES
  outfile01 = strcat(spath,'/priostats',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    
  outfile02 = strcat(spath,'/yyyy',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    
  outfile03 = strcat(spath,'/xxyy',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    
  outfile04 = strcat(spath,'/xxxx',lmodel,lprior,dataset,lmodel_ref,nobsdrat_,lambdac_,num2str(cointall),st,num2str(100*lambdas),num2str(100*ti(I)),num2str(nlags),num2str(fixparam),num2str(JustMode));    

end

%% prior params file (first block has already been thrown out)
numb3 = nburn*npara*4;  % number of bites per file
fid3 = fopen(infile3,'r');		% input file (set to read)               
if ~isempty(mspec_ref)
  %% w/ correcting model -  have to  discard nburns
  status = fseek(fid3,numb3,'bof');       % discard nburn 
  num3 = nblocks*nsim*npara*4;  % number of bites per file
else
  num3 = (nblocks*nsim-nburn)*npara*4;  % number of bites per file  
end

%% MA representation: TTT (nstate x nstate) and RRR (nstate x nshocks)
numb11 = nburn*nstate^2*4;  % number of bites per file
fid11 = fopen(infile11,'r');		% input file (set to read) 
if ~isempty(mspec_ref)
  %% w/ correcting model -  have to  discard nburns
  status = fseek(fid11,numb11,'bof');       % discard nburn 
  num11 = nblocks*nsim*nstate^2*4;  % number of bites per file
else
  num11 = (nblocks*nsim-nburn)*nstate^2*4;  % number of bites per file
end

numb12 = nburn*nstate*nshocks*4;  % number of bites per file
fid12 = fopen(infile12,'r');		% input file (set to read) 
if ~isempty(mspec_ref)
  %% w/ correcting model -  have to  discard nburns
  status = fseek(fid12,numb12,'bof');       % discard nburn 
  num12 = nblocks*nsim*nstate*nshocks*4;  % number of bites per file
else
  num12 = (nblocks*nsim-nburn)*nstate*nshocks*4;  % number of bites per file
end


fid01 = fopen(outfile01,'w');		% output file (set to write)   
fid02 = fopen(outfile02,'w');		% output file (set to write)   
fid03 = fopen(outfile03,'w');		% output file (set to write)   
fid04 = fopen(outfile04,'w');		% output file (set to write)   

priotheta = [];
iblock = 0;
while ( ftell(fid3) < num3 ) 
  % Read blocks of size nsim

  iblock = iblock+1;
  fprintf(1,' block: %2.0f  ;',iblock);
  priotheta = fread(fid3,[npara,nsim],'single')';
  TTTsim = fread(fid11,[nstate^2,nsim],'single')';
  RRRsim = fread(fid12,[nstate*nshocks,nsim],'single')';

  statssim = zeros(nsim,length(stats_0));
  yyyysim = zeros(nsim,nvar^2);
  xxyysim = zeros(nsim,k*nvar);
  xxxxsim = zeros(nsim,k^2);
  for j = 1:nsim

    params = priotheta(j,:)';          
    if any(isnan(params))
      statssim(j,:) = NaN;
      yyyysim(j,:) = NaN;
      xxyysim(j,:) = NaN;
      xxxxsim(j,:) = NaN;

    else
      
      TTT = reshape(TTTsim(j,:),nstate,nstate);
      RRR = reshape(RRRsim(j,:),nstate,nshocks);

      %valid = 1;
      %[yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,params,nvar,nlags,mspec,npara,coint,cointadd,cointall);
      
      [ZZ,DD,DDcointadd,QQ,EE,MM,retcode] = measur(TTT,RRR,valid,params,nvar,nlags,mspec,npara,coint,cointadd,cointall);
      if POPUL

        if retcode == 0

          yyyyd = zeros(nvar,nvar);
          xxyyd = zeros(1+cointall+nlags*nvar,nvar);
          xxxxd = zeros(1+cointall+nlags*nvar,1+cointall+nlags*nvar);

          return;
        end;

        HH = EE+MM*QQ*MM';
        VV = QQ*MM';

        % Compute nlags autocovariances

        % Initialize Autocovariances

        GAMM0 = zeros((nvar+coint)^2,nlags+1);

        % GA0 = d2lyap(TTT,RRR*QQ*RRR'); %% this is the .m version of dlyap.mexglx
        Dd = det(RRR*QQ*RRR');
        if ~isnan(Dd)
          GA0 = dlyap(TTT,RRR*QQ*RRR');

          Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
          GAMM0(:,1) = Gl(:);
          
          TTl = TTT;
          for l = 1:nlags;
            Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);
            GAMM0(:,l+1) = Gl(:);            
            TTl = TTl*TTT;
          end

          %% Create limit cross product matrices
          yyyydcoint0 = reshape(GAMM0(:,1),nvar+coint,nvar+coint) + DD*DD';
          yyyydcoint1 = reshape(GAMM0(:,2),nvar+coint,nvar+coint) + DD*DD';
          yyyyd       = yyyydcoint0(1:nvar,1:nvar);

          %% cointadd are treated as the first set of variables in XX
          %% coint    are treated as the second set of variables in XX
          %% composition: cointadd - coint - constant - lags
          yyxxd = zeros(nvar,cointall+1+nlags*nvar);
          xxxxd = zeros(cointall+1+nlags*nvar,cointall+1+nlags*nvar);

          %% E[yy,x(coint)]
          yyxxd(:,cointadd+1:cointall) = yyyydcoint1(1:nvar,nvar+1:nvar+coint);

          %% E[yy,x(constant)]
          yyxxd(:,cointall+1) = DD(1:nvar);

          %% E[x(coint),x(coint)]
          xxxxd(cointadd+1:cointall,cointadd+1:cointall) = yyyydcoint0(nvar+1:nvar+coint,nvar+1:nvar+coint);

          %% E[x(const),x(const)]
          xxxxd(cointall+1,cointall+1) = 1;

          %% E[x(coint),x(const)]
          xxxxd(cointadd+1:cointall,cointall+1) = DD(nvar+1:nvar+coint);
          xxxxd(cointall+1,cointadd+1:cointall) = xxxxd(cointadd+1:cointall,cointall+1)';

          %% E[x(const),x(lags)]
          xxxxd(cointall+1,cointall+1+1:cointall+1+nlags*nvar) = kron(ones(1,nlags),DD(1:nvar)');
          xxxxd(cointall+1+1:cointall+1+nlags*nvar,cointall+1) = xxxxd(cointall+1,cointall+1+1:cointall+1+nlags*nvar)';

          if cointadd > 0

            %% E[yy,x(cointadd)]
            yyxxd(:,1:cointadd) = DD(1:nvar)*DDcointadd'/2;

            %% E[x(cointadd),x(cointadd)]
            xxxxd(1:cointadd,1:cointadd) = DDcointadd*DDcointadd'/3;

            %% E[x(cointadd),x(coint)]
            xxxxd(1:cointadd,1+cointadd:cointall) = DDcointadd*DD(nvar+1:nvar+coint)'/2;
            xxxxd(1+cointadd:cointall,1:cointadd) = DD(nvar+1:nvar+coint)*DDcointadd'/2;

            %% E[x(cointadd),x(const)]
            xxxxd(1:cointadd,cointall+1) = DDcointadd/2;
            xxxxd(cointall+1,1:cointadd) = DDcointadd'/2;

          end

          for rr = 1:nlags;
            %% E[yy,x(lag rr)]
            yyyydcointrr = reshape(GAMM0(:,rr+1),nvar+coint,nvar+coint) + DD*DD';
            yyxxd(:,cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr) = yyyydcointrr(1:nvar,1:nvar);

            if cointadd > 0
              %% E[x(cointadd),x(lag rr)]
              xxxxd(1:cointadd,cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr) = DDcointadd*DD(1:nvar)'/2;
              xxxxd(cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr,1:cointadd) = DD(1:nvar)*DDcointadd'/2;
            end
            
            %% E[x(coint),x(lag rr)]
            yyyydcointrr1 = reshape(GAMM0(:,rr),nvar+coint,nvar+coint) + DD*DD';    
            xxxxd(cointadd+1:cointall,cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr) = yyyydcointrr1(nvar+1:nvar+coint,1:nvar);
            xxxxd(cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr,cointadd+1:cointall) = yyyydcointrr1(nvar+1:nvar+coint,1:nvar)';
            
            %% E[x(lag rr),x(lag ll)]
            for ll = rr:nlags;  
              yyyydcointrrll = reshape(GAMM0(:,ll-rr+1),nvar+coint,nvar+coint)+DD*DD';
              xxxxd(cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr,cointall+1+nvar*(ll-1)+1:cointall+1+nvar*ll) = yyyydcointrrll(1:nvar,1:nvar);
              xxxxd(cointall+1+nvar*(ll-1)+1:cointall+1+nvar*ll,cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr) = yyyydcointrrll(1:nvar,1:nvar)';
            end
          end
          xxyyd = yyxxd';

          YYYYd = yyyyd;
          XXYYd = xxyyd;
          XXXXd = xxxxd;

          MeY = xxyyd(1+cointall,:)';
          VaY = yyyyd-MeY*MeY';
          stdY = sqrt(diag(VaY));
          CorY = VaY./(stdY*stdY');
          CoY = xxyyd(1+cointall+1:1+cointall+nvar,:)-MeY*MeY';
          ACorY = diag(CoY)./diag(VaY);
          
          
          statssim(j,:) = [MeY' stdY' ACorY' CorY(:)'];
          yyyysim(j,:) = [yyyyd(:)'];
          xxyysim(j,:) = [xxyyd(:)'];
          xxxxsim(j,:) = [xxxxd(:)'];
          
        else
          statssim(j,:) = NaN;
          yyyysim(j,:) = NaN;
          xxyysim(j,:) = NaN;
          xxxxsim(j,:) = NaN;
        end

      else %% generate data and computes moments from that
        
        DD_coint = DD(nvar+1:nvar+coint,:);
        ZZ_ = ZZ;
        DD_ = DD;
        %% Remove cointegration component from measurement equation	                
        ZZ = ZZ(1:nvar,:);
        DD = DD(1:nvar,:);
        EE = EE(1:nvar,1:nvar);
        MM = MM(1:nvar,:);


        %% set sample length
        nobsd = nobs+nlags;

        Shocks = randn(nobsd,nshocks);
        Shocks0 = Shocks*(QQ^.5);

        z = zeros(nstate,1);
        series = zeros(nobsd,nvar+coint);

        for t = 1:nobsd
          z = TTT*z+RRR*Shocks0(t,:)';
          series(t,:) = (DD_+ZZ_*z)';
        end;

        series_coint = series(:,nvar+1:nvar+coint);
        series = series(:,1:nvar);


        %% construct VAR data

        YYd = series(1+nlags:nobsd,:);
        XXd = ones(nobsd-nlags, coint+1+nvar*nlags);
        for i = 1:1:nlags;
          XXd(:,1+coint+(i-1)*nvar+1:1+coint+i*nvar) = series(nlags-i+1:nobsd-i,:);
        end

        if coint > 0
          XXd(:,1:coint) = series_coint(nlags:nobsd-1,:);
        end
        
        YYYYd = YYd'*YYd/nobs;
        XXYYd = XXd'*YYd/nobs;
        XXXXd = XXd'*XXd/nobs;
        

        MeY = mean(YYd,1)';
        VaY = cov(YYd,1);
        stdY = sqrt(diag(VaY));
        CorY = VaY./(stdY*stdY');
        ACorY = zeros(nvar,1);
        for ii = 1:nvar;
          ACorY(ii) = corr(YYd(:,ii),XXd(:,1+cointall+ii));
        end
%if ACorY(4)<0; disp([j, ACorY(4)]); end
        statssim(j,:) = [MeY' stdY' ACorY' CorY(:)'];

      end
      
      yyyysim(j,:) = [YYYYd(:)'];
      xxyysim(j,:) = [XXYYd(:)'];
      xxxxsim(j,:) = [XXXXd(:)'];
    end
    
    
  end
  fwrite(fid01,statssim','single');

  fwrite(fid02,yyyysim','single');
  fwrite(fid03,xxyysim','single');
  fwrite(fid04,xxxxsim','single');
  
end;
fclose(fid3);
fclose(fid11);
fclose(fid12);

fclose(fid01);
fclose(fid02);
fclose(fid03);
fclose(fid04);



