close all;
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PRELIMINARY STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

run startdsgeprioeuro;

rand('state',sum(100*clock))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      paths & parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chdata = choosedata;
[mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdas,para_names,bounds,para, ...
 varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode, ...
 stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix] = varspec(chdata);
qahead = 40;

[fpath,spath,gpath] = pathspec(num2str(floor(dataset)),chdata);

fprintf(1,'model: %2.0f, dataset: %2.1f, prior: %2.0f, cointall: %2.0f,',[mspec,dataset,mprior,cointall]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[YYall,XXall,ti,nobsall] = loaddata(dataset,fpath,T0,Tend,coint,cointadd,cointall,nvar,nlags,chdata,mspec);

dataset = num2str(10*dataset);

ICUM = find(cum_for);
INCUM = find(cum_for == 0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load information for prior distribution

lprior    = num2str(mprior);
lmodel    = strcat('m',num2str(mspec));
priorfile = strcat(lmodel,lprior,'pri');
prior = feval(priorfile);

pmean  = prior(1:npara,1);
pstdd  = prior(1:npara,2);
pshape = prior(1:npara,3);

pshape = pshape.*(1-para_mask);
pmean = pmean.*(1-para_mask)+para_fix.*para_mask;
pstdd = pstdd.*(1-para_mask);


nonpolipar = [1:1:npara];
nonpolipar(polipar) = [];

% Load transformation scheme for parameters
trspec = feval('transp',mspec);

trspec(:,1) = trspec(:,1).*(1-para_mask);



%% load some params
lambdas0 = NaN;
nlags0 = 4;
coint0 = 0;

lambdas0 = NaN;
nlags0 = nlags;
coint0 = coint;
fixparam = 0;

lambdas = Inf

%infile0 = strcat(spath,lmodel,lprior,'para',num2str(cointall),st,num2str(100*lambdas0),num2str(100*ti(end)),num2str(nlags0),'Mean');
infile0 = strcat(spath,lmodel,lprior,'para',num2str(cointall),st,num2str(100*lambdas0),num2str(100*ti(end)),num2str(nlags0),'Mode');            fid0 = fopen(infile0,'r');     % input file (set to read)   
            params = fread(fid0,[npara,1],'single');
            fclose(fid0);

%% remove wadj
%params(21) = 0;


%% get TTT ...
[TTT,RRR,valid] = dsgesolv(params,mspec);
[ZZ,DD,DDcointadd,QQ,EE,MM,retcode] = measur(TTT,RRR,valid,params,nvar,nlags,mspec,npara,coint,cointadd,cointall);

DD_coint = DD(nvar+1:nvar+coint,:);
ZZ_ = ZZ;
DD_ = DD;
%% Remove cointegration component from measurement equation	                
ZZ = ZZ(1:nvar,:);
DD = DD(1:nvar,:);
EE = EE(1:nvar,1:nvar);
MM = MM(1:nvar,:);

nshocks = size(RRR,2);
nstate = size(TTT,1);

%% get rotation matrix
nirf = 16;
aairf = irfdsge(TTT,RRR,params,nvar,mspec,npara,nirf,cum_irf,1,nlags,coint,cointadd,cointall);

a0_m = [];
for ii=1:nvar
  a0_m = [a0_m,aairf(1,nvar*(ii-1)+1:nvar*ii)'];
end; 

[phimat,r_a] = qr(a0_m');         
phimat = phimat';
%% phimat is normalized (row signs are changed) so that the lower diagonal matrix (r_a') 
%% has positive diagonal elements  
phimat = repmat(sign(diag(r_a)),[1 nvar]).*phimat;

%% set sample length
nobs = 120+nlags;

Shocks = randn(nobs,nshocks);
Shocks0 = Shocks*(QQ^.5);
disp([var(Shocks0);diag(QQ)'])


z = zeros(nstate,1);
series = zeros(nobs,nvar+coint);

for t = 1:nobs
  z = TTT*z+RRR*Shocks0(t,:)';
  series(t,:) = (DD_+ZZ_*z)';
end;

series_coint = series(:,nvar+1:nvar+coint);
series = series(:,1:nvar);


dm([mean(series);DD']);
dm([mean(series_coint);DD_coint']);

%% construct VAR parameters
coint = 3
nlags = 4

%% construct VAR data

YY = series(1+nlags:nobs,:);
XX = ones(nobs-nlags, coint+1+nvar*nlags);
Shocks = Shocks(1+nlags:nobs,:);

for i = 1:1:nlags;
  XX(:,1+coint+(i-1)*nvar+1:1+coint+i*nvar) = series(nlags-i+1:nobs-i,:);
end

if coint > 0
  XX(:,1:coint) = series_coint(nlags:nobs-1,:);
  XX = [XX;[series_coint(nobs,:),1,YY(end,:),XX(end,1+coint+1:1+coint+nvar*nlags-nvar)]];
else
  XX = [XX;[1,YY(end,:),XX(end,1+1:1+nvar*nlags-nvar)]];           
end
XXpred = XX(end,:);
XX = XX(1:end-1,:);
nobs = size(YY,1)

[yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,params,nvar,nlags,mspec,npara,coint,cointadd,cointall);



if (lambdas > 0) & ~isinf(lambdas)
  YYYY = YY'*YY;
  XXYY = XX'*YY;
  XXXX = XX'*XX;


  yyyyd = YYYY + nobs*lambdas*yyyyd;
  xxyyd = XXYY + nobs*lambdas*xxyyd;
  xxxxd = XXXX + nobs*lambdas*xxxxd;
  
elseif isinf(lambdas)

  yyyyd = yyyyd;
  xxyyd = xxyyd;
  xxxxd = xxxxd;
  
end
nobsc = nobs + nobs*lambdas;    
k     = 1+nlags*nvar+coint;

%% Compute OLS estimates and reduced form residuals
c_hat  = (xxxxd\xxyyd)';
if ~isinf(lambdas)
  sig_hat = (yyyyd -xxyyd'*c_hat')/nobsc;
elseif isinf(lambdas)
  sig_hat = (yyyyd -xxyyd'*c_hat');
end
sig_chol = chol(sig_hat)';
sig_id = sig_chol*phimat;

yprederror = YY - XX*c_hat';

%% compute  identified shocks
VARShocks_1 = yprederror/(sig_id');

fprintf(1,', Max Shock difference  %2.4f ',max(max(abs(Shocks-VARShocks_1))));



for I = 1:nshocks
  figure(1)
  subplot(ceil(nshocks/2),2,I); 
  P = plot(Shocks(:,I),'r-');
  hold on;
  P = plot(VARShocks_1(:,I),'k');
  hold off;
  title(names_shocks(I));

end
  figure(1)
  print(char(strcat(gpath,'/shockinvert.pdf')),'-dpdf');
%  print(char(strcat(gpath,'/irfinvert.eps')),'-depsc2');


cc_tilded =  [c_hat(:,(coint+1+1):k) ; ...
              [eye(nvar*(nlags-1)),zeros(nvar*(nlags-1),nvar)]];

if coint > 0
  cc_coint  =  c_hat(:,1:coint);
else
  cc_coint  =  zeros(nvar,coint);
end

cc_cointtilde = [cc_coint ; zeros(nvar*(nlags-1),coint)];


%% var's irf
%% Compute MA representation from companion form VAR
sig_chol = chol(sig_hat)';

cc_dy   = eye(nvar*nlags);
cc_y    = eye(nvar*nlags);
msel    = [eye(nvar) ; zeros((nlags-1)*nvar,nvar)];

bbirf    = zeros(nirf,nvar^2);

out         = msel'*cc_dy*msel*sig_chol;
bbirf(1,:)  = out(:)';

for t = 2:nirf;                 
  cc_dy = cc_tilded*cc_dy;
  if coint
    cc_dy = cc_dy+(cc_cointtilde*cointvec*msel')*cc_y;
    cc_y  = cc_y + cc_dy;
  end
  out = msel'*cc_dy*msel*sig_chol;
  bbirf(t,:)    = out(:)';  
end;

ccum_irf = [];
for II=1:length(cum_irf)
  ccum_irf = [ccum_irf,cum_irf(II):nvar:nvar^2];    
end							
bbirf(:,ccum_irf) = cumsum(bbirf(:,ccum_irf),1);


bbirf_1 = bbirf*kron(phimat,eye(nvar));       


%% construct VAR parameters
coint = 0
nlags = 4


%% construct VAR data

YY = series(1+nlags:nobs,:);
XX = ones(nobs-nlags, coint+1+nvar*nlags);
Shocks = Shocks(1+nlags:nobs,:);

for i = 1:1:nlags;
  XX(:,1+coint+(i-1)*nvar+1:1+coint+i*nvar) = series(nlags-i+1:nobs-i,:);
end

if coint > 0
  XX(:,1:coint) = series_coint(nlags:nobs-1,:);
  XX = [XX;[series_coint(nobs,:),1,YY(end,:),XX(end,1+coint+1:1+coint+nvar*nlags-nvar)]];
else
  XX = [XX;[1,YY(end,:),XX(end,1+1:1+nvar*nlags-nvar)]];           
end
XXpred = XX(end,:);
XX = XX(1:end-1,:);
nobs = size(YY,1)


cointall = coint;

[yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,params,nvar,nlags,mspec,npara,coint,cointadd,cointall);

if (lambdas > 0) & ~isinf(lambdas)
  YYYY = YY'*YY;
  XXYY = XX'*YY;
  XXXX = XX'*XX;


  yyyyd = YYYY + nobs*lambdas*yyyyd;
  xxyyd = XXYY + nobs*lambdas*xxyyd;
  xxxxd = XXXX + nobs*lambdas*xxxxd;
  
elseif isinf(lambdas)

  yyyyd = yyyyd;
  xxyyd = xxyyd;
  xxxxd = xxxxd;
  
end
nobsc = nobs + nobs*lambdas;    
k     = 1+nlags*nvar+coint;

%% Compute OLS estimates and reduced form residuals
c_hat  = (xxxxd\xxyyd)';
if ~isinf(lambdas)
  sig_hat = (yyyyd -xxyyd'*c_hat')/nobsc;
elseif isinf(lambdas)
  sig_hat = (yyyyd -xxyyd'*c_hat');
end
sig_chol = chol(sig_hat)';
sig_id = sig_chol*phimat;

yprederror = YY - XX*c_hat';

%% compute  identified shocks
VARShocks_2 = yprederror/(sig_id');

cc_tilded =  [c_hat(:,(coint+1+1):k) ; ...
              [eye(nvar*(nlags-1)),zeros(nvar*(nlags-1),nvar)]];

if coint > 0
  cc_coint  =  c_hat(:,1:coint);
else
  cc_coint  =  zeros(nvar,coint);
end

cc_cointtilde = [cc_coint ; zeros(nvar*(nlags-1),coint)];

%% var's irf
%% Compute MA representation from companion form VAR
sig_chol = chol(sig_hat)';

cc_dy   = eye(nvar*nlags);
cc_y    = eye(nvar*nlags);
msel    = [eye(nvar) ; zeros((nlags-1)*nvar,nvar)];

bbirf    = zeros(nirf,nvar^2);

out         = msel'*cc_dy*msel*sig_chol;
bbirf(1,:)  = out(:)';

for t = 2:nirf;                 
  cc_dy = cc_tilded*cc_dy;
  if coint
    cc_dy = cc_dy+(cc_cointtilde*cointvec*msel')*cc_y;
    cc_y  = cc_y + cc_dy;
  end
  out = msel'*cc_dy*msel*sig_chol;
  bbirf(t,:)    = out(:)';  
end;

ccum_irf = [];
for II=1:length(cum_irf)
  ccum_irf = [ccum_irf,cum_irf(II):nvar:nvar^2];    
end							
bbirf(:,ccum_irf) = cumsum(bbirf(:,ccum_irf),1);


bbirf_2 = bbirf*kron(phimat,eye(nvar));       



%% construct VAR parameters
coint = 3
nlags = 6

%% construct VAR data

YY = series(1+nlags:nobs,:);
XX = ones(nobs-nlags, coint+1+nvar*nlags);
Shocks = Shocks(1+nlags:nobs,:);

for i = 1:1:nlags;
  XX(:,1+coint+(i-1)*nvar+1:1+coint+i*nvar) = series(nlags-i+1:nobs-i,:);
end

if coint > 0
  XX(:,1:coint) = series_coint(nlags:nobs-1,:);
  XX = [XX;[series_coint(nobs,:),1,YY(end,:),XX(end,1+coint+1:1+coint+nvar*nlags-nvar)]];
else
  XX = [XX;[1,YY(end,:),XX(end,1+1:1+nvar*nlags-nvar)]];           
end
XXpred = XX(end,:);
XX = XX(1:end-1,:);
nobs = size(YY,1)


cointall = coint;

[yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,params,nvar,nlags,mspec,npara,coint,cointadd,cointall);

if (lambdas > 0) & ~isinf(lambdas)
  YYYY = YY'*YY;
  XXYY = XX'*YY;
  XXXX = XX'*XX;


  yyyyd = YYYY + nobs*lambdas*yyyyd;
  xxyyd = XXYY + nobs*lambdas*xxyyd;
  xxxxd = XXXX + nobs*lambdas*xxxxd;
  
elseif isinf(lambdas)

  yyyyd = yyyyd;
  xxyyd = xxyyd;
  xxxxd = xxxxd;
  
end
nobsc = nobs + nobs*lambdas;    
k     = 1+nlags*nvar+coint;

%% Compute OLS estimates and reduced form residuals
c_hat  = (xxxxd\xxyyd)';
if ~isinf(lambdas)
  sig_hat = (yyyyd -xxyyd'*c_hat')/nobsc;
elseif isinf(lambdas)
  sig_hat = (yyyyd -xxyyd'*c_hat');
end
sig_chol = chol(sig_hat)';
sig_id = sig_chol*phimat;

yprederror = YY - XX*c_hat';

%% compute  identified shocks
VARShocks_3 = yprederror/(sig_id');


cc_tilded =  [c_hat(:,(coint+1+1):k) ; ...
              [eye(nvar*(nlags-1)),zeros(nvar*(nlags-1),nvar)]];

if coint > 0
  cc_coint  =  c_hat(:,1:coint);
else
  cc_coint  =  zeros(nvar,coint);
end

cc_cointtilde = [cc_coint ; zeros(nvar*(nlags-1),coint)];

%% var's irf
%% Compute MA representation from companion form VAR
sig_chol = chol(sig_hat)';

cc_dy   = eye(nvar*nlags);
cc_y    = eye(nvar*nlags);
msel    = [eye(nvar) ; zeros((nlags-1)*nvar,nvar)];

bbirf    = zeros(nirf,nvar^2);

out         = msel'*cc_dy*msel*sig_chol;
bbirf(1,:)  = out(:)';

for t = 2:nirf;                 
  cc_dy = cc_tilded*cc_dy;
  if coint
    cc_dy = cc_dy+(cc_cointtilde*cointvec*msel')*cc_y;
    cc_y  = cc_y + cc_dy;
  end
  out = msel'*cc_dy*msel*sig_chol;
  bbirf(t,:)    = out(:)';  
end;

ccum_irf = [];
for II=1:length(cum_irf)
  ccum_irf = [ccum_irf,cum_irf(II):nvar:nvar^2];    
end							
bbirf(:,ccum_irf) = cumsum(bbirf(:,ccum_irf),1);


bbirf_3 = bbirf*kron(phimat,eye(nvar));       






titlenames = varnames;
%titlenames(cum_irf) = strcat(titlenames(cum_irf),' - cum');
tirf = (1:1:nirf)';

for Iirf = 1:nvar
  for V = 1:nvar
    figure(1001);
    subplot(nvar,nvar,V+nvar*(Iirf-1));   
    m = aairf(:,V+nvar*(Iirf-1));   

    P = plot(tirf,m,'k-');
    set(P,'LineWidth',1.5);
    hold on;

    mbb_1 = bbirf_1(:,V+nvar*(Iirf-1));
    P = plot(tirf,mbb_1,'k:');
    set(P,'LineWidth',1.5);
    hold on;

    
    mbb_2 = bbirf_2(:,V+nvar*(Iirf-1));
    %P = plot(tirf,mbb_2,'k-.');
    set(P,'LineWidth',1.5);
    hold on;

    mbb_3 = bbirf_3(:,V+nvar*(Iirf-1));
%    P = plot(tirf,mbb_3,'r-.');
    set(P,'LineWidth',1);
    hold off;
    
    fprintf(1,'\n shock: %2.99s',names_shocks{Iirf});
    fprintf(1,'; variable: %2.99s',titlenames{V});
    fprintf(1,', max VECM %2.4f, max VAR %2.4f ',[max(abs(m-mbb_1)),max(abs(m-mbb_3))]);
    fprintf(1,', PERCENT: max VECM %2.4f, max VAR %2.4f ',[max(abs(m-mbb_1)./abs(m)),max(abs(m-mbb_3)./abs(m))]);
    
    if Iirf == 1
      title(titlenames(V));
    end
    if V == 1 
      ylabel(names_shocks(Iirf));
    end
    DORES = 0;
    if DORES
      if V == 1 
        if m(3) > 0
          set(gca,'YLim',[-.2,2.2]);
        elseif Iirf == 6
          set(gca,'YLim',[-1,2.2]);                    
        end                  
      elseif V == 2 
        %                    set(gca,'YLim',[-1,2.2]);
      elseif V == 3
        if Iirf == 5
          set(gca,'YLim',[-5,5]);                    
        end                  
      elseif V == 5
        if Iirf == 5
          set(gca,'YLim',[-.2,.2]);                    
        end                  
      elseif V == 6
        if Iirf == 5
          set(gca,'YLim',[-.1,.1]);                    
        end                  
      end
    end
    
    set(gca,'XLim',[min(tirf),max(tirf)]);
    set(gca,'XTick',tirf([1:10:end]));
    if Iirf == nvar
      set(gca,'XTickLabel',num2str(tirf([1:10:end])-1));       
    else 
      set(gca,'XTickLabel',[]);
    end		
    set(gca,'FontSize',8.5);
  end                                                
end	


saveas(figure(1001),strcat(gpath,'/irfinvert.fig'),'fig');
  print(char(strcat(gpath,'/',lmodel,lprior,num2str(cointall),st,num2str(100*lambdas0),'irfinvert.eps')),'-depsc2');
  print(char(strcat(gpath,'/',lmodel,lprior,num2str(cointall),st,num2str(100*lambdas0),'irfinvert.pdf')),'-dpdf');


