function [yyyyd,xxyyd,xxxxd,retcode] = varprio(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint,cointadd,cointall);
%% description:
%% solution to DSGE model - delivers transition equation for the state variables  S_t
%% transition equation: S_t = TC+TTT S_{t-1} +RRR eps_t, where var(eps_t) = QQ
%% define the measurement equation: X_t = ZZ S_t +D+u_t
%% where var(u_t) = HH, cov(eps_t,u_t) = VV
%% and compute  cross moment matrices for the DSGE prior

[ZZ,DD,DDcointadd,QQ,EE,MM,retcode] = measur(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint,cointadd,cointall);

if retcode == 0

    yyyyd = zeros(nvar,nvar);
    xxyyd = zeros(1+cointall+nlags*nvar,nvar);
    xxxxd = zeros(1+cointall+nlags*nvar,1+cointall+nlags*nvar);

    return;
end;

HH = EE+MM*QQ*MM';
VV = QQ*MM';

% Compute nlags autocovariances

% Initialize Autocovariances

GAMM0 = zeros((nvar+coint)^2,nlags+1);

% GA0 = d2lyap(TTT,RRR*QQ*RRR'); %% this is the .m version of dlyap.mexglx
GA0 = dlyap(TTT,RRR*QQ*RRR');

Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
GAMM0(:,1) = Gl(:);
   
TTl = TTT;
for l = 1:nlags;
    Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);
    GAMM0(:,l+1) = Gl(:);            
    TTl = TTl*TTT;
end

%% Create limit cross product matrices
yyyydcoint0 = reshape(GAMM0(:,1),nvar+coint,nvar+coint) + DD*DD';
yyyydcoint1 = reshape(GAMM0(:,2),nvar+coint,nvar+coint) + DD*DD';
yyyyd       = yyyydcoint0(1:nvar,1:nvar);

%% cointadd are treated as the first set of variables in XX
%% coint    are treated as the second set of variables in XX
%% composition: cointadd - coint - constant - lags
yyxxd = zeros(nvar,cointall+1+nlags*nvar);
xxxxd = zeros(cointall+1+nlags*nvar,cointall+1+nlags*nvar);

%% E[yy,x(coint)]
yyxxd(:,cointadd+1:cointall) = yyyydcoint1(1:nvar,nvar+1:nvar+coint);

%% E[yy,x(constant)]
yyxxd(:,cointall+1) = DD(1:nvar);

%% E[x(coint),x(coint)]
xxxxd(cointadd+1:cointall,cointadd+1:cointall) = yyyydcoint0(nvar+1:nvar+coint,nvar+1:nvar+coint);

%% E[x(const),x(const)]
xxxxd(cointall+1,cointall+1) = 1;

%% E[x(coint),x(const)]
xxxxd(cointadd+1:cointall,cointall+1) = DD(nvar+1:nvar+coint);
xxxxd(cointall+1,cointadd+1:cointall) = xxxxd(cointadd+1:cointall,cointall+1)';

%% E[x(const),x(lags)]
xxxxd(cointall+1,cointall+1+1:cointall+1+nlags*nvar) = kron(ones(1,nlags),DD(1:nvar)');
xxxxd(cointall+1+1:cointall+1+nlags*nvar,cointall+1) = xxxxd(cointall+1,cointall+1+1:cointall+1+nlags*nvar)';

if cointadd > 0

   %% E[yy,x(cointadd)]
   yyxxd(:,1:cointadd) = DD(1:nvar)*DDcointadd'/2;

   %% E[x(cointadd),x(cointadd)]
   xxxxd(1:cointadd,1:cointadd) = DDcointadd*DDcointadd'/3;

   %% E[x(cointadd),x(coint)]
   xxxxd(1:cointadd,1+cointadd:cointall) = DDcointadd*DD(nvar+1:nvar+coint)'/2;
   xxxxd(1+cointadd:cointall,1:cointadd) = DD(nvar+1:nvar+coint)*DDcointadd'/2;

   %% E[x(cointadd),x(const)]
   xxxxd(1:cointadd,cointall+1) = DDcointadd/2;
   xxxxd(cointall+1,1:cointadd) = DDcointadd'/2;

end

for rr = 1:nlags;
    %% E[yy,x(lag rr)]
    yyyydcointrr = reshape(GAMM0(:,rr+1),nvar+coint,nvar+coint) + DD*DD';
    yyxxd(:,cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr) = yyyydcointrr(1:nvar,1:nvar);

    if cointadd > 0
       %% E[x(cointadd),x(lag rr)]
       xxxxd(1:cointadd,cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr) = DDcointadd*DD(1:nvar)'/2;
       xxxxd(cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr,1:cointadd) = DD(1:nvar)*DDcointadd'/2;
    end
    
    %% E[x(coint),x(lag rr)]
    yyyydcointrr1 = reshape(GAMM0(:,rr),nvar+coint,nvar+coint) + DD*DD';    
    xxxxd(cointadd+1:cointall,cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr) = yyyydcointrr1(nvar+1:nvar+coint,1:nvar);
    xxxxd(cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr,cointadd+1:cointall) = yyyydcointrr1(nvar+1:nvar+coint,1:nvar)';
    
    %% E[x(lag rr),x(lag ll)]
    for ll = rr:nlags;  
        yyyydcointrrll = reshape(GAMM0(:,ll-rr+1),nvar+coint,nvar+coint)+DD*DD';
        xxxxd(cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr,cointall+1+nvar*(ll-1)+1:cointall+1+nvar*ll) = yyyydcointrrll(1:nvar,1:nvar);
        xxxxd(cointall+1+nvar*(ll-1)+1:cointall+1+nvar*ll,cointall+1+nvar*(rr-1)+1:cointall+1+nvar*rr) = yyyydcointrrll(1:nvar,1:nvar)';
    end
end

xxyyd = yyxxd';
clear yyxxd;
