function [yyyyd,xxyyd,xxxxd,retcode] = varprio_dmean(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint,cointadd,cointall);
%% description:
%% solution to DSGE model - delivers transition equation for the state variables  S_t
%% transition equation: S_t = TC+TTT S_{t-1} +RRR eps_t, where var(eps_t) = QQ
%% define the measurement equation: X_t = ZZ S_t +D+u_t
%% where var(u_t) = HH, cov(eps_t,u_t) = VV
%% and compute  cross moment matrices for the DSGE prior

[ZZ,DD,DDcointadd,QQ,EE,MM,retcode] = measur(TTT,RRR,valid,para,nvar,nlags,mspec,npara,coint,cointadd,cointall);

if retcode == 0

    yyyyd = zeros(nvar,nvar);
    xxyyd = zeros(nlags*nvar,nvar);
    xxxxd = zeros(nlags*nvar,nlags*nvar);

    return;
end;

HH = EE+MM*QQ*MM';
VV = QQ*MM';

% Compute nlags autocovariances

% Initialize Autocovariances

GAMM0 = zeros(nvar^2,nlags);

% GA0 = d2lyap(TTT,RRR*QQ*RRR'); %% this is the .m version of dlyap.mexglx
GA0 = dlyap(TTT,RRR*QQ*RRR');

Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
GAMM0(:,1) = Gl(:);
   
TTl = TTT;
for l = 1:nlags;
    Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);
    GAMM0(:,l+1) = Gl(:);            
    TTl = TTl*TTT;
end

%% Create limit cross product matrices
yyyyd0 = reshape(GAMM0(:,1),nvar,nvar);


%yyyyd1 = reshape(GAMM0(:,2),nvar,nvar);
yyyyd  = yyyyd0(1:nvar,1:nvar);

yyxxd = zeros(nvar,nlags*nvar);
xxxxd = zeros(nlags*nvar,nlags*nvar);

for rr = 1:nlags;
    %% E[yy,x(lag rr)]
    yyxxd(:,nvar*(rr-1)+1:nvar*rr) = reshape(GAMM0(:,rr+1),nvar,nvar);
    
    %% E[x(lag rr),x(lag ll)]
    for ll = rr:nlags;  
        yyyydrrll = reshape(GAMM0(:,ll-rr+1),nvar,nvar);
        xxxxd(nvar*(rr-1)+1:nvar*rr,nvar*(ll-1)+1:nvar*ll) = yyyydrrll(1:nvar,1:nvar);
        xxxxd(nvar*(ll-1)+1:nvar*ll,nvar*(rr-1)+1:nvar*rr) = yyyydrrll(1:nvar,1:nvar)';
    end
end

xxyyd = yyxxd';
clear yyxxd;
