%% This file contains the specification of the VAR model
%function [mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode,stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix,para_aux,mspec_ref,nobsdrat,lambdac,nlagsc,diago] = varspec;


%% choose model
mspec = 106

%% 101 - baseline - no indexation
%% 102 - partial indexation w/ tech update
%% 106 - partial indexation w/o tech update
%% 107 - full indexation w/o tech update
%% 103 - output gap targeting (partial indexation w/o tech update)
%% 104 - No price and wage rigidity
%% 105 - No wage rigidity 
%% 108 - No habit persistence
%% 109 - No inv adj cost
%% 110 - No cap utilization
%% 111 - No wage rigidity & price indexation
%% 112 - baseline w/ rho_lambda_f = 0
%% 113 - No price rigidity
%% 114 - no indexation w/ rho_lambda_f = 0
%% 5 - No wage rigidity - LS not observable 
%% 6 - partial indexation w/o tech update - LS not observable
%% 13 - No price rigidity - LS not observable

mspec_ref = NaN
nlagsc = 1
n_s = []
if isempty(mspec_ref)
  nobsdrat = NaN
  stup = [];
else
  nobsdrat = 6/100  %% 
  stup = 1
%% stup = 1; all 'endogenous' propagation mechanism parameters belong to para_type 1 (standard prior -- conditioning), exogenous 
%%           parameters belong to para_type 2 (quasi likelihood) 
%% stup = 2; all 'endogenous' and 'exogeous' propagation mechanism parameters belong to para_type 2  (quasi likelihood) 
end

%% if JustMode: do not do simulations and compute forecasts at mode
JustMode = 0           
%% choose prior
  %% * is flat on rho and jeffrey on sigma, *2 is standard,
mprior =  4
if isempty(mspec_ref)
  mprior = 10*mprior+2;
else
  %mprior = 10*mprior+1;  
  mprior = 10*mprior+3;  
end



%% choose dataset 
dataset = 6; %% 6 is w/ labor share

%% Effective sample size to generate prior 
lambdaseq = NaN%[ceil(100*((4*4+1)+4)/100)/100 .5 .75 1 1.25 1.5 2 5 Inf NaN];%%[0 ceil(1004((4*4+1)+4)/100)/100 .54.75 1 1.25 1.5 2 5 Inf NaN];

%% coint = 0 : just growth rates, otherwise number of cointegrating relationships
%% model 1: coint = 4
%% model 2: coint = 0
%% model 101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116: coint = 3
coint = 0;
cointadd = 0;
cointall = coint + cointadd;

%% number of lags
nlags = 4


    %% choose date
    dates = 2005.75;
 
    
    %% number of observations used in VAR (st = [] means use entire sample) 
    if any(mspec == [101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116]) | any(mspec == [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16]);
        stime = 120; 
        stime = 93; 
    end
%stime = []

    st = num2str(stime);
    
    %% End of Presample
    T0 = nlags; 
    
    Tend     = 0;        % End of Estimation Sample 
    



%% fixparamseq = 1 exogenous (fixed parameters) at para
fixparamseq = [0];


% quarters ahead for computing forecasts
qahead = 16;                                     

%% lags for auto/cross correlations
k_l = 8;


%% percentile for bands, etc
percent = .90

%%  Configure the Metropolis Algorithm (number of simulations)
nblocks = 21
nsim    = 10000
nburn = nsim;                                       % initial simulations to 'burn'

%% length of IRFFs
nirf = 41;

if any( dataset == [1,3,5,6] )
  %% number of  variables, names of variables, and variables for which we want cumulative forecasts
  if any(mspec == [101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116])
    nvar = 5;
    varnames = {'Y';'H';'Labor Share';'Inflation';'R'};
    varnames = {'Output Growth';'Labor Supply';'Labor Share';'Inflation';'Interest Rate'};
    cum_for = [  1   0        0           1        0];


    elseif any(mspec == [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16])
    nvar = 4;
    varnames = {'Y';'H';'Inflation';'R'};
    varnames = {'Output Growth';'Labor Supply';'Inflation';'Interest Rate'};
    cum_for = [  1   0        0         0];

  end
elseif dataset == 2
  %% number of  variables, names of variables, and variables for which we want cumulative forecasts
  nvar = 3;
  varnames = {'Y';'inflation';'R'};
  cum_for = [  0    0        0];
end

   
if any(mspec == [101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116])  | any(mspec == [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16]);



    %%%%%%%%%%%%%%%%%%%%%%%%%%
    %% names of shocks, and variables for which we want cumulative irf
    cum_irf = [1];


    if any(mspec == [101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116])  | any(mspec == [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16]);

        names_shocks = {'Tech';'\phi';'g';'\lambda_f';'Money'};    
        %names_shocks = {'Technology (z)';'Leisure (\phi)';'Gvmt Spending (g)';'Markup (\lambda_f)';'Money'};    
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    %% names of parameters 
    para_names = { '\alpha';'\zeta_p';'\iota_p';'\delta';'\Upsilon';'\Phi';'s^\prime';'h';'a^{\prime~\prime}';'\nu_l';'\nu_m';'\zeta_w';...
        '\iota_w';'\lambda_w';'r^*';...
        '\psi_1';'\psi_2';'\rho_{r}';'\pi^*';'\gamma';'W^{adj}';'\chi';'\lambda_f';'g^*';'L^{adj}';...
        '\rho_{z}';'\rho_{\phi}';'\rho_{\chi}';'\rho_{\lambda_f}';'\rho_{\mu}';'\rho_{b}';'\rho_{g}';...
        '\sigma_{z}';'\sigma_{\phi}';'\sigma_{\chi}';'\sigma_{\lambda_f}';'\sigma_{\mu}';'\sigma_{b}';'\sigma_{g}';'\sigma_{r}'};

    %%%%%%%%%%%%%%%%%%%%%%%%%%
    %% starting values for parameters
    para = zeros(100,1);

    %% para_mask is npara x 1
    %%   to fix a parameter under specification mspec, set the corresponding element of para_mask to 1
    %% para_fix is npara x 1
    %%   and contains the values for the fixed parameters (all other entries are irrelevant)

    para_mask = zeros(100,1);
    para_fix  = zeros(100,1);
    para_type  = zeros(100,1);

    %% para_type = 0 -- steady state
    %% para_type = 1 -- slope
    %% para_type = 2 -- auxiliary (rho, sigma)
    
    para(1) = .33;         para_type(1) = 0;   %para_mask(1) = 1;  para_fix(1) = para(1);              %% alp;         1
    if any(mspec == [101,102,103,105,106,107,108,109,110,111,112,114,115,116,1,2,3,5,6,7,8,9,10,11,12,14,15,16])
      para(2) = .75;       %para_mask(2) = 1;  para_fix(2) = para(2);              %% zeta_p;      2
    elseif any(mspec == [104,113,4,13])
      para(2) = 0.0001;    para_mask(2) = 1;   para_fix(2) = para(2);              %% zeta_p;      
    end
                           para_type(2) = 1;    
    if any(mspec == [101,104,111,113,114,1,4,11,13,14])
      para(3) = 0;   para_mask(3) = 1;   para_fix(3) = para(3);              %% iota_p;      3
    elseif any(mspec == [102,103,105,106,108,109,110,112,115,116,3,5,6,8,9,10,12,15,16])
      para(3) = .5;   para_type(3) = 0; %para_mask(3) = 1;   para_fix(3) = para(3);              %% iota_p;      3
    elseif any(mspec == [107,7])
      para(3) = 1;    para_mask(3) = 1;   para_fix(3) = para(3);              %% iota_p;      3
    end
    para_type(3) = 1;       if stup == 2;   para_type(3) = 2;    end;
                           
    para(4) = .025;        para_type(4) = 0; %para_mask(4) = 1;   para_fix(4) = para(4);              %% del;         4

    para(5) = 0;           para_type(5) = 0; para_mask(5) = 1;   para_fix(5) = para(5);              %% ups; .1      5

    para(6) = 0;           para_type(6)  =  0; para_mask(6) = 1;   para_fix(6) = para(6);              %% Bigphi;      6

    if any(mspec == [109,9])
      para(7) = 0;        para_mask(7) = 1;  para_fix(7) = para(7);              %% s2;      7
    else
      para(7) = 4;       %para_mask(7) = 1;  para_fix(7) = para(7);              %% s2;      7
    end
    para_type(7) = 1;       if stup == 2;   para_type(7) = 2;    end;
                           
    if any(mspec == [108,8])
      para(8) = 0;        para_mask(8) = 1;  para_fix(8) = para(8);              %% h;           8
    else
      para(8) = .7;       %para_mask(8) = 1;  para_fix(8) = para(8);              %% h;           8
    end
    para_type(8) = 1;       if stup == 2;   para_type(8) = 2;    end;

    if any(mspec == [110,10])
      para(9) = Inf;    para_mask(9) = 1;  para_fix(9) = para(9);              %% a2;          9
    else
      para(9) = .2;      %para_mask(9) = 1;  para_fix(9) = para(9);              %% a2;          9
    end
    para_type(9) = 1;       if stup == 2;   para_type(9) = 2;    end;

    para(10) = 2;           para_type(10) = 1; %para_mask(10) = 1;  para_fix(10) = para(10);           %% nu_l         10
    para(11) = 2;   para_type(11) = 1; para_mask(11) = 1;  para_fix(11) = para(11);            %% nu_m         11

    if any(mspec == [101,102,103,106,107,108,109,110,112,113,114,115,116,1,2,3,6,7,8,9,10,12,13,14,15,16])
      para(12) = .8;       %para_mask(12) = 1; para_fix(12) = para(12);            %% zeta_w;      12
    elseif any(mspec == [104,105,111,4,5,11])
      para(12) = .0001;    para_mask(12) = 1;  para_fix(12) = para(12);            %% zeta_w;      
    end
                           para_type(12) = 1;
    if any(mspec == [104,105,111,4,5,11])
      para(13) = 0;       para_mask(13) = 1;   para_fix(13) = para(13);          %% iota_w;      13
    elseif any(mspec == [101,102,103,106,108,109,110,112,113,114,115,116,1,2,3,6,8,9,10,12,13,14,15,16])
      para(13) = .5;    %para_mask(13) = 1;   para_fix(13) = para(13);          %% iota_w;      13
    elseif any(mspec == [107,7])
      para(13) = 1;       para_type(13) = 0; para_mask(13) = 1;   para_fix(13) = para(13);          %% iota_w;      13
    end
    para_type(13) = 1;       if stup == 2;   para_type(13) = 2;    end;

    para(14) = .3;          para_type(14) = 0; para_mask(14) = 1;  para_fix(14) = para(14);            %% law;         14

    para(15) = 2;           para_type(15) = 0; %para_mask(15) = 1; para_fix(15) = para(15);            %% beta;        15

    para(16) = 1.7;         para_type(16) = 1; %para_mask(16) = 1; para_fix(16) = para(16);            %% psi1;        16

    para(17) = .125;        para_type(17) = 1; %para_mask(17) = 1; para_fix(17) = para(17);            %% psi2;        17

    para(18) = .8;          para_type(18) = 1; %para_mask(18) = 1; para_fix(18) = para(18);            %% rho_r;       18

    para(19) = 3;           para_type(19) = 0; %para_mask(19) = 1; para_fix(19) = para(19);            %% pistar;      19
    

    npara = 19;

    %% exogenous processes - level
    para(npara+1) = 2;      para_type(npara+1) = 0; %para_mask(npara+1) = 1;    para_fix(npara+1) = para(npara+1);      %% gam;     
    para(npara+2) =  0;     para_type(npara+2) = 0; para_mask(npara+2) = 1;    para_fix(npara+2) = para(npara+2);      %% Wadj;

    para(npara+3) = .1;     para_type(npara+3) = 1; para_mask(npara+3) = 1;    para_fix(npara+3) = para(npara+3);      %% chi;     

    para(npara+4) = .15;    para_type(npara+4) = 0; %para_mask(npara+4) = 1;     para_fix(npara+4) = para(npara+4);      %% laf;     
    para(npara+5) = .15;    para_type(npara+5) = 0; %para_mask(npara+5) = 1;     para_fix(npara+5) = para(npara+5);      %% gstar;
    para(npara+6) = 252;    para_type(npara+6) = 0; %para_mask(npara+6) = 1;    para_fix(npara+6) = para(npara+6);      %% Ladj


    npara = npara+6;    

    %% exogenous processes - autocorrelation
    para(npara+1) = .2;    para_type(npara+1) = 2; %para_mask(npara+1) = 1;    para_fix(npara+1) = para(npara+1);      %% rho_z;
    
    para(npara+2) = .4;    para_type(npara+2) = 2; %para_mask(npara+2) = 1;    para_fix(npara+2) = para(npara+2);      %% rho_phi;

    para(npara+3) = 0;     para_type(npara+3) = 2; para_mask(npara+3) = 1;     para_fix(npara+3) = para(npara+3);      %% rho_chi;

    if any(mspec == [112,114,12,14,6])
      para(npara+4) = 0;   para_mask(npara+4) = 1;     para_fix(npara+4) = para(npara+4);      %% rho_laf;
    elseif any(mspec == [101,102,103,104,105,106,107,108,109,110,111,113,115,116,1,2,3,4,5,6,7,8,9,10,11,13,15,16])
      para(npara+4) = .75; %para_mask(npara+4) = 1;    para_fix(npara+4) = para(npara+4);      %% rho_laf;
    end
                           para_type(npara+4) = 2;
    para(npara+5) = 0; para_type(npara+5) = 2; para_mask(npara+5) = 1;    para_fix(npara+5) = para(npara+5);      %% rho_mu;

    para(npara+6) = 0;     para_type(npara+6) = 2; para_mask(npara+6) = 1;     para_fix(npara+6) = para(npara+6);      %% rho_b;

    para(npara+7) = .9;    para_type(npara+7) = 2; %para_mask(npara+7) = 1;    para_fix(npara+7) = para(npara+7);      %% rho_g;

    npara = npara+7;    

    %% exogenous processes - standard deviation
    para(npara+1) = .4;    para_type(npara+1) = 2; %para_mask(npara+1) = 1;    para_fix(npara+1) = para(npara+1);      %% sig_z;
    para(npara+2) =  1;    para_type(npara+2) = 2; %para_mask(npara+2) = 1;    para_fix(npara+2) = para(npara+2);      %% sig_phi;
    para(npara+3) = 0;     para_type(npara+3) = 2; para_mask(npara+3) = 1;     para_fix(npara+3) = para(npara+3);      %% sig_chi;
    para(npara+4) = 1;     para_type(npara+4) = 2; %para_mask(npara+4) = 1;    para_fix(npara+4) = para(npara+4);      %% sig_laf;
    para(npara+5) = 0;     para_type(npara+5) = 2; para_mask(npara+5) = 1;    para_fix(npara+5) = para(npara+5);      %% sig_mu;
    para(npara+6) = 0;     para_type(npara+6) = 2; para_mask(npara+6) = 1;    para_fix(npara+6) = para(npara+6);  %% sig_b;
    para(npara+7) = .30;   para_type(npara+7) = 2; %para_mask(npara+7) = 1;        para_fix(npara+7) = para(npara+7);  %% sig_g;
    para(npara+8) = .10;   para_type(npara+8) = 2; %para_mask(npara+8) = 1;        para_fix(npara+8) = para(npara+8);  %% sig_r;

    npara = npara+8;
    
    para = para(1:npara);
    para_mask = para_mask(1:npara);
    para_fix = para_fix(1:npara);
    para_type = para_type(1:npara);
    
    para_type( find(para_mask) ) = inf;


    if isempty(mspec_ref)
      %% standard prior on auxiliary params
      para_type( find(para_type==2) ) = 1;
    end
    
    %% number of parameters in DSGE model   

    para = para.*(1-para_mask)+para_fix.*para_mask;


    %% identify policy and non policy parameters
    polipar = [16];
    polivalue = [2];


    %%%%%%%%%%%%%%%%%%%%%%%%%%
    %% bounds for MH

    bounds = zeros(100,2);
        
    bounds(1,:) = [1E-5 .99999];    %% alp;         1
    bounds(2,:) = [1E-5 .99999];    %% zeta_p;      2
    bounds(3,:) = [0 1];            %% iota_p;         3
    bounds(4,:) = [1E-5 .99999];    %% del;         3
    bounds(5,:) = [0    10];     %% ups;         
    bounds(6,:) = [0    5];     %% Bigphi;      
    bounds(7,:) = [0    20];        %% s2;          6
    bounds(8,:) = [1E-5 .99999];    %% h;           7
    bounds(9,:) = [0    .99999];    %% a2;          8
    bounds(10,:) = [1E-5 10];        %% nu_l         9
    bounds(11,:) = [1E-5    100];        %% nu_m         10
    bounds(12,:) = [1E-5    .99999];    %% zeta_w;      11
    bounds(13,:) = [0 1];            %% iota_w;         3
    bounds(14,:) = [1E-5    50];        %% law;         12
    bounds(15,:) = [1E-5    10];        %% beta;        
    bounds(16,:) = [1E-5    10];        %% psi1;        14
    bounds(17,:) = [1E-5    10];        %% psi2;        15
    bounds(18,:) = [1E-5    .99999];    %% rho_r;       16
    bounds(19,:) = [-1  10];        %% pistar;      17
    
    npara = 19;
    
    %% exogenous processes - level
    bounds(npara+1,:) = [1E-6   10];        %% gam;     18
    bounds(npara+2,:) = [0      10];        %% Lstar;   19
    bounds(npara+3,:) = [1E-6   10];        %% chi;     
    bounds(npara+4,:) = [1E-5   50];        %% laf;     21
    bounds(npara+5,:) = [1E-5   .99999];    %% gstar;   22
    bounds(npara+6,:) = [1E-6   5000];        %% Ladj     23
    
    npara = npara+6;    
    
    %% exogenous processes - autocorrelation
    bounds(npara+1,:) = [0      .99999];    %% rho_z;   24
    bounds(npara+2,:) = [1E-5   .99999];    %% rho_phi;
    bounds(npara+3,:) = [1E-5   .99999];    %% rho_chi;
    bounds(npara+4,:) = [1E-5   .99999];    %% rho_laf;
    bounds(npara+5,:) = [1E-5   .99999];    %% rho_mu;
    bounds(npara+6,:) = [1E-5   .99999];    %% rho_b;
    bounds(npara+7,:) = [1E-5   .99999];    %% rho_g;
    
    npara = npara+7;    
    
    %% exogenous processes - standard deviation
    bounds(npara+1,:) = [1E-7   100];       %% sig_z;
    bounds(npara+2,:) = [1E-7   100];       %% sig_phi;
    bounds(npara+3,:) = [1E-7   100];       %% sig_chi;
    bounds(npara+4,:) = [1E-7   100];     %% sig_laf;
    bounds(npara+5,:) = [1E-7   100];       %% sig_mu;
    bounds(npara+6,:) = [1E-7   100];       %% sig_b;
    bounds(npara+7,:) = [1E-7   100];       %% sig_g;
    bounds(npara+8,:) = [1E-7   100];       %% sig_r;
    
    npara = npara+8;
    bounds = bounds(1:npara,:);
    
        
end
