% This file contains the specification of the VAR model
function [mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,JustMode,stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix] = varspec(chdata);


%% choose model
mspec = 2;

%% choose prior
mprior = 1; %% 1 is standard

%% choose dataset 
if any(mspec == [1,2,4,5])
	dataset = 1;
elseif any(mspec == [3])
	dataset = 2;
end

%% choose date
dates = 2002.75;

%% Effective sample size to generate prior 
lambdaseq = 1 %[.75];        
%lambdaseq = [1 5];        

%% number of lags
nlags = 3

%% fixparamseq = 1 exogenous (fixed parameters) at para
fixparamseq = [0];

%% coint = 0 : just growth rates, otherwise number of cointegrating relationships
coint = 0;

%% if JustMode: do not do simulations and compute forecasts at mode
JustMode = 0;           

% quarters ahead for computing forecasts
qahead = 16;                                     

%%  Configure the Metropolis Algorithm (number of simulations)
nblocks = 11;
nsim    = 1000
%nblocks = 1;
%nsim    = 100;
nburn = nsim;                                       % initial simulations to 'burn'

%% number of observations used in VAR (st = [] means use entire sample) 
stime = 68; 
st = num2str(stime);


%% End of Presample
T0 = (20-1 - nlags) + nlags; %23+nlags

Tend     = 0;        % End of Estimation Sample 


if dataset == 1
	%% number of  variables, names of variables, and variables for which we want cumulative forecasts
	nvar = 8;
	varnames = {'Y';'C';'I';'H';'W';'inflation';'M2';'R'};
	cum_for = [  0   0   0   0   0      0         0   0];
elseif dataset == 2
	%% number of  variables, names of variables, and variables for which we want cumulative forecasts
	nvar = 3;
	varnames = {'Y';'inflation';'R'};
	cum_for = [  0    0        0];
end

   
if any(mspec == [1,2,4,5,3])


	%%%%%%%%%%%%%%%%%%%%%%%%%%
	%% names of shocks, and variables for which we want cumulative irf
	cum_irf = [1 2 3];
	if any(mspec == [1,2,4,5])
		names_shocks = {'Tech';'\phi';'\chi';'\mu';'b';'g';'\lambda_f';'Money'};    
	elseif any(mspec == [3])
		names_shocks = {'Tech';'g';'Money'};    
	end
	%%%%%%%%%%%%%%%%%%%%%%%%%%
	%% names of parameters 
	para_names = { '\alpha';'\zeta_p';'\delta';'\Upsilon';'\Phi';'s^\prime';'h';'a^{\prime~\prime}';'\nu';'\zeta_w';'\lambda_w';'r^*';...
		'\psi_1';'\psi_2';'\rho_{r}';'\pi^*';'\gamma';'L^*';'\chi';'\lambda_f';'g^*';'L^{adj}';...
		'\rho_{z}';'\rho_{\phi}';'\rho_{\chi}';'\rho_{\lambda_f}';'\rho_{\mu}';'\rho_{b}';'\rho_{g}';...
		'\sigma_{z}';'\sigma_{\phi}';'\sigma_{\chi}';'\sigma_{\lambda_f}';'\sigma_{\mu}';'\sigma_{b}';'\sigma_{g}';'\sigma_{r}'};

	%%%%%%%%%%%%%%%%%%%%%%%%%%
	%% starting values for parameters
	para = zeros(100,1);

	%% para_mask is npara x 1
	%%   to fix a parameter under specification mspec, set the corresponding element of para_mask to 1
	%% para_fix is npara x 1
	%%   and contains the values for the fixed parameters (all other entries are irrelevant)

	para_mask = zeros(100,1);
 	para_fix  = zeros(100,1);


	para(1) = .25;		para_mask(1) = 1;		para_fix(1) = para(1);				%% alp;			1
	if any(mspec == [1,2,3,5])
		para(2) = .75; 		%para_mask(2) = 1;	para_fix(2) = para(2);			 	%% zeta_p;		2
	elseif any(mspec == [4])
		para(2) = 0.1; 		para_mask(2) = 1;	para_fix(2) = para(2);			 	%% zeta_p;		2
	end
	para(3) = .025;		para_mask(3) = 1;		para_fix(3) = para(3);			 	%% del;			3
	para(4) = .1;		%para_mask(4) = 1;		para_fix(4) = para(4);				%% ups;
	para(5) = .45;		%para_mask(5) = 1;		para_fix(5) = para(5);				%% Bigphi;		5
	para(6) = 4;		%para_mask(6) = 1;		para_fix(6) = para(6);				%% s2; 
	para(7) = .7;		%para_mask(7) = 1;		para_fix(7) = para(7);				%% h;			7
	para(8) = .2;		%para_mask(8) = 1;		para_fix(8) = para(8);				%% a2;
	para(9) = 2;		%para_mask(9) = 1;		para_fix(9) = para(9);				%% nu;			9
	if any(mspec == [1,2,3,4])
		para(10) = .8;	%para_mask(10) = 1;		para_fix(10) = para(10);			%% zeta_w;		10
	elseif any(mspec == [5])
		para(10) = .1;	para_mask(10) = 1;		para_fix(10) = para(10);			%% zeta_w;		10
	end
	para(11) = .3;		para_mask(11) = 1;		para_fix(11) = para(11);			%% law;			11

	if mspec == 1
		para(12) = .9;		para_mask(12) = 1;		para_fix(12) = para(12);		%% rstar;		12
	elseif any(mspec == [2,4,5,3])
		para(12) = .9;		%para_mask(12) = 1;		para_fix(12) = para(12);		%% rstar;		12
	end;

	para(13) = 1.7;		%para_mask(13) = 1;		para_fix(13) = para(13);			%% psi1;		13
	para(14) = .125;	%para_mask(14) = 1;		para_fix(14) = para(14);			%% psi2;		14
	para(15) = .8;		%para_mask(15) = 1;		para_fix(15) = para(15);			%% rho_r;		15
	if mspec == 1
		para(16) = .65;		para_mask(16) = 1;		para_fix(16) = para(16);		%% pistar;		16
	elseif any(mspec == [2,4,5,3])
		para(16) = .65;		%para_mask(16) = 1;		para_fix(16) = para(16);		%% pistar;		16
	end

	npara = 16;

	%% exogenous processes - level
	para(npara+1) = .5;	%para_mask(npara+1) = 1;		para_fix(npara+1) = para(npara+1);		%% gam;		17
	para(npara+2) =  0;	para_mask(npara+2) = 1;			para_fix(npara+2) = para(npara+2);		%% Lstar;
	if any(mspec == [1,2,4,5])
		para(npara+3) = .1;	%para_mask(npara+3) = 1;		para_fix(npara+3) = para(npara+3);	%% chi;		19
	elseif any(mspec == [3])
		para(npara+3) = .1;	para_mask(npara+3) = 1;		para_fix(npara+3) = para(npara+3);		%% chi;		19
	end
	para(npara+4) = .3;	para_mask(npara+4) = 1;			para_fix(npara+4) = para(npara+4);		%% laf;		20
	para(npara+5) = .15;	para_mask(npara+5) = 1;			para_fix(npara+5) = para(npara+5);		%% gstar;
	if any(mspec == [1,2,4,5])
		para(npara+6) = 5;	%para_mask(npara+6) = 1;	para_fix(npara+6) = para(npara+6);		%% Ladj
	elseif any(mspec == [3])
		para(npara+6) = 0;	para_mask(npara+6) = 1;		para_fix(npara+6) = para(npara+6);		%% Ladj
	end

	npara = npara+6;	

	%% exogenous processes - autocorrelation
	para(npara+1) = .0;	 para_mask(npara+1) = 1;		para_fix(npara+1) = para(npara+1);		%% rho_z;
	if any(mspec == [1,2,4,5])
		para(npara+2) = .4;	%para_mask(npara+2) = 1;		para_fix(npara+2) = para(npara+2);	%% rho_phi;
	elseif any(mspec == [3])
		para(npara+2) = 0;	para_mask(npara+2) = 1;		para_fix(npara+2) = para(npara+2);		%% rho_phi;
	end
	if any(mspec == [1,2,4,5])
		para(npara+3) = .9;	%para_mask(npara+3) = 1;		para_fix(npara+3) = para(npara+3);	%% rho_chi;
	elseif any(mspec == [3])
		para(npara+3) = .9;	para_mask(npara+3) = 1;		para_fix(npara+3) = para(npara+3);		%% rho_chi;
	end
	para(npara+4) = 0;	para_mask(npara+4) = 1;			para_fix(npara+4) = para(npara+4);		%% rho_laf;
	if any(mspec == [1,2,4,5])
		para(npara+5) = .5;	%para_mask(npara+5) = 1;	para_fix(npara+5) = para(npara+5);		%% rho_mu;
	elseif any(mspec == [3])
		para(npara+5) = .5;	para_mask(npara+5) = 1;	para_fix(npara+5) = para(npara+5);			%% rho_mu;
	end
	para(npara+6) = 0;	para_mask(npara+6) = 1;			para_fix(npara+6) = para(npara+6);		%% rho_b;
	if mspec == 1
		para(npara+7) = .9;	%para_mask(npara+7) = 1;	para_fix(npara+7) = para(npara+7);		%% rho_g;
	elseif any(mspec == [2,4,5,3])
		para(npara+7) = .9;	para_mask(npara+7) = 1;		para_fix(npara+7) = para(npara+7);		%% rho_g;
	end
	npara = npara+7;	

	%% exogenous processes - standard deviation
	para(npara+1) = .4;	%para_mask(npara+1) = 1;		para_fix(npara+1) = para(npara+1);		%% sig_z;
	if any(mspec == [1,2,4,5])
		para(npara+2) =  1;	%para_mask(npara+2) = 1;	para_fix(npara+2) = para(npara+2);		%% sig_phi;
	elseif any(mspec == [3])
		para(npara+2) =  1;	para_mask(npara+2) = 1;	para_fix(npara+2) = para(npara+2);		%% sig_phi;
	end
	if any(mspec == [1,2,4,5])
		para(npara+3) = 1;	%para_mask(npara+3) = 1;	para_fix(npara+3) = para(npara+3);		%% sig_chi;
	elseif any(mspec == [3])
		para(npara+3) = 1;	para_mask(npara+3) = 1;		para_fix(npara+3) = para(npara+3);		%% sig_chi;
	end
	if any(mspec == [1,2,4,5])
		para(npara+4) = 1;	%para_mask(npara+4) = 1;	para_fix(npara+4) = para(npara+4);		%% sig_laf;
	elseif any(mspec == [3])
		para(npara+4) = 0;	para_mask(npara+4) = 1;		para_fix(npara+4) = para(npara+4);	%% sig_laf;
	end
	if any(mspec == [1,2,4,5])
		para(npara+5) = 1;	%para_mask(npara+5) = 1;		para_fix(npara+5) = para(npara+5);	%% sig_mu;
	elseif any(mspec == [3])
		para(npara+5) = 0;	para_mask(npara+5) = 1;		para_fix(npara+5) = para(npara+5);	%% sig_mu;
	end
	if any(mspec == [1,2,4,5])
		para(npara+6) = .20;	%para_mask(npara+6) = 1;	para_fix(npara+6) = para(npara+6);		%% sig_b;
	elseif any(mspec == [3])
		para(npara+6) = 0;	para_mask(npara+6) = 1;	para_fix(npara+6) = para(npara+6);		%% sig_b;
	end
	para(npara+7) = .30;	%para_mask(npara+7) = 1;		para_fix(npara+7) = para(npara+7);		%% sig_g;
	para(npara+8) = .10;	%para_mask(npara+8) = 1;		para_fix(npara+8) = para(npara+8);		%% sig_r;

	npara = npara+8;

	para = para(1:npara);
	para_mask = para_mask(1:npara);
	para_fix = para_fix(1:npara);
	%% number of parameters in DSGE model 	

	para = para.*(1-para_mask)+para_fix.*para_mask;


	%% identify policy and non policy parameters
	polipar = [13];
	polivalue = [2];


	%%%%%%%%%%%%%%%%%%%%%%%%%%
	%% bounds for MH

	bounds = zeros(100,2);
		
	bounds(1,:) = [1E-5	.99999]; 	%% alp;			1
	bounds(2,:) = [1E-5	.99999];   	%% zeta_p;		2
	bounds(3,:) = [1E-5	.99999]; 	%% del;			3
	bounds(4,:) = [1E-6	2];		%% ups;			
	bounds(5,:) = [0	2];		%% Bigphi;		
	bounds(6,:) = [0	20];		%% s2;			6
	bounds(7,:) = [1E-5	.99999]; 	%% h;			7
	bounds(8,:) = [0	.99999];	%% a2;			8
	bounds(9,:) = [1E-5	10];		%% nu;			9
	bounds(10,:) = [1E-5	.99999];	%% zeta_w;		10
	bounds(11,:) = [1E-5  	50];		%% law;			11
	bounds(12,:) = [1E-5	10];		%% rstar;		12
	bounds(13,:) = [1E-5	10];		%% psi1;		13
	bounds(14,:) = [1E-5	10];		%% psi2;		14
	bounds(15,:) = [1E-5	.99999];	%% rho_r;		15
	bounds(16,:) = [-1	10];		%% pistar;		16
	
	
	npara = 16;
	
	%% exogenous processes - level
	bounds(npara+1,:) = [1E-6	10];		%% gam;		17
	bounds(npara+2,:) = [0		10];		%% Lstar;	18
	bounds(npara+3,:) = [1E-6	10];		%% chi;		
	bounds(npara+4,:) = [1E-5  	50];		%% laf;		20
	bounds(npara+5,:) = [1E-5	.99999];	%% gstar;	21
	bounds(npara+6,:) = [1E-6	10];		%% Ladj		22
	
	npara = npara+6;	
	
	%% exogenous processes - autocorrelation
	bounds(npara+1,:) = [0		.99999];	%% rho_z;	23
	bounds(npara+2,:) = [1E-5	.99999];	%% rho_phi;
	bounds(npara+3,:) = [1E-5	.99999];	%% rho_chi;
	bounds(npara+4,:) = [1E-5	.99999];	%% rho_laf;
	bounds(npara+5,:) = [1E-5	.99999];	%% rho_mu;
	bounds(npara+6,:) = [1E-5	.99999];	%% rho_b;
	bounds(npara+7,:) = [1E-5	.99999];	%% rho_g;
	
	npara = npara+7;	
	
	%% exogenous processes - standard deviation
	bounds(npara+1,:) = [1E-7	100];		%% sig_z;
	bounds(npara+2,:) = [1E-7	100];		%% sig_phi;
	bounds(npara+3,:) = [1E-7	100];		%% sig_chi;
	bounds(npara+4,:) = [1E-7	100];		%% sig_laf;
	bounds(npara+5,:) = [1E-7	100];		%% sig_mu;
	bounds(npara+6,:) = [1E-7	100];		%% sig_b;
	bounds(npara+7,:) = [1E-7	100];		%% sig_g;
	bounds(npara+8,:) = [1E-7	100];		%% sig_r;
	
	npara = npara+8;
	bounds = bounds(1:npara,:);
	
		
end





