%% This file contains the specification of the VAR model
function [mspec,mprior,dataset,dates,nvar,nlags,T0,Tend,npara,lambdaseq,para_names,bounds,para,varnames,cum_for,polipar,polivalue,cum_irf,names_shocks,fixparamseq,coint,cointadd,cointall,cointvec,cointaddvec,JustMode,stime,st,nblocks,nsim,nburn,qahead,para_mask,para_fix] = varspecups(chdata);


%% choose model
mspec = 101;

%% choose prior
mprior = 1; %% 1 is standard

%% choose dataset 
dataset = [];
if any(mspec == [3])
    dataset = 2;
elseif any(mspec == [101,102,103,104,105])
    dataset = 3;
elseif any(mspec == [1])
    dataset = 1;
end

%% Effective sample size to generate prior 
lambdaseq = [NaN] %[0 .5 .75 1 1.25 1.5 2 5 Inf NaN];

%% coint = 0 : just growth rates, otherwise number of cointegrating relationships
%% model 1: coint = 4
%% model 2: coint = 0
%% model 101,102,103,104,105: coint = 3
coint = 3;
cointadd = 0;
cointall = coint + cointadd;

%% number of lags
nlags = 4

if strcmp(chdata,'euro')

    %% choose date
    dates = 2002.75;
    
    %% number of observations used in VAR (st = [] means use entire sample) 
    stime = 68; 
    st = num2str(stime);
    
    %% End of Presample
    T0 = (20-1 - nlags) + nlags; %23+nlags
    
    Tend     = 0;        % End of Estimation Sample 

elseif strcmp(chdata,'us')

    %% choose date
    dates = 2004.00;
   
    %% number of observations used in VAR (st = [] means use entire sample) 
    if any(mspec == [1,101,102,103,104,105])
        stime = 120; 
    elseif any(mspec == [3])
        stime = 80; 
    end
%stime = []
    st = num2str(stime);
    
    %% End of Presample
    T0 = nlags; 
    
    Tend     = 0;        % End of Estimation Sample 
    
end


%% fixparamseq = 1 exogenous (fixed parameters) at para
fixparamseq = [0];

%% if JustMode: do not do simulations and compute forecasts at mode
JustMode = 0;           

% quarters ahead for computing forecasts
qahead = 16;                                     

%%  Configure the Metropolis Algorithm (number of simulations)
nblocks = 11
nsim    = 10000
nburn = nsim;                                       % initial simulations to 'burn'

if any( dataset == [1,3,4] )
    %% number of  variables, names of variables, and variables for which we want cumulative forecasts
    if any(mspec == [1])
        nvar = 8;
        varnames = {'Y';'C';'I';'H';'W';'inflation';'M2';'R'};
        cum_for = [  1   1   1   0   1      1         1   0];
    elseif any(mspec == [101,102,103,104,105])
        nvar = 7;
        varnames = {'Y';'C';'I';'H';'W';'inflation';'R'};
        cum_for = [  1   1   1   0   1      1        0];

	if coint 
		cointvec = zeros(coint,nvar);
		%% C - Y
		cointvec(1,1) = -1;
		cointvec(1,2) = 1;
		%% I - Y
		cointvec(2,1) = -1;
		cointvec(2,3) = 1;
		%% W - Y
		cointvec(3,1) = -1;
		cointvec(3,5) = 1;
	else
		cointvec = [];
	end
        cointaddvec = zeros(cointadd,nvar);
        if cointadd > 0
           %% Y
           cointaddvec(1,1) = 1;
	else
	   cointaddvec = [];	
        end;

    end
elseif dataset == 2
    %% number of  variables, names of variables, and variables for which we want cumulative forecasts
    nvar = 3;
    varnames = {'Y';'inflation';'R'};
    cum_for = [  0    0        0];
end

   
if any(mspec == [1,3,101,102,103,104,105])


    %%%%%%%%%%%%%%%%%%%%%%%%%%
    %% names of shocks, and variables for which we want cumulative irf
    cum_irf = [1 2 3 5];

    if any(mspec == [1])
        names_shocks = {'Tech';'\phi';'\chi';'\mu';'b';'g';'\lambda_f';'Money'};    
    elseif any(mspec == [3])
        names_shocks = {'Tech';'g';'Money'};    
        cum_irf = [1 ];
    elseif any(mspec == [101,102,103,104,105])
        names_shocks = {'Tech';'\phi';'\mu';'b';'g';'\lambda_f';'Money'};    
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    %% names of parameters 
    para_names = { '\alpha';'\zeta_p';'\iota_p';'\delta';'\Upsilon';'\Phi';'s^\prime';'h';'a^{\prime~\prime}';'\nu_l';'\nu_m';'\zeta_w';...
        '\iota_w';'\lambda_w';'r^*';...
        '\psi_1';'\psi_2';'\rho_{r}';'\pi^*';'\gamma';'W^{adj}';'\chi';'\lambda_f';'g^*';'L^{adj}';...
        '\rho_{z}';'\rho_{\phi}';'\rho_{\chi}';'\rho_{\lambda_f}';'\rho_{\mu}';'\rho_{b}';'\rho_{g}';...
        '\sigma_{z}';'\sigma_{\phi}';'\sigma_{\chi}';'\sigma_{\lambda_f}';'\sigma_{\mu}';'\sigma_{b}';'\sigma_{g}';'\sigma_{r}'};

    %%%%%%%%%%%%%%%%%%%%%%%%%%
    %% starting values for parameters
    para = zeros(100,1);

    %% para_mask is npara x 1
    %%   to fix a parameter under specification mspec, set the corresponding element of para_mask to 1
    %% para_fix is npara x 1
    %%   and contains the values for the fixed parameters (all other entries are irrelevant)

    para_mask = zeros(100,1);
    para_fix  = zeros(100,1);


    if any(mspec == [3])
        para(1) = .25;      para_mask(1) = 1;   para_fix(1) = para(1);              %% alp;         1
    elseif any(mspec == [1,101,102,103,104,105])
        para(1) = .25;      %para_mask(1) = 1;  para_fix(1) = para(1);              %% alp;         1
    end
    if any(mspec == [1,101,102,103,104,105,3])
        para(2) = .75;      %para_mask(2) = 1;  para_fix(2) = para(2);              %% zeta_p;      2
    elseif any(mspec == [])
        para(2) = 0.1;      para_mask(2) = 1;   para_fix(2) = para(2);              %% zeta_p;      
    end
    if any(mspec == [1,101,103,104,105,3])
        para(3) = 0;        para_mask(3) = 1;   para_fix(3) = para(3);              %% iota_p;      3
    elseif any(mspec == [102])
        para(3) = 1;        para_mask(3) = 1;   para_fix(3) = para(3);              %% iota_p;      3
    end

    para(4) = .025;         para_mask(4) = 1;   para_fix(4) = para(4);              %% del;         4
    para(5) = .1;            %para_mask(5) = 1;   para_fix(5) = para(5);              %% ups; .1      5
    para(6) = 0;            para_mask(6) = 1;   para_fix(6) = para(6);              %% Bigphi;      6
    para(7) = 4;            %para_mask(7) = 1;  para_fix(7) = para(7);              %% s2;      7
    para(8) = .7;           %para_mask(8) = 1;  para_fix(8) = para(8);              %% h;           8
    para(9) = .2;           %para_mask(9) = 1;  para_fix(9) = para(9);              %% a2;          9
    para(10) = 2;           %para_mask(10) = 1;  para_fix(10) = para(10);           %% nu_l         10

    if any(mspec == [1])
        para(11) = 2;   %para_mask(11) = 1; para_fix(11) = para(11);            %% nu_m         11
    elseif any(mspec == [101,102,103,104,105,3])
        para(11) = 2;   para_mask(11) = 1;  para_fix(11) = para(11);            %% nu_m         11
    end
    if any(mspec == [1,101,102,103,104,105,3])
        para(12) = .8;      %para_mask(12) = 1; para_fix(12) = para(12);            %% zeta_w;      12
    elseif any(mspec == [])
        para(12) = .8;      para_mask(12) = 1;  para_fix(12) = para(12);            %% zeta_w;      
    end
    if any(mspec == [1,101,103,104,105,3])
        para(13) = 0;       para_mask(13) = 1;   para_fix(13) = para(13);          %% iota_w;      13
    elseif any(mspec == [102])
        para(13) = 1;       para_mask(13) = 1;   para_fix(13) = para(13);          %% iota_w;      13
    end

    para(14) = .3;          para_mask(14) = 1;  para_fix(14) = para(14);            %% law;         14

    if any(mspec == [1,101,102,103,104,105,3])
        para(15) = .5;  %para_mask(15) = 1; para_fix(15) = para(15);            %% beta;        15
    elseif any(mspec == [])
        para(15) = .5;  para_mask(15) = 1;  para_fix(15) = para(15);            %% beta;        15
    end

    para(16) = 1.7;         %para_mask(16) = 1; para_fix(16) = para(16);            %% psi1;        16

    para(17) = .125;        %para_mask(17) = 1; para_fix(17) = para(17);            %% psi2;        17

    para(18) = .8;          %para_mask(18) = 1; para_fix(18) = para(18);            %% rho_r;       18

    para(19) = .65;         %para_mask(19) = 1; para_fix(19) = para(19);            %% pistar;      19
   

    npara = 19;

    %% exogenous processes - level
    para(npara+1) = .5;     %para_mask(npara+1) = 1;    para_fix(npara+1) = para(npara+1);      %% gam;     
    para(npara+2) =  5.5;   %para_mask(npara+2) = 1;    para_fix(npara+2) = para(npara+2);      %% Wadj;

    para(npara+3) = .1;      para_mask(npara+3) = 1;    para_fix(npara+3) = para(npara+3);      %% chi;     

    para(npara+4) = .3;      para_mask(npara+4) = 1;     para_fix(npara+4) = para(npara+4);      %% laf;     
    para(npara+5) = .15;    %para_mask(npara+5) = 1;     para_fix(npara+5) = para(npara+5);      %% gstar;
    if any(mspec == [1,101,102,103,104,105,2])
        para(npara+6) = 5;  %para_mask(npara+6) = 1;    para_fix(npara+6) = para(npara+6);      %% Ladj
    elseif any(mspec == [3])
        para(npara+6) = 0;  para_mask(npara+6) = 1;     para_fix(npara+6) = para(npara+6);      %% Ladj
    end

    npara = npara+6;    

    %% exogenous processes - autocorrelation
    if any(mspec == [1,101,102,103,104,105,3])
        para(npara+1) = .2;  %para_mask(npara+1) = 1;    para_fix(npara+1) = para(npara+1);      %% rho_z;
    elseif any(mspec == [])
        para(npara+1) = .5;  para_mask(npara+1) = 1;    para_fix(npara+1) = para(npara+1);      %% rho_z;
    end
    if any(mspec == [1,101,102,103,104,105])
        para(npara+2) = .4; %para_mask(npara+2) = 1;    para_fix(npara+2) = para(npara+2);      %% rho_phi;
    elseif any(mspec == [3])
        para(npara+2) = 0;  para_mask(npara+2) = 1;     para_fix(npara+2) = para(npara+2);      %% rho_phi;
    end
    if any(mspec == [1])
        para(npara+3) = .9; %para_mask(npara+3) = 1;    para_fix(npara+3) = para(npara+3);      %% rho_chi;
    elseif any(mspec == [3,101,102,103,104,105])
        para(npara+3) = .9; para_mask(npara+3) = 1;     para_fix(npara+3) = para(npara+3);      %% rho_chi;
    end
    if any(mspec == [3])
        para(npara+4) = 0;  para_mask(npara+4) = 1;     para_fix(npara+4) = para(npara+4);      %% rho_laf;
    elseif any(mspec == [1,101,102,103,104,105])
        para(npara+4) = .9; %para_mask(npara+4) = 1;    para_fix(npara+4) = para(npara+4);      %% rho_laf;
    end
    if any(mspec == [1,101,102,103,104,105])
        para(npara+5) = .5; %para_mask(npara+5) = 1;    para_fix(npara+5) = para(npara+5);      %% rho_mu;
    elseif any(mspec == [3])
        para(npara+5) = .5; para_mask(npara+5) = 1;     para_fix(npara+5) = para(npara+5);      %% rho_mu;
    end
    if any(mspec == [1,101,102,103,104,105])
        para(npara+6) = .8;     %para_mask(npara+6) = 1;     para_fix(npara+6) = para(npara+6);      %% rho_b;
    elseif any(mspec == [3])
        para(npara+6) = 0;     para_mask(npara+6) = 1;     para_fix(npara+6) = para(npara+6);      %% rho_b;
    end
    if any(mspec == [1,101,102,103,104,105,3])
        para(npara+7) = .9; %para_mask(npara+7) = 1;    para_fix(npara+7) = para(npara+7);      %% rho_g;
    elseif any(mspec == [])
        para(npara+7) = 0; para_mask(npara+7) = 1;     para_fix(npara+7) = para(npara+7);      %% rho_g;
    end
    npara = npara+7;    

    %% exogenous processes - standard deviation
    para(npara+1) = .4;     %para_mask(npara+1) = 1;    para_fix(npara+1) = para(npara+1);      %% sig_z;
    if any(mspec == [1,101,102,103,104,105])
        para(npara+2) =  1; %para_mask(npara+2) = 1;    para_fix(npara+2) = para(npara+2);      %% sig_phi;
    elseif any(mspec == [3])
        para(npara+2) =  1; para_mask(npara+2) = 1;     para_fix(npara+2) = para(npara+2);      %% sig_phi;
    end
    if any(mspec == [1])
        para(npara+3) = 1;  %para_mask(npara+3) = 1;    para_fix(npara+3) = para(npara+3);      %% sig_chi;
    elseif any(mspec == [3,101,102,103,104,105])
        para(npara+3) = 0;  para_mask(npara+3) = 1;     para_fix(npara+3) = para(npara+3);      %% sig_chi;
    end
    if any(mspec == [1,101,102,103,104,105])
        para(npara+4) = 1;  %para_mask(npara+4) = 1;    para_fix(npara+4) = para(npara+4);      %% sig_laf;
    elseif any(mspec == [3])
        para(npara+4) = 0;  para_mask(npara+4) = 1;     para_fix(npara+4) = para(npara+4);      %% sig_laf;
    end
    if any(mspec == [1,101,102,103,104,105])
        para(npara+5) = 1;  %para_mask(npara+5) = 1;    para_fix(npara+5) = para(npara+5);      %% sig_mu;
    elseif any(mspec == [3])
        para(npara+5) = 0;  para_mask(npara+5) = 1;     para_fix(npara+5) = para(npara+5);      %% sig_mu;
    end
    if any(mspec == [1,101,102,103,104,105])
        para(npara+6) = .20;    %para_mask(npara+6) = 1;    para_fix(npara+6) = para(npara+6);  %% sig_b;
    elseif any(mspec == [3])
        para(npara+6) = 0;  para_mask(npara+6) = 1;     para_fix(npara+6) = para(npara+6);      %% sig_b;
    end
    para(npara+7) = .30;    %para_mask(npara+7) = 1;        para_fix(npara+7) = para(npara+7);  %% sig_g;
    para(npara+8) = .10;    %para_mask(npara+8) = 1;        para_fix(npara+8) = para(npara+8);  %% sig_r;

    npara = npara+8;

    para = para(1:npara);
    para_mask = para_mask(1:npara);
    para_fix = para_fix(1:npara);
    %% number of parameters in DSGE model   

    para = para.*(1-para_mask)+para_fix.*para_mask;


    %% identify policy and non policy parameters
    polipar = [16];
    polivalue = [2];


    %%%%%%%%%%%%%%%%%%%%%%%%%%
    %% bounds for MH

    bounds = zeros(100,2);
        
    bounds(1,:) = [1E-5 .99999];    %% alp;         1
    bounds(2,:) = [1E-5 .99999];    %% zeta_p;      2
    bounds(3,:) = [0 1];            %% iota_p;         3
    bounds(4,:) = [1E-5 .99999];    %% del;         3
    bounds(5,:) = [0    2];     %% ups;         
    bounds(6,:) = [0    2];     %% Bigphi;      
    bounds(7,:) = [0    20];        %% s2;          6
    bounds(8,:) = [1E-5 .99999];    %% h;           7
    bounds(9,:) = [0    .99999];    %% a2;          8
    bounds(10,:) = [1E-5 10];        %% nu_l         9
    bounds(11,:) = [1E-5    100];        %% nu_m         10
    bounds(12,:) = [1E-5    .99999];    %% zeta_w;      11
    bounds(13,:) = [0 1];            %% iota_w;         3
    bounds(14,:) = [1E-5    50];        %% law;         12
    bounds(15,:) = [1E-5    10];        %% beta;        13
    bounds(16,:) = [1E-5    10];        %% psi1;        14
    bounds(17,:) = [1E-5    10];        %% psi2;        15
    bounds(18,:) = [1E-5    .99999];    %% rho_r;       16
    bounds(19,:) = [-1  10];        %% pistar;      17
    
    npara = 19;
    
    %% exogenous processes - level
    bounds(npara+1,:) = [1E-6   10];        %% gam;     18
    bounds(npara+2,:) = [0      10];        %% Lstar;   19
    bounds(npara+3,:) = [1E-6   10];        %% chi;     
    bounds(npara+4,:) = [1E-5   50];        %% laf;     21
    bounds(npara+5,:) = [1E-5   .99999];    %% gstar;   22
    bounds(npara+6,:) = [1E-6   10];        %% Ladj     23
    
    npara = npara+6;    
    
    %% exogenous processes - autocorrelation
    bounds(npara+1,:) = [0      .99999];    %% rho_z;   24
    bounds(npara+2,:) = [1E-5   .99999];    %% rho_phi;
    bounds(npara+3,:) = [1E-5   .99999];    %% rho_chi;
    bounds(npara+4,:) = [1E-5   .99999];    %% rho_laf;
    bounds(npara+5,:) = [1E-5   .99999];    %% rho_mu;
    bounds(npara+6,:) = [1E-5   .99999];    %% rho_b;
    bounds(npara+7,:) = [1E-5   .99999];    %% rho_g;
    
    npara = npara+7;    
    
    %% exogenous processes - standard deviation
    bounds(npara+1,:) = [1E-7   100];       %% sig_z;
    bounds(npara+2,:) = [1E-7   100];       %% sig_phi;
    bounds(npara+3,:) = [1E-7   100];       %% sig_chi;
    bounds(npara+4,:) = [1E-7   10000];     %% sig_laf;
    bounds(npara+5,:) = [1E-7   100];       %% sig_mu;
    bounds(npara+6,:) = [1E-7   100];       %% sig_b;
    bounds(npara+7,:) = [1E-7   100];       %% sig_g;
    bounds(npara+8,:) = [1E-7   100];       %% sig_r;
    
    npara = npara+8;
    bounds = bounds(1:npara,:);
    
        
end
